/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME.....MQS1                                            */
/* FUNCTION TYPE.....TPF                                             */
/* SEGMENT NAME....  QMQZ04                                          */
/*                                                                   */
/* DESCRIPTION:                                                      */
/*   The MQRC Lookup Table stores MQRC errors corresponding to the   */
/*   MQRC error numbers.  Input is the MQRC Error number.  Output    */
/*   is a string.                                                    */
/*   Error return is NULL.                                           */
/*********************************************************************/

#include <qmqz.h>

//static const sMQRC_Lookup_Table MQRC_Lookup_Table[] =
const static char* MQRC_Lookup_Table[] = {
  ""                          , // 2000
  "ALIAS BASE Q TYPE ERROR"   , // 2001
  "ALREADY CONNECTED"         , // 2002
  "BACKED OUT"                , // 2003
  "BUFFER ERROR"              , // 2004
  "BUFFER LENGTH ERROR"       , // 2005
  "CHAR ATTR LENGTH ERROR"    , // 2006
  "CHAR ATTRS ERROR"          , // 2007
  "CHAR ATTRS TOO SHORT"      , // 2008
  "CONNECTION BROKEN"         , // 2009
  "DATA LENGTH ERROR"         , // 2010
  "DYNAMIC Q NAME ERROR"      , // 2011
  "ENVIRONMENT ERROR"         , // 2012
  "EXPIRY ERROR"              , // 2013
  "FEEDBACK ERROR"            , // 2014
  ""                          , // 2015
  "GET INHIBITED"             , // 2016
  "HANDLE NOT AVAILABLE"      , // 2017
  "HCONN ERROR"               , // 2018
  "HOBJ ERROR"                , // 2019
  "INHIBIT VALUE ERROR"       , // 2020
  "INT ATTR COUNT ERROR"      , // 2021
  "INT ATTR COUNT TOO SMALL"  , // 2022
  "INT ATTRS ARRAY ERROR"     , // 2023
  "SYNCPOINT LIMIT REACHED"   , // 2024
  "MAX CONNS LIMIT REACHED"   , // 2025
  "MD ERROR"                  , // 2026
  "MISSING REPLY TO Q"        , // 2027
  ""                          , // 2026
  "MSG TYPE ERROR"            , // 2029
  "MSG TOO BIG FOR Q"         , // 2030
  "MSG TOO BIG FOR Q MGR"     , // 2031
  ""                          , // 2030
  "NO MSG AVAILABLE"          , // 2033
  "NO MSG UNDER CURSOR"       , // 2034
  "NOT AUTHORIZED"            , // 2035
  "NOT OPEN FOR BROWSE"       , // 2036
  "NOT OPEN FOR INPUT"        , // 2037
  "NOT OPEN FOR INQUIRE"      , // 2038
  "NOT OPEN FOR OUTPUT"       , // 2039
  "NOT OPEN FOR SET"          , // 2040
  "OBJECT CHANGED"            , // 2041
  "OBJECT IN USE"             , // 2042
  "OBJECT TYPE ERROR"         , // 2043
  "OD ERROR"                  , // 2044
  "OPTION NOT VALID FOR TYPE" , // 2045
  "OPTIONS ERROR"             , // 2046
  "PERSISTENCE ERROR"         , // 2047
  "PERSISTENT NOT ALLOWED"    , // 2048
  "PRIORITY EXCEEDS MAXIMUM"  , // 2049
  "PRIORITY ERROR"            , // 2050
  "PUT INHIBITED"             , // 2051
  "Q DELETED"                 , // 2052
  "Q FULL"                    , // 2053
  ""                          , // 2054
  "Q NOT EMPTY"               , // 2055
  "Q SPACE NOT AVAILABLE"     , // 2056
  "Q TYPE ERROR"              , // 2057
  "Q MGR NAME ERROR"          , // 2058
  "Q MGR NOT AVAILABLE"       , // 2059
  ""                          , // 2060
  "REPORT OPTIONS ERROR"      , // 2061
  "SECOND MARK NOT ALLOWED"   , // 2062
  "SECURITY ERROR"            , // 2063
  ""                          , // 2064
  "SELECTOR COUNT ERROR"      , // 2065
  "SELECTOR LIMIT EXCEEDED"   , // 2066
  "SELECTOR ERROR"            , // 2067
  "SELECTOR NOT FOR TYPE"     , // 2068
  "SIGNAL OUTSTANDING"        , // 2069
  "SIGNAL REQUEST ACCEPTED"   , // 2070
  "STORAGE NOT AVAILABLE"     , // 2071
  "SYNCPOINT NOT AVAILABLE"   , // 2072
  ""                          , // 2073
  ""                          , // 2074
  "TRIGGER CONTROL ERROR"     , // 2075
  "TRIGGER DEPTH ERROR"       , // 2076
  "TRIGGER MSG PRIORITY ERR"  , // 2077
  "TRIGGER TYPE ERROR"        , // 2078
  "TRUNCATED MSG ACCEPTED"    , // 2079
  "TRUNCATED MSG FAILED"      , // 2080
  ""                          , // 2081
  "UNKNOWN ALIAS BASE Q"      , // 2082
  ""                          , // 2083
  ""                          , // 2084
  "UNKNOWN OBJECT NAME"       , // 2085
  "UNKNOWN OBJECT Q MGR"      , // 2086
  "UNKNOWN REMOTE Q MGR"      , // 2087
  "FAST CHANNEL NOT AVAIL"    , // 2088
  ""                          , // 2089
  "WAIT INTERVAL ERROR"       , // 2090
  "XMIT Q TYPE ERROR"         , // 2091
  "XMIT Q USAGE ERROR"        , // 2092
  "NOT OPEN FOR PASS ALL"     , // 2093
  "NOT OPEN FOR PASS IDENT"   , // 2094
  "NOT OPEN FOR SET ALL"      , // 2095
  "NOT OPEN FOR SET IDENT"    , // 2096
  "CONTEXT HANDLE ERROR"      , // 2097
  "CONTEXT NOT AVAILABLE"     , // 2098
  "SIGNAL1 ERROR"             , // 2099
  "OBJECT ALREADY EXISTS"     , // 2100
  "OBJECT DAMAGED"            , // 2101
  "RESOURCE PROBLEM"          , // 2102
  "ANOTHER Q MGR CONNECTED"   , // 2103
  "UNKNOWN REPORT OPTION"     , // 2104
  "STORAGE CLASS ERROR"       , // 2105
  "COD NOT VALID FOR XCF Q"   , // 2106
  "XWAIT CANCELED"            , // 2107
  "XWAIT ERROR"               , // 2108
  "SUPPRESSED BY EXIT"        , // 2109
  "FORMAT ERROR"              , // 2110
  "SOURCE CCSID ERROR"        , // 2111
  "SOURCE INTEGER ENC ERROR"  , // 2112
  "SOURCE DECIMAL ENC ERROR"  , // 2113
  "SOURCE FLOAT ENC ERROR"    , // 2114
  "TARGET CCSID ERROR"        , // 2115
  "TARGET INTEGER ENC ERROR"  , // 2116
  "TARGET DECIMAL ENC ERROR"  , // 2117
  "TARGET FLOAT ENC ERROR"    , // 2118
  "NOT CONVERTED"             , // 2119
  "CONVERTED MSG TOO BIG"     , // 2120
  "NO EXTERNAL PARTICIPANTS"  , // 2121
  "PARTICIPANT NOT AVAILABLE" , // 2122
  "OUTCOME MIXED"             , // 2123
  "OUTCOME PENDING"           , // 2124
  "BRIDGE STARTED"            , // 2125
  "BRIDGE STOPPED"            , // 2126
  "ADAPTER STORAGE SHORTAG"   , // 2127
  "UOW IN PROGRESS"           , // 2128
  "ADAPTER CONN LOAD ERROR"   , // 2129
  "ADAPTER SERV LOAD ERROR"   , // 2130
  "ADAPTER DEFS ERROR"        , // 2131
  "ADAPTER DEFS LOAD ERROR"   , // 2132
  "ADAPTER CONV LOAD ERROR"   , // 2133
  "BO ERROR"                  , // 2134
  "DH ERROR"                  , // 2135
  "MULTIPLE REASONS"          , // 2136
  "OPEN FAILED"               , // 2137
  "ADAPTER DISC LOAD ERROR"   , // 2138
  "CNO ERROR"                 , // 2139
  "CICS WAIT FAILED"          , // 2140
  "DLH ERROR"                 , // 2141
  "HEADER ERROR"              , // 2142
  "SOURCE LENGTH ERROR"       , // 2143
  "TARGET LENGTH ERROR"       , // 2144
  "SOURCE BUFFER ERROR"       , // 2145
  "TARGET BUFFER ERROR"       , // 2146
  ""                          , // 2146
  "IIH ERROR"                 , // 2148
  "PCF ERROR"                 , // 2149
  "DBCS ERROR"                , // 2150
  "TRUNCATED"                 , // 2151
  "OBJECT NAME ERROR"         , // 2152
  "OBJECT Q MGR NAME ERROR"   , // 2153
  "RECS PRESENT ERROR"        , // 2154
  "OBJECT RECORDS ERROR"      , // 2155
  "RESPONSE RECORDS ERROR"    , // 2156
  "ASID MISMATCH"             , // 2157
  "PMO RECORD FLAGS ERROR"    , // 2158
  "PUT MSG RECORDS ERROR"     , // 2159
  "CONN ID IN USE"            , // 2160
  "Q MGR QUIESCING"           , // 2161
  "Q MGR STOPPING"            , // 2162
  "DUPLICATE RECOV COORD"     , // 2163
  ""                          , // 2164
  ""                          , // 2165
  ""                          , // 2166
  ""                          , // 2167
  ""                          , // 2168
  ""                          , // 2169
  ""                          , // 2170
  ""                          , // 2171
  ""                          , // 2172
  "PMO ERROR"                 , // 2173
  ""                          , // 2174
  ""                          , // 2175
  ""                          , // 2176
  ""                          , // 2177
  ""                          , // 2178
  ""                          , // 2179
  ""                          , // 2180
  ""                          , // 2181
  "API EXIT NOT FOUND"        , // 2182
  "API EXIT LOAD ERROR"       , // 2183
  "REMOTE Q NAME ERROR"       , // 2184
  "INCONSISTENT PERSISTENCE"  , // 2185
  "GMO ERROR"                 , // 2186
  ""                          , // 2187
  ""                          , // 2188
  ""                          , // 2189
  ""                          , // 2190
  "TMC ERROR"                 , // 2191
  "PAGESET FULL"              , // 2192
  "PAGESET ERROR"             , // 2193
  "NAME NOT VALID FOR TYPE"   , // 2194
  "UNEXPECTED ERROR"          , // 2195
  "UNKNOWN XMIT Q"            , // 2196
  "UNKNOWN DEF XMIT Q"        , // 2197
  "DEF XMIT Q TYPE ERROR"     , // 2198
  "DEF XMIT Q USAGE ERROR"    , // 2199
  ""                          , // 2200
  "NAME IN USE"               , // 2201
  "CONNECTION QUIESCING"      , // 2202
  "CONNECTION STOPPING"       , // 2203
  "ADAPTER NOT AVAILABLE"     , // 2204
  ""                          , // 2205
  "MSG ID ERROR"              , // 2206
  "CORREL ID ERROR"           , // 2207
  "FILE SYSTEM ERROR"         , // 2208
  "NO MSG LOCKED"             , // 2209
  ""                          , // 2210
  ""                          , // 2211
  ""                          , // 2212
  ""                          , // 2213
  ""                          , // 2214
  ""                          , // 2215
  "FILE NOT AUDITED"          , // 2216
  "CONNECTION NOT AUTHORIZED" , // 2217
  "MSG TOO BIG FOR CHANNEL"   , // 2218
  "CALL IN PROGRESS"          , // 2219
  "RMH ERROR"                 , // 2220
  ""                          , // 2221
  "Q MGR ACTIVE"              , // 2222
  "Q MGR NOT ACTIVE"          , // 2223
  "Q DEPTH HIGH"              , // 2224
  "Q DEPTH LOW"               , // 2225
  "Q SERVICE INTERVAL HIGH"   , // 2226
  "Q SERVICE INTERVAL OK"     , // 2227
  ""                          , // 2228
  ""                          , // 2229
  ""                          , // 2230
  ""                          , // 2231
  ""                          , // 2232
  "CHANNEL AUTO DEF OK"       , // 2233
  "CHANNEL AUTO DEF ERROR"    , // 2234
  "CFH ERROR"                 , // 2235
  "CFIL ERROR"                , // 2236
  "CFIN ERROR"                , // 2237
  "CFSL ERROR"                , // 2238
  "CFST ERROR"                , // 2239
  ""                          , // 2240
  "INCOMPLETE GROUP"          , // 2241
  "INCOMPLETE MSG"            , // 2242
  "INCONSISTENT CCSIDS"       , // 2243
  "INCONSISTENT ENCODINGS"    , // 2244
  "INCONSISTENT UOW"          , // 2245
  "INVALID MSG UNDER CURSOR"  , // 2246
  "MATCH OPTIONS ERROR"       , // 2247
  "MDE ERROR"                 , // 2248
  "MSG FLAGS ERROR"           , // 2249
  "MSG SEQ NUMBER ERROR"      , // 2250
  "OFFSET ERROR"              , // 2251
  "ORIGINAL LENGTH ERROR"     , // 2252
  "SEGMENT LENGTH ZERO"       , // 2253
  ""                          , // 2254
  "UOW NOT AVAILABLE"         , // 2255
  "WRONG GMO VERSION"         , // 2256
  "WRONG MD VERSION"          , // 2257
  "GROUP ID ERROR"            , // 2258
  "INCONSISTENT BROWSE"       , // 2259
  "XQH ERROR"                 , // 2260
  "SRC ENV ERROR"             , // 2261
  "SRC NAME ERROR"            , // 2262
  "DEST ENV ERROR"            , // 2263
  "DEST NAME ERROR"           , // 2264
  "TM ERROR"                  , // 2265
  ""                          , // 2266
  ""                          , // 2267
  ""                          , // 2268
  ""                          , // 2269
  ""                          , // 2270
  ""                          , // 2271
  ""                          , // 2272
  ""                          , // 2273
  ""                          , // 2274
  ""                          , // 2275
  ""                          , // 2276
  ""                          , // 2277
  ""                          , // 2278
  ""                          , // 2279
  "HCONFIG ERROR"             , // 2280
  "FUNCTION ERROR"            , // 2281
  "CHANNEL STARTED"           , // 2282
  "CHANNEL STOPPED"           , // 2283
  "CHANNEL CONV ERROR"        , // 2284
  "SERVICE NOT AVAILABLE"     , // 2285
  "INITIALIZATION FAILED"     , // 2286
  "TERMINATION FAILED"        , // 2287
  "UNKNOWN Q NAME"            , // 2288
  "SERVICE ERROR"             , // 2289
  "Q ALREADY EXISTS"          , // 2290
  "USER ID NOT AVAILABLE"     , // 2291
  "UNKNOWN ENTITY"            , // 2292
  "UNKNOWN AUTH ENTITY"       , // 2293
  "UNKNOWN REF OBJECT"        , // 2294
  "CHANNEL ACTIVATED"         , // 2295
  "CHANNEL NOT ACTIVATED"     };// 2296


char *MQRC_String(int Reason) {                                      // Finds string corresponding to MQRC_Reason
  if (Reason == 0)
     return "NO ERROR";
  else
  if ((Reason < 2000) || (Reason > 2296)) return "";                 // Value not in table
  else return (char *)MQRC_Lookup_Table[Reason-2000]; }              // Get string

