/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2006, 2020                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*  d26214  01/03/19  Add option for version to fix driver with mq   */
/*                    client (ZMQID)                                 */
/*  d26533  11/06/19  z/TPF WebSphere MQ support for dynamic routing */
/*  26653   10/28/20  Add option on LONGRUNNING and MQGET commands to*/
/*                    validate files.                                */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME.....QMQZ07                                          */
/* FUNCTION TYPE.....TPF                                             */
/*                                                                   */
/* SEGMENT NAME......qmqz07.cpp                                      */
/*                                                                   */
/* DESCRIPTION: part of SO  QMQZ                                     */
/*                                                                   */
/*********************************************************************/

#include <qmqz.h>
#include <fstream>
#include <zlib.h>

//#include <tpftime.h>

#define MSG_SEQ_PFX "QMQZ.Message."

#ifdef TPF_COMPRESS
extern "C" int tpf_compress(void * input, unsigned long input_length, void * output, unsigned long output_length,unsigned long * consumedInput, unsigned long * generatedOutput);
extern "C" int tpf_expand(void * input, unsigned long input_length, void * output, unsigned long output_length,unsigned long * consumedInput, unsigned long * generatedOutput);
#endif
void QMQZ_LongRunning(sInput *Input)  {
  MQueue  myQueue;                         // Contains data for the MQ APIs
  char*   message;                         // message to put to queue
  char    zmsg[250]="";                    // buffer for message to other ECBs
  int     numECBs = 0;                     // Number of ECBs to start
  int     msgLength = 500;                 // message length
  int     timeInterval = 0;
  int     runTime = 0;
  int     period = 0;
  int     i = 0;
  int     utilization = 0; 
  time_t  startTime;
  tpf_TOD_type tod_before_sleep;
  tpf_TOD_type tod_after_sleep;
  tpf_TOD_type tod_at_next_sleep;
  NOPARSE childData;
  String  apiParm("PUTGET");
  String  file;
  int     comp = 0;
  enum {PUTGET, PUT, PUT1, GET, BROWSE, LOCK, GETBY, GETUNDER, SINGLE} apiInput = PUTGET;
  boolean validateMsg = false;                                                  //26653
  MQLONG Reason =  MQRC_NONE;                                                   //26653

//  setenv("ZLIB_SIM","1",1);
  Trace("QMQZ07:  LONGRUNNING MQCONN");
  myQueue.Conn(Input[1].Value.C());
  myQueue.SetObjectName(Input[2].Value.C());

  for (int j = 3; Input[j].Param != ""; j++)
  {
    if (Input[j].Param == "SLEEPinterval-d*")
       timeInterval = Input[j].Value.ToInt();

    else if (Input[j].Param == "LOOPCount-d*")
       period = Input[j].Value.ToInt();

    else if (Input[j].Param == "LOOPTime-d*")
       runTime = Input[j].Value.ToInt();

    else if (Input[j].Param == "LENgth-d*")
       msgLength = Input[j].Value.ToInt();

    else if (Input[j].Param == "UTil-d*")
       {   
       utilization = Input[j].Value.ToInt();
       if (utilization > 99)
          utilization = 99; 
       }

    else if (Input[j].Param == PARM_API_PUTGET) // API Action
    {
      apiParm  = Input[j].Value;
      apiInput = PUTGET;
    }

    else if (Input[j].Param == PARM_API_SINGLE)
    {
      apiParm  = Input[j].Value;
      apiInput = SINGLE;
    }

    else if (Input[j].Param == PARM_API_PUT)
    {
      apiParm  = Input[j].Value;
      apiInput = PUT;
    }

    else if (Input[j].Param == PARM_API_PUT1)                                   //26533
    {                                                                           //26533
      apiParm  = Input[j].Value;                                                //26533
      apiInput = PUT1;                                                          //26533
    }                                                                           //26533

    else if (Input[j].Param == PARM_API_GET)
    {
      apiParm  = Input[j].Value;
      apiInput = GET;
    }

    else if (Input[j].Param == PARM_API_GETBY)
    {
      apiParm  = Input[j].Value;
      apiInput = GETBY;
    }

    else if (Input[j].Param == PARM_API_GETUNDER)
    {
      apiParm  = Input[j].Value;
      apiInput = GETUNDER;
    }

    else if (Input[j].Param == PARM_API_BROWSE)
    {
      apiParm  = Input[j].Value;
      apiInput = BROWSE;
    }

    else if (Input[j].Param == PARM_API_LOCK)
    {
      apiParm  = Input[j].Value;
      apiInput = LOCK;
    }

    else if (Input[j].Param == "ECB-d*")         // ECB
      CheckNumEcbs(numECBs = Input[j].Value.ToInt());

    else if (Input[j].Param == "Ver-d*")         // GMO Version                 //d26214
    {                                                                           //d26214
      CheckGmoVersion(Input[j].Value.ToInt());                                  //d26214
      myQueue.SetGmoVersion(Input[j].Value.ToInt());                            //d26214
    }                                                                           //d26214
    else if (Input[j].Param == PARM_FILE)  {
      file = Input[j].Value.C();
    }
    else if (Input[j].Param == PARM_COMPRESS) {
      comp = Input[j].Value.ToInt();
    }
    else if (Input[j].Param == PARM_VALIDATE)     // VALIDATE FILE              //26653
    {                                                                           //26653
      if (Input[j].Value == "Y") {                                              //26653
        validateMsg = true;                                                     //26653
      }                                                                         //26653
    }                                                                           //26653
  }  // no more parameters

  if (numECBs > 1)
  {
    printf("Starting a long running parent ECB\n");
    if (period != 0)
    {
       sprintf(zmsg, "ZTEST MQS1 LONGRUNNING %s %s SLEEP-%d LOOPC-%d API-%s",
               myQueue.GetManagerName(),
               myQueue.GetObjectName(),
               timeInterval,
               period,
               apiParm.C());
       childData.time = 0;
       childData.count = period;
    }
    else
    {
       sprintf(zmsg, "ZTEST MQS1 LONGRUNNING %s %s SLEEP-%d LOOPT-%d API-%s",
               myQueue.GetManagerName(),
               myQueue.GetObjectName(),
               timeInterval,
               runTime,
               apiParm.C());
       childData.count = 0;
       childData.time = runTime;
    }

    childData.length = msgLength;
    strcpy(childData.qmgr,myQueue.GetManagerName());
    strcpy(childData.queue,myQueue.GetObjectName());

    for (int j=0; j < (numECBs-1); j++)
    {
      // For every 200 runs, issue an defrc to avoid CTL-10
      if (j%200 == 199)
        defrc();

      if (apiInput == SINGLE)
      {
        CreateECBs(1, (char*) &childData, TRUE);  //Create and wait
      }
      else
      {
        CreateECBs(1, zmsg, FALSE);  //Create and return
        if (numECBs < 50) sleep(1);
      }
    }
  }
  else
  {
    printf("Starting a long running ECB\n");
  }

  switch (apiInput)
  {
     case PUTGET:
     case SINGLE:
     case GETBY:
        myQueue.SetOpenOptions(MQOO_OUTPUT + MQOO_INPUT_AS_Q_DEF);
        break;
     case PUT:
     case PUT1:                                                                 //26533
        myQueue.SetOpenOptions(MQOO_OUTPUT);
        break;
     case GET:
        myQueue.SetOpenOptions(MQOO_INPUT_AS_Q_DEF);
        break;
     case BROWSE:
     case LOCK:
        myQueue.SetOpenOptions(MQOO_INPUT_AS_Q_DEF | MQOO_BROWSE);
        break;
     case GETUNDER:
        myQueue.SetOpenOptions(MQOO_OUTPUT + MQOO_INPUT_AS_Q_DEF + MQOO_BROWSE);
        break;
  }

  int fileLength = 0;
  char * compBuffer = NULL;
  if (file == "") {
     if (((message = (char*) malloc(msgLength)) == NULL) && msgLength)
       Error("No malloc space");
     memset(message, 'Z', msgLength);
  } else {
     printf("processing message from file %s\n",file.C());
     std::ifstream fsIn;
     fsIn.open(file.C());
     fsIn.seekg(0, std::ios::end);
     fileLength = fsIn.tellg(); 
     if (fileLength == -1) 
       Error("Could not read file");
     if (msgLength > fileLength) 
       Error(sf("File only contains %d bytes of data",fileLength));
     if (((message = (char*) malloc(fileLength)) == NULL) && fileLength)
       Error("No malloc space");
     if (comp)
       if (((compBuffer = (char*) malloc(fileLength)) == NULL) && fileLength)
         Error("No malloc space");
     fsIn.seekg(0, std::ios::beg);
     fsIn.read((char *) message, fileLength);
     if (msgLength == 0) msgLength = fileLength;
     printf("queuing %d bytes from %d total of file %s\n",msgLength,fileLength,file.C());
  }
  int ret;

  char   *buffer = (char*) malloc(msgLength + 1);                               //26533

  myQueue.SetGetError(MQRC_NO_MSG_AVAILABLE);
  Trace("QMQZ07:  LONGRUNNING MQOPEN");

  // Only open if not MQPUT1
  if (apiInput != PUT1)                                                         //26533
      myQueue.Open();                                                           //26533

  myQueue.SetGetOptions(MQGMO_ACCEPT_TRUNCATED_MSG + MQGMO_WAIT);
  myQueue.SetWaitInterval(1000);
  myQueue.SetWaitPrint(false);

  startTime = time(NULL);

  tod_at_next_sleep.stck_hex = 0;
  while ((i < period) || (difftime(time(NULL), startTime) < runTime))
  {
     switch (apiInput)
     {
        case PUTGET:
           if (fileLength == 0) {
              MessagetoPut(message, i, msgLength);
           } 
           Trace("QMQZ07:  LONGRUNNING MQPUT/MQGET");
           if (comp > 0) {
              if (comp < 11) {
                 z_stream strm;
                 strm.zalloc = Z_NULL;
                 strm.zfree = Z_NULL;
                 strm.opaque = Z_NULL;
                 if (comp == 10) {
                     // force software zlib
                     setenv("_HZC_COMPRESSION_METHOD","SOFTWARE",1);
                     int zlibWindowBits = -15;
                     int zlibFlushMode = Z_DEFAULT_STRATEGY;
                     deflateInit2(&strm,Z_DEFAULT_COMPRESSION,Z_DEFLATED, zlibWindowBits, 8, zlibFlushMode);
                 } else {
                     deflateInit(&strm, comp);
                 }
                 strm.avail_in = msgLength;  
                 strm.next_in = (Bytef *)message;
                 strm.avail_out = fileLength;
                 strm.next_out = (Bytef *)compBuffer;
                 ret = deflate(&strm, Z_FINISH);
                 if (ret != Z_STREAM_END) {   
                    Error(sf("Issue during deflate: %d",ret));
                 }
                 int deflSize = fileLength - strm.avail_out;
                 myQueue.Put(compBuffer, deflSize);
                 int bytesRead = msgLength - strm.avail_in;  
                 int bytesCompressed = fileLength - strm.avail_out;
                 Trace(sf("QMQZ07:  compressed %d bytes to %d",bytesRead,bytesCompressed));
                 deflateEnd(&strm);
              } else {
#ifdef TPF_COMPRESS
                 unsigned long bytesRead;
                 unsigned long bytesCompressed;
                 int rc = tpf_compress(message,msgLength,compBuffer,fileLength,&bytesRead,&bytesCompressed);
              //   Trace(sf("QMQZ07:  compressed %d bytes to %d",bytesRead,bytesCompressed));
                 if (rc == 1) { 
                    Error("Issue during tpf_compress");
                 }
                 myQueue.Put(compBuffer, bytesCompressed);
#else
                 myQueue.Put(message, msgLength);
#endif
              }
           } else {
              myQueue.Put(message, msgLength);
           }
           tpf_tcpip_message_cnt(NSDB_INPUT_CNT, 1414,NSDB_TCP_S, 1);           //26653
           usleep(timeInterval);
           Trace("QMQZ07:  LONGRUNNING MQPUT/MQGET");

           // check if we should validate the contents and size of message
           if (validateMsg == true) {                                           //26653
             Reason = myQueue.Get(buffer, msgLength, file.C(), message);        //26653
           }                                                                    //26653
           else {                                                               //26653
              Reason = myQueue.Get(buffer, msgLength);                          //26653
           }                                                                    //26653

           if (Reason == MQRC_NO_MSG_AVAILABLE) {                               //26653
             break;
           }
           if (myQueue.getReturnLength() == 0) break;
           if (comp > 0) {
              int retLength = myQueue.getReturnLength();
              if (comp < 11) {
                 z_stream strm;
                 strm.zalloc = Z_NULL;
                 strm.zfree = Z_NULL;
                 strm.opaque = Z_NULL;
                 if (comp == 10) {
                    defrc();
                    inflateInit2(&strm, -15); 
                 } else {
                    inflateInit(&strm); 
                 }
                 strm.avail_in = retLength;
                 strm.next_in = (Bytef *)buffer;
                 strm.avail_out = fileLength;
                 strm.next_out = (Bytef *)compBuffer;
                 Trace(sf("inflate: strm.ai=%d,strm.ao=%d",strm.avail_in,strm.avail_out));
                 ret = inflate(&strm, Z_FINISH);
                 if (ret != Z_STREAM_END) {   
                    Error(sf("Issue during inflate: %d filelength=%d",ret,strm.avail_in));
                 }
                 int bytesRead = retLength - strm.avail_in;  
                 int bytesExpanded = fileLength - strm.avail_out;
                 Trace(sf("QMQZ07:  expanded %d bytes to %d",bytesRead,bytesExpanded));
                 inflateEnd(&strm);
              } else {
#ifdef TPF_COMPRESS
                 unsigned long bytesRead;
                 unsigned long bytesExpanded;   
                 int rc = tpf_expand(buffer,retLength,compBuffer,fileLength,&bytesRead,&bytesExpanded);
                 Trace(sf("QMQZ07:  expanded %d bytes to %d",bytesRead,bytesExpanded));
                 if (rc == 1) { 
                    Error("Issue during tpf_expand");
                 }
#endif
              }
           } 
           break;

        case SINGLE:
           if (fileLength == 0) {
              MessagetoPut(message, 0, msgLength);
           } 
           if (comp > 0) {
              z_stream strm;
              strm.zalloc = Z_NULL;
              strm.zfree = Z_NULL;
              strm.opaque = Z_NULL;
              ret = deflateInit(&strm, comp);
              strm.avail_in = msgLength;  
              strm.next_in = (Bytef *)message;
              strm.avail_out = fileLength;
              strm.next_out = (Bytef *)compBuffer;
              Trace(sf("deflate: strm.ai=%d,strm.ao=%d",strm.avail_in,strm.avail_out));
              ret = deflate(&strm, Z_FINISH);
              if (ret == Z_STREAM_ERROR) {
                 Error("Issue during deflate");
              }
              int deflSize = fileLength - strm.avail_out;
              myQueue.Put(compBuffer, deflSize);
           } else {
              myQueue.Put(message, msgLength);
           }
           usleep(timeInterval);
           myQueue.Get(buffer, msgLength);
           break;

        case PUT:
           if (fileLength == 0) {
              MessagetoPut(message, i, msgLength);
           } 
           Trace("QMQZ07:  LONGRUNNING MQPUT");
           if (comp > 0) {
              if (comp < 11) {
                 z_stream strm;
                 strm.zalloc = Z_NULL;
                 strm.zfree = Z_NULL;
                 strm.opaque = Z_NULL;
                 if (comp == 10) {
                     // force software zlib
                     setenv("_HZC_COMPRESSION_METHOD","SOFTWARE",1);
                     int zlibWindowBits = -15;
                     int zlibFlushMode = Z_DEFAULT_STRATEGY;
                     deflateInit2(&strm,Z_DEFAULT_COMPRESSION,Z_DEFLATED, zlibWindowBits, 8, zlibFlushMode);
                 } else {
                     if (comp < 10) {
                        deflateInit(&strm, comp);
                     } else {
                        deflateInit(&strm, Z_DEFAULT_COMPRESSION);
                     }
                 }
                 strm.avail_in = msgLength;  
                 strm.next_in = (Bytef *)message;
                 strm.avail_out = fileLength;
                 strm.next_out = (Bytef *)compBuffer;
                 Trace(sf("deflate: strm.ai=%d,strm.ao=%d",strm.avail_in,strm.avail_out));
                 ret = deflate(&strm, Z_FINISH);
                 if (ret != Z_STREAM_END) {   
                    Error(sf("Issue during deflate: %d",ret));
                 }
                 int deflSize = fileLength - strm.avail_out;
                 myQueue.Put(compBuffer, deflSize);
                 int bytesRead = msgLength - strm.avail_in;  
                 int bytesCompressed = fileLength - strm.avail_out;
                 Trace(sf("QMQZ07:  compressed %d bytes to %d",bytesRead,bytesCompressed));
                 deflateEnd(&strm);
              } else {
#ifdef TPF_COMPRESS
                 unsigned long bytesRead;
                 unsigned long bytesCompressed;
                 int rc = tpf_compress(message,msgLength,compBuffer,fileLength,&bytesRead,&bytesCompressed);
//               Trace(sf("QMQZ07:  compressed %d bytes to %d",bytesRead,bytesCompressed));
                 if (rc == 1) { 
                    Error("Issue during tpf_compress");
                 }
                 myQueue.Put(compBuffer, bytesCompressed);
#else
                 myQueue.Put(message, msgLength);
#endif
              }
           } else {
              myQueue.Put(message, msgLength);
           }
           break;

        case PUT1:                                                              //26533
           if (fileLength == 0) {                                               //26533
              MessagetoPut(message, i, msgLength);                              //26533
           }                                                                    //26533
           Trace("QMQZ07:  LONGRUNNING MQPUT1");                                //26533
           if (comp > 0) {                                                      //26533
              if (comp < 11) {                                                  //26533
                 z_stream strm;                                                 //26533
                 strm.zalloc = Z_NULL;                                          //26533
                 strm.zfree = Z_NULL;                                           //26533
                 strm.opaque = Z_NULL;                                          //26533
                 if (comp == 10) {                                              //26533
                     // force software zlib
                     setenv("_HZC_COMPRESSION_METHOD","SOFTWARE",1);            //26533
                     int zlibWindowBits = -15;                                  //26533
                     int zlibFlushMode = Z_DEFAULT_STRATEGY;                    //26533
                     deflateInit2(&strm,Z_DEFAULT_COMPRESSION,Z_DEFLATED, zlibWindowBits, 8, zlibFlushMode);
                 } else {                                                       //26533
                     if (comp < 10) {                                           //26533
                        deflateInit(&strm, comp);                               //26533
                     } else {                                                   //26533
                        deflateInit(&strm, Z_DEFAULT_COMPRESSION);              //26533
                     }                                                          //26533
                 }                                                              //26533
                 strm.avail_in = msgLength;                                     //26533
                 strm.next_in = (Bytef *)message;                               //26533
                 strm.avail_out = fileLength;                                   //26533
                 strm.next_out = (Bytef *)compBuffer;                           //26533
                 Trace(sf("deflate: strm.ai=%d,strm.ao=%d",strm.avail_in,strm.avail_out));
                 ret = deflate(&strm, Z_FINISH);                                //26533
                 if (ret != Z_STREAM_END) {                                     //26533
                    Error(sf("Issue during deflate: %d",ret));                  //26533
                 }                                                              //26533
                 int deflSize = fileLength - strm.avail_out;                    //26533
                 myQueue.Put(compBuffer, deflSize);                             //26533
                 int bytesRead = msgLength - strm.avail_in;                     //26533
                 int bytesCompressed = fileLength - strm.avail_out;             //26533
                 Trace(sf("QMQZ07:  compressed %d bytes to %d",bytesRead,bytesCompressed));
                 deflateEnd(&strm);                                             //26533
              } else {                                                          //26533
#ifdef TPF_COMPRESS                                                             //26533
                 unsigned long bytesRead;                                       //26533
                 unsigned long bytesCompressed;                                 //26533
                 int rc = tpf_compress(message,msgLength,compBuffer,fileLength,&bytesRead,&bytesCompressed);
//               Trace(sf("QMQZ07:  compressed %d bytes to %d",bytesRead,bytesCompressed));
                 if (rc == 1) {                                                 //26533
                    Error("Issue during tpf_compress");                         //26533
                 }                                                              //26533
                 myQueue.Put(compBuffer, bytesCompressed);                      //26533
#else                                                                           //26533
                 myQueue.Put(message, msgLength);                               //26533
#endif                                                                          //26533
              }                                                                 //26533
           } else {                                                             //26533
              myQueue.Put1(message, msgLength);                                 //26533
           }                                                                    //26533
           break;                                                               //26533

        case GET:
           Trace("QMQZ07:  LONGRUNNING MQGET");
           myQueue.ClearMessageDescriptor();
           {
              Trace(sf("QMQZ07:  Issuing MQGET with %p (%d)",buffer,msgLength));
              MQLONG reason = myQueue.Get(buffer, msgLength);
              if (reason == MQRC_NO_MSG_AVAILABLE) {
                 Trace("QMQZ07:  NO MESSAGES");
                 break;
              }
              if (myQueue.getCompletionCode() != MQCC_OK) {
                 Trace(sf("QMQZ07:  completion code %d",myQueue.getCompletionCode()));
                 break;
              }
              if (myQueue.getReturnLength() == 0) {
                 Trace(sf("QMQZ07:  MESSAGE LENGTH 0, reason %d",reason));
                 break;
              }
           }
           if (comp > 0) {
              int retLength = myQueue.getReturnLength();
              if (comp < 11) {
                 z_stream strm;
                 strm.zalloc = Z_NULL;
                 strm.zfree = Z_NULL;
                 strm.opaque = Z_NULL;
                 if (comp == 10) {
                    // force software zlib
                    setenv("_HZC_COMPRESSION_METHOD","SOFTWARE",1);
                    defrc();
                    inflateInit2(&strm, -15); 
                 } else {
                    inflateInit(&strm); 
                 }
                 strm.avail_in = retLength;
                 strm.next_in = (Bytef *)buffer;
                 strm.avail_out = fileLength;
                 strm.next_out = (Bytef *)compBuffer;
                 Trace(sf("inflate: strm.ai=%d,strm.ao=%d",strm.avail_in,strm.avail_out));
                 ret = inflate(&strm, Z_FINISH);
                 if (ret != Z_STREAM_END) {   
                    Error(sf("Issue during inflate: %d",ret));
                 }
                 int bytesRead = msgLength - strm.avail_in;  
                 int bytesExpanded = fileLength - strm.avail_out;
                 Trace(sf("QMQZ07:  expanded %d bytes to %d",bytesRead,bytesExpanded));
                 inflateEnd(&strm);
              } else {
#ifdef TPF_COMPRESS
                 unsigned long bytesRead;
                 unsigned long bytesExpanded;   
                 int rc = tpf_expand(buffer,retLength,compBuffer,fileLength,&bytesRead,&bytesExpanded);
                 Trace(sf("QMQZ07:  expanded %d bytes to %d",bytesRead,bytesExpanded));
                 if (rc == 1) { 
                    Error("Issue during tpf_expand");
                 }
#endif
              }
           }
           break;

        case GETBY:
           if (fileLength == 0) {
              MessagetoPut(message, i, msgLength);
           } 
           Trace("QMQZ07:  LONGRUNNING MQPUT/MQGET");
           if (comp > 0) {
              z_stream strm;
              strm.zalloc = Z_NULL;
              strm.zfree = Z_NULL;
              strm.opaque = Z_NULL;
              ret = deflateInit(&strm, comp);
              strm.avail_in = msgLength;  
              strm.next_in = (Bytef *)message;
              strm.avail_out = fileLength;
              strm.next_out = (Bytef *)compBuffer;
              Trace(sf("deflate: strm.ai=%d,strm.ao=%d",strm.avail_in,strm.avail_out));
              ret = deflate(&strm, Z_FINISH);
              if (ret == Z_STREAM_ERROR) {
                 Error("Issue during deflate");
              }
              int deflSize = fileLength - strm.avail_out;
              myQueue.Put(compBuffer, deflSize);
           } else {
              myQueue.Put(message, msgLength);
           }

           myQueue.ClearMatchOptions();
           myQueue.SetMatchOptions(MQMO_MATCH_MSG_ID + MQMO_MATCH_CORREL_ID);
           Trace("QMQZ07:  LONGRUNNING MQGET WITH MSGID AND CORRELID");
           myQueue.Get(buffer, msgLength);
           break;

        case GETUNDER:
           myQueue.SetGetOptions(MQGMO_BROWSE_NEXT+MQGMO_LOCK+
                                 MQGMO_ACCEPT_TRUNCATED_MSG);
           Trace("QMQZ07:  LONGRUNNING LOCK");
           myQueue.Get(buffer, msgLength);

           myQueue.SetGetOptions(MQGMO_MSG_UNDER_CURSOR +
                                 MQGMO_ACCEPT_TRUNCATED_MSG);
           Trace("QMQZ07:  LONGRUNNING GET MSG UNDER CURSOR");
           myQueue.Get(buffer, msgLength);

           MessagetoPut(message, i, msgLength);
           Trace("QMQZ07:  LONGRUNNING MQPUT");
           myQueue.Put(message, msgLength);
           break;

        case BROWSE:
           if (i == 0)
              myQueue.AddGetOptions(MQGMO_BROWSE_FIRST);
           else
           // Remove MQGMO_BROWSE_FIRST and add MQGMO_BROWSE_NEXT.
              myQueue.AddGetOptions(MQGMO_BROWSE_NEXT, MQGMO_BROWSE_FIRST);
           Trace("QMQZ07:  LONGRUNNING BROWSE");
           myQueue.Get(buffer, msgLength);
           break;

        case LOCK:
           if (i == 0)
              myQueue.AddGetOptions(MQGMO_BROWSE_FIRST+MQGMO_LOCK);
           else
           // Remove MQGMO_BROWSE_FIRST and add MQGMO_BROWSE_NEXT.
              myQueue.AddGetOptions(MQGMO_BROWSE_NEXT+MQGMO_LOCK, MQGMO_BROWSE_FIRST+MQGMO_LOCK);
           Trace("QMQZ07:  LONGRUNNING LOCK");
           myQueue.Get(buffer, msgLength);
           break;

       }  // end of switch

       // if current TOD > tod_at_next_sleep then usleep for 4 milliseconds and 
       // check tod on return to set tod_at_next_sleep for how long the ECB should exit before
       // sleeping for another 4 mills
       if (utilization == 0)
         {
          usleep(timeInterval);
         }
       else
         {
         tpf_STCK(&tod_before_sleep);

         if (tod_before_sleep.stck_hex > tod_at_next_sleep.stck_hex)
           {
            usleep(4095);
            tpf_STCK(&tod_after_sleep);

            tod_at_next_sleep.stck_hex =
                tod_after_sleep.stck_hex +
                utilization * (tod_after_sleep.stck_hex - tod_before_sleep.stck_hex)/
                (100-utilization);
            }
         }


     // For every 200 runs, issue an defrc to avoid CTL-10
     if ((timeInterval ==0) && (utilization == 0))
        if (i%200 == 199)
           defrc();
     i++;
  }

  free(buffer);
  free(message);

  // Only close if not MQPUT1
  if (apiInput != PUT1)                                                         //26533
  {                                                                             //26533
     Trace("QMQZ07:  LONGRUNNING MQCLOSE");                                     //26533
     myQueue.Close();                                                           //26533
  }                                                                             //26533

  Trace("QMQZ07:  LONGRUNNING MQDISC");
  myQueue.Disc();

  if (numECBs > 1)
   printf("This long running parent ECB finishes\n");
  else
   printf("This long running ECB finishes\n");
}


//**************************************************************************
void QMQZ_LongRunning_noparser(NOPARSE * input) {
  if ((input->count == 0) && (input->time == 0))
     exit(0);
  MQueue  myQueue;                         // Contains data for the MQ APIs
  char*   message;                         // message to put to queue
  int     msgLength = input->length;       // message length
  time_t  startTime;
  int     i = 0;
  char   *buffer = (char*) malloc(msgLength + 1);
  if (((message = (char*)malloc(msgLength)) == NULL) && msgLength)
    Error("No malloc space");
  memset(message, 'Z', msgLength);
  myQueue.Conn(input->qmgr);
  myQueue.SetObjectName(input->queue);
  myQueue.SetOpenOptions(MQOO_OUTPUT + MQOO_INPUT_AS_Q_DEF);
  myQueue.SetGetError(MQRC_NO_MSG_AVAILABLE);
  myQueue.Open();
  myQueue.SetGetOptions(MQGMO_ACCEPT_TRUNCATED_MSG);

  startTime = time(NULL);

  while ((i < input->count) || (difftime(time(NULL), startTime) < input->time))
  {
     MessagetoPut(message, 0, msgLength);
     myQueue.Put(message, msgLength);
     tpf_tcpip_message_cnt(NSDB_INPUT_CNT, 1414,NSDB_TCP_S, 1);                 //26653
     myQueue.Get(buffer, msgLength);
     // For every 200 runs, issue an defrc to avoid CTL-10
     if (i%200 == 199)
        defrc();
     i++;
  }
  free(buffer);
  free(message);
  myQueue.Close();
  myQueue.Disc();

}

