/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2006, 2023                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*  26359   06/14/18  Remove "min" & "max" for PJ45111               */
/*  d26352  20180810  Compiler compatibility updates                 */
/*  d26214  01/03/19  Add option for version to fix driver with mq   */
/*                    client (ZMQID)                                 */
/*  d26533  11/20/19  z/TPF WebSphere MQ support for dynamic routing */
/*  d26725  09/29/21  Create z/TPF MQ DevOps Test Cases              */
/*  d26841  20230927  MQS1 Driver changes for 64-bit MQ support      */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME.....MQS1                                            */
/* FUNCTION TYPE.....TPF                                             */
/* SEGMENT NAME....  qmqz06.cpp                                      */
/*                                                                   */
/* DESCRIPTION:                                                      */
/*   QMQZ06 is a suite of test cases for the MQ drivers.             */
/*   Individual Test cases are documented in MQ Driver Docs          */
/*                                                                   */
/*********************************************************************/
#include <qmqz.h>
#include <tpf/c_devops.h>                                                       //d26725
#include <tpf/malloc.h>                                                         //d26725

#define DEFAULT_MSG_BUFFER 200
#define PRIME_CRAS         0x00010000                                           //d26725
#define ROUTE_LEN          3                                                    //d26725
#define PRT_INFO           1                                                    //d26725

static void Case1(),   Case2(),   Case3(),   Case4();
static void Case5(),   Case6(),   Case7(),   Case8();
static void Case9(),   Case10(),  Case11(),  Case12();
static void Case13(),  Case14(),  Case15(),  Case16();
static void Case17(),  Case18(),  Case19(),  Case20();
static void Case21(),  Case22(),  Case23(),  Case24();
static void Case25(),  Case26(),  Case27(),  Case28();
static void Case29(),  Case30(),  Case31(),  Case32();
static void Case33(),  Case34(),  Case35(),  Case36();
static void Case37(),  Case38(),  Case39(),  Case40();
static void Case41(),  Case42(),  Case43(),  Case44();
static void Case45(),  Case46(),  Case47(),  Case48();
static void Case49(),  Case50(),  Case51(),  Case52();
static void Case53(),  Case54(),  Case55(),  Case56();
static void Case57(),  Case58(),  Case59(),  Case60();
static void Case61(),  Case62(),  Case63(),  Case64();
static void Case65(),  Case66(),  Case67(),  Case68();
static void Case69(),  Case70(),  Case71(),  Case72();
static void Case73(),  Case74(),  Case75();

static void Case101(), Case102(), Case103(), Case104();

static void Case200(), Case201(), Case202(), Case203();
static void Case204(), Case205(), Case206(), Case207();
static void Case208(), Case209(), Case210(), Case211();
static void Case212(), Case213(), Case214(), Case215();
static void Case216(), Case217(), Case218(), Case219();
static void Case220(), Case221(), Case222(), Case223();
static void Case224(), Case225(), Case226(), Case227();
static void Case228(), Case229(), Case230(), Case231();
static void Case232(), Case233(), Case234(), Case235();
static void Case236(), Case237(), Case238(), Case239();
static void Case240(), Case241(), Case242(), Case243();
static void Case244(), Case245(), Case246(), Case247();
static void Case248(), Case249(), Case250(), Case251();
static void Case252(), Case253();                                               //d26725

static void Conn();
static void Disc();
static void Open(int H = 1, int Opt = 0);
static void Close(int H = 1);
// Following not used commented out to remove compiler warning
//static void ReOpen(int H = 1, int Opt = 0);
//static void Inq(MQLONG attr, long *pValue, int H);
static void Rigor4();

// Get the function associated with a case number.
static CFunction getCaseFunc(short caseNbr);

// Get the index in the case table or a case number.
static int getCasePos(short caseNbr);

// ----------------------------
// Notes on sCaseTable (below):
// ----------------------------
// openOptions contains the MQOO values for the corresponding Hobj.
// This handles a maximum of 3 Hobj's.  Increase it if you need more.
// If an option of -1 is encountered, no more handles are opened.
//
// nbrQueues is the number of queues that must be specified for this case.
// If it's zero, it defaults to one.
//
// nbrHobjs is the number of Hobjs needed for this case.  If it's less
// than nbrQueues, it defaults to nbrQueues.  If it's more than nbrQueues,
// then the excess starts back from the first queue defined.  So if there
// are 2 queues and 3 Hobjs, The 1st and 3rd Hobjs will be used with the
// 1st queue and the 2nd Hobj will be used with the 2nd queue.  If the
// user specifies more Hobj's than is needed by the case, the excess are
// ignored.
//
// needsLocalQM, if 1, means that the test case must use the local queue
// manager.  If another queue manager is specified with the QM-x parameter,
// this test case is skipped.

struct sCaseTable
{
  short     caseNumber;      // case number
  CFunction caseFunc;        // case function
  MQLONG    openOptions[3];  // open options (zero = default)
  short     nbrQueues;       // number of queues needed (zero = 1)
  short     nbrHobjs;        // nbr of Hobj's needed (zero = nbrQueues)
  char      needsLocalQM;    // 1 = requires local QMGR (no QM-x)
};

// Static variables.

#define DEFAULT_OPEN_OPTIONS (MQOO_INPUT_AS_Q_DEF | MQOO_OUTPUT)

// CaseTable elements must be in sequence by case number.

static const sCaseTable CaseTable[] =
{
  {1, Case1}, {2, Case2}, {3, Case3, {0}, 0, 0, 1},
  {4, Case4}, {5, Case5}, {6, Case6}, {7, Case7}, {8, Case8},
  {9, Case9}, {10, Case10},
  {11, Case11}, {12, Case12}, {13, Case13}, {14, Case14}, {15, Case15},
  {16, Case16}, {17, Case17}, {18, Case18}, {19, Case19}, {20, Case20},
  {21, Case21}, {22, Case22}, {23, Case23}, {24, Case24}, {25, Case25},
  {26, Case26}, {27, Case27}, {28, Case28}, {29, Case29}, {30, Case30},
  {31, Case31}, {32, Case32}, {33, Case33}, {34, Case34}, {35, Case35},
  {36, Case36}, {37, Case37}, {38, Case38}, {39, Case39}, {40, Case40},
  {41, Case41}, {42, Case42}, {43, Case43}, {44, Case44}, {45, Case45},
  {46, Case46}, {47, Case47}, {48, Case48}, {49, Case49}, {50, Case50},
  {51, Case51, {0}, 3}, {52, Case52, {0}, 3}, {53, Case53, {0}, 3},
  {54, Case54, {0}, 3}, {55, Case55, {0}, 3}, {56, Case56, {0}, 3},
  {57, Case57}, {58, Case58}, {59, Case59}, {60, Case60},
  {61, Case61}, {62, Case62}, {63, Case63}, {64, Case64}, {65, Case65},
  {66, Case66}, {67, Case67}, {68, Case68}, {69, Case69}, {70, Case70},
  {71, Case71}, {72, Case72}, {73, Case73}, {74, Case74}, {75, Case75},

  {101, Case101, {MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {102, Case102}, {103, Case103}, {104, Case104},

  {200, Case200, {MQOO_BROWSE}},
  {201, Case201, {MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {202, Case202, {MQOO_OUTPUT}},
  {203, Case203, {MQOO_INPUT_AS_Q_DEF}},
  {204, Case204, {MQOO_OUTPUT | MQOO_INPUT_AS_Q_DEF}},
  {205, Case205, {MQOO_OUTPUT | MQOO_BROWSE}},
  {206, Case206, {MQOO_OUTPUT | MQOO_BROWSE}},
  {207, Case207, {MQOO_OUTPUT | MQOO_BROWSE}},
  {208, Case208, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {209, Case209, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {210, Case210, {MQOO_OUTPUT | MQOO_BROWSE}},       // 1 Hobj for 1 Q
  {211, Case211, {MQOO_OUTPUT | MQOO_BROWSE}, 1, 2}, // 2 Hobj for 1 Q
  {212, Case212, {MQOO_OUTPUT | MQOO_BROWSE}, 1, 2}, // 2 Hobj for 1 Q
  {213, Case213, {MQOO_OUTPUT | MQOO_BROWSE}, 1, 2}, // 2 Hobj for 1 Q
  {214, Case214, {MQOO_OUTPUT | MQOO_BROWSE}, 1, 2}, // 2 Hobj for 1 Q
  {215, Case215, {MQOO_OUTPUT | MQOO_BROWSE}},       // 1 Hobj for 1 Q
  {216, Case216, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF},
                 1, 2},
  {217, Case217, {MQOO_OUTPUT | MQOO_BROWSE}},       // 1 Hobj for 1 Q
  {218, Case218, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {219, Case219, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {220, Case220, {MQOO_OUTPUT | MQOO_BROWSE}, 1, 2}, // 2 Hobj for 1 Q
  {221, Case221, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF},
                 1, 2},
  {222, Case222, {MQOO_OUTPUT | MQOO_BROWSE}},       // 1 Hobj for 1 Q
  {223, Case223, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF,
                  MQOO_INPUT_AS_Q_DEF}, 1, 2},       // 2 Hobj for 1 Q
  {224, Case224, {MQOO_OUTPUT | MQOO_BROWSE}},       // 1 Hobj for 1 Q
  {225, Case225, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {226, Case226, {MQOO_OUTPUT | MQOO_BROWSE}},       // 1 Hobj for 1 Q
  {227, Case227, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF,
                  MQOO_INPUT_AS_Q_DEF}, 1, 2},       // 2 Hobj for 1 Q
  {228, Case228, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF,
                  MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}, 1, 2},
  {229, Case229, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {230, Case230, {MQOO_OUTPUT | MQOO_BROWSE,
                  MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}, 1, 2},
  {231, Case231, {MQOO_OUTPUT | MQOO_BROWSE,
                  MQOO_INPUT_AS_Q_DEF}, 1, 2},
  {232, Case232, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {233, Case233, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {234, Case234, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {235, Case235, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {236, Case236, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {237, Case237, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {238, Case238, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {239, Case239, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {240, Case240},
  {241, Case241},
  {242, Case242},
  {243, Case243, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {244, Case244, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {245, Case245, {MQOO_OUTPUT | MQOO_INPUT_AS_Q_DEF}},
  {246, Case246, {MQOO_OUTPUT | MQOO_BROWSE,
                  MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}, 1, 2},
  {247, Case247, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {248, Case248, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {249, Case249, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {250, Case250, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {251, Case251, {MQOO_OUTPUT | MQOO_BROWSE | MQOO_INPUT_AS_Q_DEF}},
  {252, Case252, {MQOO_OUTPUT | MQOO_INPUT_AS_Q_DEF}},
  {253, Case253, {MQOO_OUTPUT | MQOO_INPUT_AS_Q_DEF}},
};                                                                              //d26725

static int CasePos;               // current case table index

#define NUM_TEST_CASES   sizeof(CaseTable) / sizeof(sCaseTable)

static CFunction getCaseFunc(short caseNbr)
{
  for (unsigned int n = 0; n < NUM_TEST_CASES; n++)
  {
    if (CaseTable[n].caseNumber == caseNbr)
      return CaseTable[n].caseFunc;
  }

  return NULL;          // case number not found
}

static int getCasePos(short caseNbr)
{
  for (unsigned int n = 0; n < NUM_TEST_CASES; n++)
  {
    if (CaseTable[n].caseNumber == caseNbr)
      return n;
  }

  return -1;            // case number not found
}

// Sorry about the global variables but it makes this much easier
sABACK *pBackupArray = NULL;           // Heap storage for backup array
int     queues = 1;                    // Number of queues inputed
char    *qNamesMemoryCompressed[3] =   // Names of queues for automated tests   //d26725
        {                                                                       //d26725
         "MQ_TEST_MEMORY_COMPRESSED_1",                                         //d26725
         "MQ_TEST_MEMORY_COMPRESSED_2",                                         //d26725
         "MQ_TEST_MEMORY_COMPRESSED_3"                                          //d26725
        };                                                                      //d26725
char    *qNamesMemoryUncompressed[3] =                                          //d26725
        {                                                                       //d26725
         "MQ_TEST_MEMORY_UNCOMPRESSED_1",                                       //d26725
         "MQ_TEST_MEMORY_UNCOMPRESSED_2",                                       //d26725
         "MQ_TEST_MEMORY_UNCOMPRESSED_3"                                        //d26725
        };                                                                      //d26725
char    *qNamesCommonCompressed[3] =                                            //d26725
        {                                                                       //d26725
         "MQ_TEST_COMMON_COMPRESSED_1",                                         //d26725
         "MQ_TEST_COMMON_COMPRESSED_2",                                         //d26725
         "MQ_TEST_COMMON_COMPRESSED_3"                                          //d26725
        };                                                                      //d26725
char    *qNamesCommonUncompressed[3] =                                          //d26725
        {                                                                       //d26725
         "MQ_TEST_COMMON_UNCOMPRESSED_1",                                       //d26725
         "MQ_TEST_COMMON_UNCOMPRESSED_2",                                       //d26725
         "MQ_TEST_COMMON_UNCOMPRESSED_3"                                        //d26725
        };                                                                      //d26725
boolean common = FALSE;                                                         //d26725
boolean compsize = FALSE;                                                       //d26725
unsigned long memBeforeConn = 0;                                                //d26725
int     HelpVar = 0;                   // > 0 = only want HELP output
int     MQOnly = 0;                    // Do/Don't do DASD transactions
int     testcase = 0;                  // Case to run
char    output[250];                   // String to send to terminal
MQMD    md, defmd = {MQMD_DEFAULT};    // Message Descriptor
MQGMO   gmo, defgmo = {MQGMO_DEFAULT}; // Get Msg Options
MQPMO   pmo, defpmo = {MQPMO_DEFAULT}; // Put Msg Options
MQHOBJ  Hobj[6] =                      // Object Handle queues
            { MQHO_UNUSABLE_HOBJ, MQHO_UNUSABLE_HOBJ,
              MQHO_UNUSABLE_HOBJ, MQHO_UNUSABLE_HOBJ,
              MQHO_UNUSABLE_HOBJ, MQHO_UNUSABLE_HOBJ
            };

static MQHCONN Hconn = MQHC_UNUSABLE_HCONN;   // Connection Handle
static char    TrgtQName[6][MQ_Q_NAME_LENGTH + 1] = {"","","","","",""};
static BOOL    HelpOnly = FALSE;
static MQCHAR  QMgrName[MQ_Q_MGR_NAME_LENGTH + 1] = "";

static void QMQZ_Sleep(unsigned int secs)
{
  if (secs > 10)
    out(sf("sleeping for %d seconds", secs));

  sleep(secs);
}

void QMQZ_Suite(sInput *Input)         // Called from QMQZ
{
  int     seconds = 0;                 // number of seconds to run
  int     low = CaseTable[0].caseNumber; // when running all
  int     hi  = CaseTable[NUM_TEST_CASES - 1].caseNumber;

  if (Input[1].Param == "PRINT")       // Output code only
  {
    out("MQ Transactions will be output and not run");
    HelpVar = 1;
    HelpOnly = TRUE;
    Input++;
  }

  if (Input[1].Param == "ALL")         // TEST MQS1 CASE ALL ...
    testcase = 0;                      // take testcase=0 if testing all cases
  else if (Input[1].Param == "RANGE")
    testcase = 0;
  else if (Input[1].Param == "RANDOM") // TEST MQS1 CASE RANDOM ...
    testcase = -1;                     // take testcase=-1 for random testing
  else if (Input[1].Param == "TIME")   // ZTEST MQS1 CASE TIME ...
  {
    Input++;                           // advance to next input parameter
    seconds = Input[1].Value.ToInt();  // number of seconds to run
    testcase = -2;                     // take testcase=-2 for timed running
  }
  else if (Input[1].Param == "SPECial")
    testcase = -3;
  else if (Input[1].Param == "RIGOR1")
  {
    low = 0;
    hi = 3000;
    testcase = -4;                     // take testcase = -4 for RIGOR1
  }
  else if (Input[1].Param == "RIGOR2")
  {
    low = 1;
    hi = 12600;
    testcase = -5;                     // take testcase = -5 for RIGOR2
  }
  else if (Input[1].Param == "RIGOR3")
  {
    testcase = -6;                     // take testcase = -6 for RIGOR3
    hi = 1;
  }
  else if (Input[1].Param == "RIGOR4")
    testcase = -7;                     // 10000 MQOPENs
  else if (Input[1].Param == "RIGORBACKUP") // spawned ECB
  {
    pBackupArray = (sABACK*) Input[2].Value.ToPtr();
    RigorWatchA();
  }
  else if (Input[1].Param == "d*")     // ZTEST MQS1 CASE casenumber
  {
    testcase = Input[1].Value.ToInt(); // otherwise get testcase number
    CheckNumCases();
  }
  else Error("Driver Error - Case command not recognized");

  Input[2].Value.CopyTo(TrgtQName[0]); // empty (hopefully) queue name

  while (Input[3].Param == QN)         // multiple queues inputed
  {
    queues++;
    Input[3].Value.CopyTo(TrgtQName[queues - 1]); // Get next queue name
    Input++;
  }

  if (Input[3].Param == "MQONLY")      // only do MQ transactions
  {
    MQOnly = 1;
    Input++;
    out("MQ Transactions only (no DASD I/O)");
  }

  if (Input[3].Param == "Low-d*")
  {
    low = Input[3].Value.ToInt();
    CheckNumCases(low);
    Input++;
  }

  if (Input[3].Param == "Hi-d*")
  {
    hi = Input[3].Value.ToInt();
    CheckNumCases(hi);
    Input++;
  }

  if (Input[3].Param == "Low-d*")
  {
    low = Input[3].Value.ToInt();
    CheckNumCases(low);
    Input++;
  }

  if (Input[3].Param == "QM-(a*)*:48")
  {
    Input[3].Value.CopyTo(QMgrName);
    Input++;
  }

  MQLONG  Reason;                      // stores return codes
  MQLONG  CompCode;                    // stores completion code

  //Store memory in use prior to connecting to the queue manager
  memBeforeConn = GetMemoryInUse();                                             //d26725

  // Connect to Queue Manager.
  MQCONN(QMgrName, &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);

  memcpy(md.Format, MQFMT_STRING, MQ_FORMAT_LENGTH); // prep for puts or gets
  md.CodedCharSetId = CCSID_EBCDIC;

  // Test cases <= 0 run all cases.

  if ((testcase <= 0) && (testcase >= -3))
  {
    out(sf("Cases that require more than %d queue%s will be skipped",
           queues, queues > 1 ? "s" : ""));
  }

  switch (testcase)
  {
    case  0:  RunAllCases(low, hi); break;
    case -1:  RunRandomized();      break;
    case -2:  RunTimed(seconds);    break;
    case -3:  RunSpecial();         break;
    case -4:  Rigor1(low, hi);      break;
    case -5:  Rigor2(low, hi);      break;
    case -6:  Rigor3(hi);           break;
    case -7:  Rigor4();             break;
    default:  RunCase(2);           break; // run testcase, print output
  }

  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason);
  out("MQ Testing Successful.");      // Return to QMQZ
}

void EmptyAllQueues()
{
  MQLONG msgLen;                       // Msg length in bytes
  MQLONG Reason;                       // stores return codes
  MQLONG CompCode;                     // stores completion code
  char   buffer[50];                   // Buffer for queue msgs
  MQOD   ObjDesc = {MQOD_DEFAULT};     // Object Descriptor
                                       // Target queue names
  gmo = defgmo;
  gmo.Version = MQGMO_VERSION_2;                                                //d26533
  gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG; // prep for gets

  for (int i = 0; i < queues; i++)     // Open the 1 or more queues
  {
    strcpy(ObjDesc.ObjectName, TrgtQName[i]);    // Set queue name
    MQOPEN(Hconn, &ObjDesc, MQOO_INPUT_AS_Q_DEF | MQOO_INQUIRE,
           &Hobj[i], &CompCode, &Reason);        // Open queue i

    if (CompCode == MQCC_FAILED)
      continue;

    // See if the queue is empty.

    MQLONG Selectors[] = {MQIA_CURRENT_Q_DEPTH};
    MQLONG QDepth[1];

    MQINQ(Hconn, Hobj[i], 1, Selectors, 1, QDepth, 0, NULL,
          &CompCode, &Reason);
    Trace("MQINQ");

    if (Reason == MQRC_NONE)
    {
      if (QDepth[0] != 0)
      {
        out(sf("Queue has %d message%s, Emptying %s",
               QDepth[0], QDepth[0] > 1 ? "s" : "", TrgtQName[i]));

        while ((Reason == MQRC_NONE) ||
               (Reason == MQRC_TRUNCATED_MSG_ACCEPTED))
        {
          md = defmd;
          MQGET(Hconn, Hobj[i], &md, &gmo, 50, (MQBYTE*) buffer,
                &msgLen, &CompCode, &Reason);  // look for more messages
        }

        // See if the queue is empty now.  If not,
        // there were probably some locked messages.

        MQINQ(Hconn, Hobj[i], 1, Selectors, 1, QDepth, 0, NULL,
              &CompCode, &Reason);
        Trace("MQINQ");

        if (Reason == MQRC_NONE)
        {
          if (QDepth[0] != 0)
          {
            Error(sf("Could not empty %s, messages may be locked",
                     TrgtQName[i]));
          }
        }
        else
          Error("MQINQ Error");
      }
    }
    else // Unexpected error
      Error("MQINQ Error");

    // Close the queue.  We'll open it again for each test case.

    MQCLOSE(Hconn, &Hobj[i], 0, &CompCode, &Reason);
    CheckError("MQCLOSE", Reason);
  }
}


void RunAllCases(int low, int hi)      // Run all cases in order
{
  if (low > hi) Error("No cases to run");

  out(sf("Running case %d to %d in order", low, hi));

  for (testcase = low; testcase <= hi; testcase++)
  {
    if (getCaseFunc(testcase) != NULL)
      RunCase(1);
  }
}


void RunRandomized()                   // Run all cases out of order
{
  // test cases that were run
  BYTE *UsedArray = (BYTE *) calloc(NUM_TEST_CASES, sizeof(BYTE));
  int   place;                         // random testcase to find

  if (UsedArray == NULL) Error("No MALLOC space for randomizing array");

  out(sf("Running %d Test Cases in Random Order", NUM_TEST_CASES));

  srand(time(NULL));                        // Set up random generator

  for (int i = NUM_TEST_CASES; i >= 1; i--) // run cases
  {
    place = rand() % i + 1;       // randomly select a remaining case

    // Go until place unrun cases found

    for (testcase = 0; place > 0; testcase++)
      if (UsedArray[testcase] == 0) place--;  // Count each unrun case

    UsedArray[testcase - 1] = 1;            // Mark case as run

    if (getCaseFunc(testcase) != NULL)      // if it's a valid case number...
      RunCase(1);                           // ...run it
  }
}


void RunTimed(int seconds)                  // Run for requested duration
{
  struct timeval StartTime;                 // Starting Time
  struct timeval Current;                   // Current Time
  double Diff = 0;                          // Elapsed time
  struct timezone NoUse;                    // Dummy Variable
  int    interval = PRINT_INTERVAL;         // Output every PRINT_INTERVAL seconds
  int    Total = 0;                         // total number of cases run

  out(sf("Running test cases for %d seconds", seconds));

  gettimeofday(&StartTime, &NoUse);         // Get initial starting time

  for (Total = 1; Diff < seconds; Total++)  // Run until time is up
  {
    if (Diff >= interval)                   // Print an occasional status message
    {
      sprintf(output, "%d Cases Completed in %f seconds", Total, Diff);
      out(output);
      interval += PRINT_INTERVAL;           // wait another PRINT_INTERVAL seconds
    }

    testcase = Total % NUM_TEST_CASES + 1;  // sequentially run testcases

    if (getCaseFunc(testcase) != NULL)      // run case number if it's valid
      RunCase(0);                           // don't print to screen

    gettimeofday(&Current, &NoUse);         // Refresh current time
    Diff = (Current.tv_sec - StartTime.tv_sec) +  // New elapsed time
           ((double) (Current.tv_usec - StartTime.tv_usec)) / 1000000;
  }

  sprintf(output, "Time Has Expired, %d Cases Ran in %f seconds",
                  Total, Diff);
  out(output);
}


void RunSpecial()
{
  int CasesToRun[] = { 12, 15, 14, 54, 11, 14, 11 };  // Enter order of cases

  for (unsigned int i = 0; i < sizeof(CasesToRun) / sizeof(int); i++) // go thorough table
  {
    testcase = CasesToRun[i];
    RunCase(1);                                       // run each case
  }
}


void RunCase(int Display)              // Run testcase
{
  int i;
  if (Display == 2) HelpVar = 1;

  // If we don't have enough queues for this test case, skip it.

  if (CaseTable[getCasePos(testcase)].nbrQueues > queues)
  {
    if (Display == 2)                  // only one case going to be run
    {
      Error(sf("%d queues must be inputed",
               CaseTable[getCasePos(testcase)].nbrQueues));
    }

    return;                            // Skip this case
  }

  // If we need the local queue manager, verify that we have it.

  if (CaseTable[getCasePos(testcase)].needsLocalQM &&
      !isLocalQM(QMgrName))
  {
    if (Display == 2)                  // only one case going to be run
      Error("This case requires the local queue manager");

    return;                            // Skip this case
  }

  // Make sure all the queues are empty.

  EmptyAllQueues();

  // Set the open options as required by the test case.

  MQLONG OpenOptions;
  MQOD   ObjDesc = {MQOD_DEFAULT};     // Object Descriptor
  MQLONG CompCode;
  MQLONG Reason;

  CasePos = getCasePos(testcase);

  // Open all the needed queues with the number
  // of Hobj's needed for this test case.

  short nbrQueues = MAX(CaseTable[CasePos].nbrQueues, 1);                       //26359
  short nbrHobjs = MAX(CaseTable[CasePos].nbrHobjs, nbrQueues);                 //26359

  for (i = 0; i < nbrHobjs; i++)
  {
    // If the open options for this queue are zero, then
    //   if it's the first queue, use the default, otherwise
    //   use the first queue's open options.

    if (CaseTable[CasePos].openOptions[i] == 0)
    {
      if (CaseTable[CasePos].openOptions[0] == 0)
        OpenOptions = DEFAULT_OPEN_OPTIONS;
      else
        OpenOptions = CaseTable[CasePos].openOptions[0];
    }
    else if (CaseTable[CasePos].openOptions[i] == -1)
      break;  // no more handles should be open
    else
      OpenOptions = CaseTable[CasePos].openOptions[i];

    // Set queue name
    strcpy(ObjDesc.ObjectName, TrgtQName[i % nbrQueues]);
    MQOPEN(Hconn, &ObjDesc, OpenOptions, &Hobj[i], &CompCode, &Reason);
    CheckError("MQOPEN", Reason);
  }

  if (Display)                         // Don't display on timed
    out(sf("Testing Case %d", testcase));

  // Set the put and get options to their default and
  // clear the MsgId and CorrelId in the MQMD.  Each
  // test case can modify these as required.

  gmo = defgmo;
  gmo.Version = MQGMO_VERSION_2;                                                //d26533
  gmo.MatchOptions = MQMO_NONE;                                                 //d26841
  pmo = defpmo;
  md  = defmd;

  getCaseFunc(testcase)();

  // Don't close queues for case 51.  Let them close automatically.

  if (testcase != 51)
  {
    for (i = 1; i <= nbrHobjs; i++)    // Close the Hobj(s)
    {
      // The test case may have already closed this handle.
      // If so, skip the close for that handle.
      if (Hobj[i - 1] != MQHOBJ(MQHO_UNUSABLE_HOBJ))
        Close(i);
    }
  }
}


void CaseError(char *msg)              // Exit routine
{
  if (IsAutomatedTestCase()) {                                                  //d26725
    ZdevoError(msg);                                                            //d26725
    exit(1);                                                                    //d26725
  }                                                                             //d26725
  else                                                                          //d26725
  {                                                                             //d26725
    Error(sf("Case %d: %s", testcase, msg)); // Error Message                   //d26725
  }                                                                             //d26725
}


int HelpCode(char *OutString)
{
  if (HelpVar > 0)
  {
    out(sf("%*s %s", HelpVar, "", OutString));
    return (HelpOnly);
  }
  else return 0;
}


void CheckNumCases()
{ CheckNumCases(testcase); }


void CheckNumCases(int casenum)
{
  if (getCaseFunc(casenum) == NULL)
    Error(sf("Case number %d does not exist.", casenum));
}


void CheckError(char *API, MQLONG Reason) // General Error Output
{
  char  casechar[20];
  char  output[100];                   // Output buffer

  Trace(API);

  if (Reason)                          // Reason != MQRC_NONE
  {
    if (testcase) sprintf(casechar, "case %d: ", testcase);
    else strcpy(casechar, "");

    sprintf(output, "%s%s ended with reason code %d, %s",  // Returned a value
            casechar, API, Reason, MQRC_String(Reason));   // msg w/MQRC Reason

    if (IsAutomatedTestCase()) {                                                //d26725
      ZdevoError(output);                                                       //d26725
      exit(1);                                                                  //d26725
    }                                                                           //d26725
    else                                                                        //d26725
    {                                                                           //d26725
      Error(output);                                        // Exit             //d26725
    }                                                                           //d26725
  }
}

// Put a message

void Put(char *StringIn, int H)
{
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725
  char output[100];

  sprintf(output, "Put(\"%s\") to %s, handle %d",
                  StringIn,
                  TrgtQName[(H - 1) % queues], H);

  if (HelpCode(output)) return;

  MQLONG  CompCode;                    // stores completion code
  MQLONG  Reason;                      // stores return codes

  keyBefore = GetProtectKey();                                                  //d26725

  MQPUT(Hconn, Hobj[H - 1], &md, &pmo, strlen(StringIn) + 1, StringIn,
        &CompCode, &Reason);           // strlen +1 for end of string marker

  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output,"protect key in MQPUT() %u different than expected %u",    //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725

  CheckError("MQPUT", Reason);
}


// Put Message of certain length

void Put(int BytesToPut, char MsgToPut, int H)
{
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725
  char output[100];

  sprintf(output, "MQPUT(%d bytes of '%c') to %s, handle %d",
                  BytesToPut,
                  MsgToPut,
                  TrgtQName[(H - 1) % queues], H);

  if (HelpCode(output)) return;

  MQLONG  CompCode;                    // stores completion code
  MQLONG  Reason;                      // stores return codes
  char *Message = (char *) malloc(BytesToPut);  // make space available

  if (Message == NULL && BytesToPut)   // Message NULL anyway if BytesToPut == 0
    CaseError("MALLOC space not available");

  memset(Message, MsgToPut, BytesToPut);  // Write simple message

  keyBefore = GetProtectKey();                                                  //d26725
  MQPUT(Hconn, Hobj[H - 1], &md, &pmo, BytesToPut, Message,
        &CompCode, &Reason);

  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output,"protect key in MQPUT() %u different than expected %u",    //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725

  CheckError("MQPUT", Reason);
  free(Message);
}

void Put(MQLONG CodeIn, int BytesToPut, char MsgToPut, int H)
{
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725
  char output[100];

  sprintf(output, "MQPUT(%d bytes of '%c', look for %s) to %s, handle %d",
                  BytesToPut,
                  MsgToPut,
                  MQRC_String(CodeIn),
                  TrgtQName[(H - 1) % queues], H);

  if (HelpCode(output)) return;

  MQLONG  CompCode;                    // stores completion code
  MQLONG  Reason;                      // stores return codes
  char *Message = (char *) malloc(BytesToPut);  // make space available

  if (Message == NULL && BytesToPut)   // Message NULL anyway if BytesToPut == 0
    CaseError("MALLOC space not available");

  memset(Message, MsgToPut, BytesToPut);  // Write simple message

  keyBefore = GetProtectKey();                                                  //d26725

  MQPUT(Hconn, Hobj[H - 1], &md, &pmo, BytesToPut, Message,
        &CompCode, &Reason);

  if (Reason != CodeIn)
    CheckError("MQPUT", Reason);

  if (Reason == MQRC_NONE && CodeIn != MQRC_NONE)
    Error("Error: Reason = MQRC_NONE");

  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output,"protect key in MQPUT() %u different than expected %u",    //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725

  free(Message);
}

// Commented out since not used to remove compiler warning
#if 0
// Inquire for an integer value.

static void Inq(MQLONG attr, long *pValue, int H)
{
  MQLONG CompCode;                    // stores completion code
  MQLONG Reason;                      // stores return codes
  MQLONG selector[1] = {attr};
  MQLONG intRet[1];
  unsigned char keyBefore = GetProtectKey();                                    //d26725
  unsigned char keyAfter;                                                       //d26725
  char   output[100];                                                           //d26725

  MQINQ(Hconn, Hobj[H - 1],
        1, selector,
        1, intRet,
        0, NULL,
        &CompCode, &Reason);

  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output,"protect key in MQINQ() %u different from expected %u",    //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725
  CheckError("MQINQ", Reason);

  *pValue = intRet[0];
}
#endif


// Connect

static void Conn()
{
  MQLONG CompCode, Reason;
  char   output[100];                                                           //d26725
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725

  if (HelpCode("MQCONN")) return;

  keyBefore = GetProtectKey();                                                  //d26725

  MQCONN("", &Hconn, &CompCode, &Reason);

  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output,"protect key in MQCONN() %u different from expected %u",   //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725
  CheckError("MQCONN", Reason);
}


// Disconnect

static void Disc()
{
  MQLONG CompCode, Reason;
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725

  if (HelpCode("MQDISC")) return;

  keyBefore = GetProtectKey();                                                  //d26725

  MQDISC(&Hconn, &CompCode, &Reason);

  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output,"protect key in MQDISC() %u different from expected %u",   //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725
  CheckError("MQDISC", Reason);
}


// Open an Hobj.

static void Open(int H, int Opt)
{
  MQOD   ObjDesc = {MQOD_DEFAULT};    // Object Descriptor
  MQLONG CompCode, Reason;
  char   Output[100];
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725

  sprintf(Output, "MQOPEN(%s, handle %d)",
                  TrgtQName[(H - 1) % queues], H);
  if (HelpCode(Output)) return;

  // Set queue name.
  strcpy(ObjDesc.ObjectName, TrgtQName[(H - 1) % queues]);

  keyBefore = GetProtectKey();                                                  //d26725

  MQOPEN(Hconn, &ObjDesc, CaseTable[CasePos].openOptions[Opt],
         &Hobj[H - 1], &CompCode, &Reason);      // Open queue i

  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(Output,"protect key in MQOPEN() %u different from expected %u",   //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(Output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725
  CheckError("MQOPEN", Reason);
}


// Close an Hobj.

static void Close(int H)
{
  char   Output[100];
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725

  sprintf(Output, "MQCLOSE(%s, handle %d)",
                  TrgtQName[(H - 1) % queues], H);
  if (HelpCode(Output)) return;

  MQLONG CompCode, Reason;

  keyBefore = GetProtectKey();                                                  //d26725

  MQCLOSE(Hconn, &Hobj[H - 1], 0, &CompCode, &Reason);

  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(Output,"protect key in MQCLOSE() %u different from expected %u",  //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(Output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725
  CheckError("MQCLOSE", Reason);
}


// Commented out to remove warning from compiler

#if 0
// ReOpen (close, then open) an Hobj.

static void ReOpen(int H, int Opt)
{
  Close(H);
  Open(H, Opt);
}
#endif


// Get msg of certain length and verify

void Get(int BytesToGet, char MsgToGet, int H)
{
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725
  char output[100];

  sprintf(output, "MQGET(%d bytes of '%c') from %s, handle %d",
                  BytesToGet,
                  MsgToGet,
                  TrgtQName[(H - 1) % queues], H);

  if (HelpCode(output)) return;

  MQLONG  msgLen;                      // Msg length in bytes
  MQLONG  CompCode;                    // stores completion code
  MQLONG  Reason;                      // stores return codes
  char *Message = (char *) malloc(BytesToGet); // make space available

  if (Message == NULL && BytesToGet)   // Message == NULL if BytesToPut == 0
    CaseError("MALLOC space not available");

  keyBefore = GetProtectKey();                                                  //d26725

  MQGET(Hconn, Hobj[H - 1], &md, &gmo, BytesToGet,
        (MQBYTE *) Message, &msgLen, &CompCode, &Reason);  // get msg
  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output,"protect key in MQGET() %u different from expected %u",    //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725

  CheckError("MQGET", Reason);              // check for error

  if (msgLen != BytesToGet)                 // check correct length
  {
    sprintf(output,"Message Length %d different from expected %d",
            msgLen, BytesToGet);
    CaseError(output);
  }

  for (int i = 0; i < BytesToGet; i++) // check correct message
  {
    if (Message[i] != MsgToGet)
      CaseError("Incorrect Message Text Received");
  }

  free(Message);
}


// Get and compare strings

void Get(char *StringIn, int H)
{
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725
  char output2[100];

  sprintf(output2, "Get(\"%s\") from %s, handle %d",
                   StringIn,
                   TrgtQName[(H - 1) % queues], H);

  if (HelpCode(output2)) return;

  MQLONG  msgLen;                      // Msg length in bytes
  MQLONG  CompCode;                    // stores completion code
  MQLONG  Reason;                      // stores return codes
  char   *GetString;                   // Receiving the string

  GetString = (char *) malloc(strlen(StringIn) + 1);         // make space available
  keyBefore = GetProtectKey();                                                  //d26725

  MQGET(Hconn, Hobj[H - 1], &md, &gmo, strlen(StringIn) + 1, // get the message,
        (MQBYTE *) GetString, &msgLen, &CompCode, &Reason);  //   incl eos marker

  if (Reason == MQRC_NONE) {                                                    //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output2,"protect key in MQGET() %u different than expected %u",   //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output2);                                                       //d26725
    }                                                                           //d26725
  }                                                                             //d26725

  CheckError("MQGET", Reason);                               // check for error

  if (strcmp(StringIn, GetString) != 0)                      // Compare strings
  {
    sprintf(output2, "Unknown Message Received: %s", GetString);
    CaseError(output2);
  }

  free(GetString);
}


// Get and look at error code

void Get(MQLONG CodeIn, int H, int BytesToGet)
{
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725
  char   output[DEFAULT_MSG_BUFFER];
  MQLONG msgLen;                       // Msg length in bytes
  MQLONG CompCode;                     // stores completion code
  MQLONG Reason;                       // stores return codes

  if (CodeIn == MQRC_NO_MSG_AVAILABLE)
  {
    sprintf(output, "MQGET(NOMSG) from %s, handle %d",
                    TrgtQName[(H - 1) % queues], H);
  }
  else
  {
    sprintf(output, "MQGET(look for %s) from %s, handle %d",
                    MQRC_String(CodeIn),
                    TrgtQName[(H - 1) % queues], H);
  }

  if (HelpCode(output)) return;

  char *GetString = (char *) malloc(BytesToGet);

  if (!GetString && BytesToGet)
    CaseError("no malloc space");

  keyBefore = GetProtectKey();                                                  //d26725

  MQGET(Hconn, Hobj[H - 1], &md, &gmo, BytesToGet, // Get a message
        GetString, &msgLen, &CompCode, &Reason);

  if (Reason == CodeIn) {                                                       //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output,"protect key in MQGET() %u different than expected %u",    //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    CheckError("MQGET", Reason);                                                //d26725
  }                                                                             //d26725

  if (Reason == MQRC_NONE && CodeIn != MQRC_NONE)
    Error("Error: Reason = MQRC_NONE");

  free(GetString);                                                              //d26725
}


// Get, look for err code

void Get(MQLONG CodeIn, int BytesToGet, char CharToGet, int H)
{
  unsigned char keyBefore;                                                      //d26725
  unsigned char keyAfter;                                                       //d26725
  MQLONG msgLen;                       // Msg length in bytes
  MQLONG CompCode;                     // stores completion code
  MQLONG Reason;                       // stores return codes
  char   output[200];                                                           //d26725

  if (CharToGet != '\0')
  {
    sprintf(output,
            "MQGET(%d bytes, look for %s) from %s, handle %d,"
            " check message",
            BytesToGet,
            MQRC_String(CodeIn),
            TrgtQName[(H - 1) % queues], H);
  }
  else
  {
    sprintf(output, "MQGET(%d bytes, look for %s) from %s, handle %d",
                    BytesToGet,
                    MQRC_String(CodeIn),
                    TrgtQName[(H - 1) % queues], H);
  }

  if (HelpCode(output)) return;

  char *GetString = (char *) malloc(BytesToGet);

  if (!GetString && BytesToGet)
    CaseError("no malloc space");

  keyBefore = GetProtectKey();                                                  //d26725

  MQGET(Hconn, Hobj[H - 1], &md, &gmo, BytesToGet, // Get a message
        GetString, &msgLen, &CompCode, &Reason);
  if (Reason == CodeIn) {                                                       //d26725                                                                         //d26725
    //validate protection key did not change
    keyAfter = GetProtectKey();                                                 //d26725
    if (keyAfter != keyBefore) {                                                //d26725
      sprintf(output,"protect key in MQGET() %u different than expected %u",    //d26725
              keyAfter, keyBefore);                                             //d26725
      CaseError(output);                                                        //d26725
    }                                                                           //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    CheckError("MQGET", Reason);                                                //d26725
  }                                                                             //d26725

  if (Reason == MQRC_NONE && CodeIn != MQRC_NONE)
    Error("Error: Reason = MQRC_NONE");

  if (Reason == MQRC_NONE && CharToGet != '\0')
  {
    for (int i = 0; i < BytesToGet; i++)
      if (GetString[i] != CharToGet) CaseError("Incorrect Message");
  }

  free(GetString);                                                              //d26725
}


// Put a messge on each queue

void PutOneOnEach(int MessageStart)
{
  char Message[40];                                                             //d26352

  for (int QueueNum = 1; QueueNum <= queues; QueueNum++)
  {
    sprintf(Message, "Msg %d on Queue %d", MessageStart + QueueNum - 1,
                                           QueueNum);
    Put(Message, QueueNum);
  }
}


void GetOneFromEach(int MessageStart)  // Get a message from each queue
{
  if (MessageStart == NOMSG)
  {
    for (int QueueNum = 1; QueueNum <= queues; QueueNum++)
      Get(NOMSG, QueueNum);

    return;
  }

  char Message[20];

  for (int QueueNum = queues; QueueNum >= 1; QueueNum--)   // Reversed
  {
    sprintf(Message, "Msg %d on Queue %d", MessageStart + QueueNum - 1,
                                           QueueNum);
    Get(Message, QueueNum);
  }
}


void TX_Begin()                        // tx_begin() with error check
{
  if (HelpCode("TX_Begin();"))
    return;

  if (HelpVar > 0)
    HelpVar++;

  int TX_ErrorCode = tx_begin();

  CheckError("tx_begin", TX_ErrorCode);
}


void TX_Rollback()                     // tx_rollback() with error check
{
  if (HelpVar > 1)
    HelpVar--;

  if (HelpCode("TX_Rollback();")) return;

  int TX_ErrorCode = tx_rollback();

  CheckError("tx_rollback", TX_ErrorCode);
}


void TX_Commit()                       // tx_commit() with error check
{
  if (HelpVar > 1)
    HelpVar--;

  if (HelpCode("TX_Commit();")) return;

  int TX_ErrorCode = tx_commit();

  CheckError("tx_commit", TX_ErrorCode);
}


void FileSomething(char CharToPut)     // DASD Transaction
{
  unsigned int PoolAddress;            // File pool address
  char *Record;                        // Record read from disk
  char BlockType[2] = {0x00, 0xE6};    // Short term 4KB record
  // Header record ID and program name
  char Header[8] = {0x00, 0xE6, 0x00, 0x00, 0xD8, 0xD4, 0xD8, 0xE9};
  int i;

  if (MQOnly) return;                  // User wants to skip DASD transactions
  if (HelpCode("FileSomething()")) return;

  if (levtest(D4)) relcc(D4);          // Free a storage block

  // Get a short term 4KB record, fill it with CharToPut, and file it out again.

  PoolAddress = getfc(D4, GETFC_TYPE0, BlockType,
                GETFC_BLOCK|GETFC_FILL, GETFC_NOSERRC, CharToPut);
  if (!PoolAddress) CaseError("QMQZ06: getfc() Error");
  memcpy(ecbptr()->ce1cr4, Header, 8);  // Copy header to core block

  // File core block and release it.
  // No error return on file_record
  file_record(D4, &PoolAddress, BlockType, '\0', NOHOLD);

  // Read in the 4KB record
  Record = (char *) find_record(D4, &PoolAddress, BlockType, '\0', NOHOLD);
  if (!Record) CaseError("QMQZ06: find_record() Error");  // Got a NULL pointer

  for (i = 0; i < 8; i++)
  {
    if (Record[i] != Header[i])        // verify content by testing each character
      CaseError("Incorrect Record Data");  // Test for correct header
  }

  for (; i < 4095; i++)
  {
    if (Record[i] != CharToPut)            // Verify correct message
      CaseError("Incorrect Record Data");  // by looking for filled character
  }

  relfc(D4);                               // release the DASD pool
  relcc(D4);                               // release the core block
  Trace("FileSomething");
}


/*********************************************************************/
/*                        MQS1 Regression test cases                 */
/*********************************************************************/

static void Case1()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   Get("Message 1");
  TX_Commit();
}

static void Case2()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    Get("Message 1");
   TX_Commit();
   Get(NOMSG);
  TX_Commit();
}

static void Case3()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    Get("Message 1");
   TX_Rollback();
   Get("Message 1");
  TX_Commit();
}

static void Case4()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    Put("Message 2");
    Get("Message 1");
   TX_Commit();
   Get("Message 2");
  TX_Commit();
}

static void Case5()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    Put("Message 2");
    Get("Message 1");
   TX_Rollback();
   Get("Message 1");
  TX_Commit();
}

static void Case6()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    Put("Message 2");
   TX_Commit();
   Get("Message 1");
   Get("Message 2");
  TX_Commit();
}

static void Case7()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    Put("Message 2");
   TX_Rollback();
   Get("Message 1");
   Get(NOMSG);
  TX_Commit();
}

static void Case8()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    Get("Message 1");
   TX_Commit();
  TX_Commit();
}

static void Case9()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    TX_Begin();
     FileSomething('C');
     Put("Message 2");
    TX_Commit();
    Get("Message 1");
    Get("Message 2");
   TX_Commit();
  TX_Commit();
}

static void Case10()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    TX_Begin();
     FileSomething('C');
     Put("Message 2");
    TX_Rollback();
    Get("Message 1");
    Get(NOMSG);
   TX_Commit();
  TX_Commit();
}

static void Case11()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    TX_Begin();
     FileSomething('C');
     Get("Message 1");
    TX_Rollback();
    Get("Message 1");
    Get(NOMSG);
   TX_Commit();
  TX_Commit();
}

static void Case12()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
   TX_Begin();
    FileSomething('B');
    TX_Begin();
     FileSomething('C');
     Get("Message 1");
    TX_Commit();
    Get(NOMSG);
   TX_Commit();
  TX_Commit();
}

static void Case13()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
    TX_Begin();
     FileSomething('C');
     Put("Message 2");
    TX_Commit();
    Get("Message 1");
    Get("Message 2");
   TX_Commit();
  TX_Commit();
}

static void Case14()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
    TX_Begin();
     FileSomething('C');
     Put("Message 2");
    TX_Rollback();
    Get("Message 1");
    Get(NOMSG);
   TX_Commit();
  TX_Commit();
}

static void Case15()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
    TX_Begin();
     FileSomething('C');
     Get("Message 1");
    TX_Commit();
    Get(NOMSG);
   TX_Commit();
  TX_Commit();
}

static void Case16()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
    TX_Begin();
     FileSomething('C');
     Get("Message 1");
    TX_Rollback();
    Get("Message 1");
   TX_Commit();
  TX_Commit();
}

static void Case17()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Commit();
   Get("Message 1");
  TX_Commit();
}

static void Case18()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Rollback();
   Get(NOMSG);
  TX_Commit();
}

static void Case19()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Commit();
   Put("Message 2");
   Get("Message 1");
   Get("Message 2");
  TX_Commit();
}

static void Case20()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Rollback();
   Put("Message 2");
   Get("Message 2");
  TX_Commit();
}

static void Case21()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   Get("Message 1");
  TX_Commit();
}

static void Case22()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 2");
   TX_Commit();
   Get("Message 1");
   Get("Message 2");
  TX_Commit();
}

static void Case23()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 2");
   TX_Rollback();
   Get("Message 1");
   Get(NOMSG);
  TX_Commit();
}

static void Case24()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Get("Message 1");
   TX_Rollback();
   Get("Message 1");
   Get(NOMSG);
  TX_Commit();
}

static void Case25()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Get("Message 1");
   TX_Commit();
   Get(NOMSG);
  TX_Commit();
}

static void Case26()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Get("Message 1");
   TX_Commit();
  TX_Commit();
}

static void Case27()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   Put("Message 2");
   TX_Begin();
    FileSomething('B');
    Get("Message 1");
    Get("Message 2");
   TX_Commit();
  TX_Commit();
}

static void Case28()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
  TX_Commit();
  Get("Message 1");
}

static void Case29()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
  TX_Rollback();
  Get(NOMSG);
}

static void Case30()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
  TX_Commit();
  Put("Message 2");
  Get("Message 1");
  Get("Message 2");
}

static void Case31()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
  TX_Rollback();
  Put("Message 2");
  Get("Message 2");
}

static void Case32()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Commit();
  TX_Commit();
  Get("Message 1");
}

static void Case33()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Commit();
  TX_Rollback();
  Get(NOMSG);
}

static void Case34()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Rollback();
  TX_Commit();
  Get(NOMSG);
}

static void Case35()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Rollback();
  TX_Rollback();
  Get(NOMSG);
}

static void Case36()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Commit();
   Put("Message 2");
  TX_Commit();
  Get("Message 1");
  Get("Message 2");
}

static void Case37()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Commit();
   Put("Message 2");
  TX_Rollback();
  Get(NOMSG);
}

static void Case38()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Rollback();
   Put("Message 2");
  TX_Commit();
  Get("Message 2");
  Get(NOMSG);
}

static void Case39()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    Put("Message 1");
   TX_Rollback();
   Put("Message 2");
  TX_Rollback();
  Get(NOMSG);
}

static void Case40()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   Get("Message 1");
  TX_Commit();
  Get(NOMSG);
}

static void Case41()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   Get("Message 1");
  TX_Rollback();
  Get("Message 1");
}

static void Case42()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   Put("Message 2");
  TX_Commit();
  Get("Message 1");
  Get("Message 2");
}

static void Case43()
{
  Put("Message 1");
  TX_Begin();
   FileSomething('A');
   Put("Message 2");
  TX_Rollback();
  Get("Message 1");
  Get(NOMSG);
}

static void Case44()
{
  Put(0, 'X');
  Get(0, 'X');
}

static void Case45()
{
  Put(1, 'A');
  Get(1, 'A');
}

static void Case46()
{
  Put(FirstMsgSegmentSize, 'B');
  Get(FirstMsgSegmentSize, 'B');
}

static void Case47()
{
  Put(FirstMsgSegmentSize + 1, 'C');
  Get(FirstMsgSegmentSize + 1, 'C');
}

static void Case48()
{
  Put(FirstMsgSegmentSize + NextMsgSegmentSize, 'D');
  Get(FirstMsgSegmentSize + NextMsgSegmentSize, 'D');
}

static void Case49()
{
  Put(FirstMsgSegmentSize + NextMsgSegmentSize + 1, 'E');
  Get(FirstMsgSegmentSize + NextMsgSegmentSize + 1, 'E');
}

static void Case50()
{
  Put(FirstMsgSegmentSize + NextMsgSegmentSize + NextMsgSegmentSize, 'F');
  Get(FirstMsgSegmentSize + NextMsgSegmentSize + NextMsgSegmentSize, 'F');
}

// For this case, the queue handles are not explicitly closed.
// They should close automatically when the ECB exits.

static void Case51()
{
  PutOneOnEach(1);
  PutOneOnEach(4);
  GetOneFromEach(1);
  GetOneFromEach(4);
}

static void Case52()
{
  TX_Begin();
   FileSomething('A');
   PutOneOnEach(1);
   PutOneOnEach(4);
   GetOneFromEach(1);
   GetOneFromEach(4);
  TX_Commit();
}

static void Case53()
{
  TX_Begin();
   FileSomething('A');
   PutOneOnEach(1);
   PutOneOnEach(4);
  TX_Rollback();
  GetOneFromEach(NOMSG);
}

static void Case54()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    PutOneOnEach(1);
   TX_Commit();
   PutOneOnEach(4);
  TX_Commit();
  GetOneFromEach(1);
  GetOneFromEach(4);
}

static void Case55()
{
  TX_Begin();
   FileSomething('A');
   TX_Begin();
    FileSomething('B');
    PutOneOnEach(1);
   TX_Rollback();
   PutOneOnEach(4);
  TX_Commit();
  GetOneFromEach(4);
}

static void Case56()
{
  TX_Begin();
   FileSomething('A');
   PutOneOnEach(1);
   TX_Begin();
    FileSomething('B');

    for (int i = 4; i <= 13; i +=3)
      PutOneOnEach(i);

    GetOneFromEach(1);
    GetOneFromEach(4);
   TX_Rollback();
   PutOneOnEach(16);
  TX_Commit();
  GetOneFromEach(1);
  GetOneFromEach(16);
  GetOneFromEach(NOMSG);
}

static void Case57()
{
  Put(40000, 'A');
  Get(40000, 'A');
}

static void Case58()
{
  TX_Begin();
   FileSomething('A');
   Put(40000, 'B');
   Get(40000, 'B');
  TX_Commit();
}

static void Case59()
{
  TX_Begin();
   FileSomething('A');
   Put(40000, 'C');
  TX_Commit();
  Get(40000, 'C');
}

static void Case60()
{
  TX_Begin();
   FileSomething('A');
   Put(40000, 'D');
  TX_Rollback();
  Get(NOMSG);
}

static void Case61()
{
  Put(40000, 'E');
  TX_Begin();
   FileSomething('A');
   Get(40000, 'E');
  TX_Commit();
}

static void Case62()
{
  TX_Begin();
   FileSomething('A');
   Put(40000, 'F');
   Put(0, 'X');
  TX_Commit();
  Get(40000, 'F');
  Get(0, 'Y');
}

static void Case63()
{
  TX_Begin();
   FileSomething('A');
   Put(1048576, 'G');
  TX_Commit();
  Get(1048576, 'G');
}

static void Case64()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
  TX_Commit();
  TX_Begin();
   FileSomething('B');
   Get("Message 1");
  TX_Commit();
}

static void Case65()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
  TX_Commit();
  TX_Begin();
   FileSomething('B');
   Get("Message 1");
  TX_Rollback();
  Get("Message 1");
}

static void Case66()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
  TX_Commit();
  TX_Begin();
   FileSomething('B');
   Put("Message 2");
   Get("Message 1");
  TX_Commit();
  TX_Begin();
   FileSomething('C');
   Get("Message 2");
  TX_Commit();
}

static void Case67()
{
  TX_Begin();
   FileSomething('A');
   Put("Message 1");
  TX_Commit();
  TX_Begin();
   FileSomething('B');
   Put("Message 2");
   Get("Message 1");
  TX_Rollback();
  TX_Begin();
   FileSomething('C');
   Get("Message 1");
  TX_Commit();
}

static void Case68()
{
  Put(10, 'A');
  HelpCode("MQGET Options = MQGMO_NONE");
  gmo.Options = MQGMO_NONE;
  Get(MQRC_TRUNCATED_MSG_FAILED, 0, '\0');
  HelpCode("MQGET Options = MQGMO_ACCEPT_TRUNCATED_MSG");
  gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
  Get(MQRC_TRUNCATED_MSG_ACCEPTED, 0, '\0');
}

static void Case69()
{
  TX_Begin();
   FileSomething('A');
   Put(100, 'A');
   HelpCode("Get Options = MQGMO_ACCEPT_TRUNCATED_MSG");
   gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
   Get(MQRC_TRUNCATED_MSG_ACCEPTED, 50, 'A');
  TX_Commit();
}

static void Case70()
{
  TX_Begin();
   FileSomething('A');
   Put(100, 'B');
   HelpCode("Get Options = MQGMO_ACCEPT_TRUNCATED_MSG");
   gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
   Get(MQRC_TRUNCATED_MSG_ACCEPTED, 50, 'B');
  TX_Rollback();
}

static void Case71()
{
  TX_Begin();
   FileSomething('A');
   Put(100, 'C');
   HelpCode("Get Options = MQGMO_NONE");
   gmo.Options = MQGMO_NONE;
   Get(MQRC_TRUNCATED_MSG_FAILED, 50, '\0');
  TX_Commit();
  HelpCode("Get Options = MQGMO_ACCEPT_TRUNCATED_MSG");
  gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
  Get(MQRC_TRUNCATED_MSG_ACCEPTED, 50, 'C');
}

static void Case72()
{
  TX_Begin();
   FileSomething('A');
   Put(100, 'C');
   HelpCode("Get Options = MQGMO_NONE");
   gmo.Options = MQGMO_NONE;
   Get(MQRC_TRUNCATED_MSG_FAILED, 50, '\0');
  TX_Rollback();
  HelpCode("Get Options = MQGMO_ACCEPT_TRUNCATED_MSG");
  gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
}

static void Case73()
{
  Put("Message 1");
  Put("Message 2");
  Put("Message 3");
  TX_Begin();
   Get("Message 1");
   Put("Message 4");
   Get("Message 2");
   TX_Begin();
    TX_Begin();
     Get("Message 3");
    TX_Rollback();
    Put("Message 5");
    Get("Message 3");
    Get("Message 4");
    Put("Message 6");
    Get("Message 5");
    Put("Message 7");
    Put("Message 8");
    Get("Message 6");
   TX_Commit();
   Put("Message 9");
   Get("Message 7");
  TX_Commit();
  Get("Message 8");
  Get("Message 9");
}

/*----------------------------------------------------------------------
   ==============
   =Test Case 74=
   ==============
    A truncated message is rejected due to insufficient buffer size,
    then reread with the correct buffer size.  Verify that the message
    is read correctly.
----------------------------------------------------------------------*/
static void Case74()
{
  TX_Begin();
   Put(100, 'A');
  TX_Commit();
  gmo.Options = MQGMO_NONE;
  Get(MQRC_TRUNCATED_MSG_FAILED, 50, 'A');
  Get(MQRC_NONE, 100, 'A');
}

static void Case75() {}

/*********************************************************************/
/* Test cases from Function Test Plan.                               */
/*********************************************************************/

/*----------------------------------------------------------------------
   ===============
   =Test Case 101=
   ===============
    Test if MQGMO_WAIT option can be used in conjunction with other
    MQGMO get message options.

    Expected Result: See optionList
----------------------------------------------------------------------*/
static void Case101()
{
  struct sOptionList
  {
    MQLONG option;                          // option
    MQLONG rc;                              // expected result
  } optionList[] =
       {{ MQGMO_SYNCPOINT,                MQRC_OPTIONS_ERROR       },
        { MQGMO_SYNCPOINT_IF_PERSISTENT,  MQRC_OPTIONS_ERROR       },
        { MQGMO_NO_SYNCPOINT,             MQRC_OPTIONS_ERROR       },
        { MQGMO_MARK_SKIP_BACKOUT,        MQRC_OPTIONS_ERROR       },
        { MQGMO_BROWSE_FIRST,             MQRC_NO_MSG_AVAILABLE    },
        { MQGMO_BROWSE_NEXT,              MQRC_NO_MSG_AVAILABLE    },
        { MQGMO_BROWSE_MSG_UNDER_CURSOR,  MQRC_NO_MSG_UNDER_CURSOR },
        { MQGMO_MSG_UNDER_CURSOR,         MQRC_NO_MSG_UNDER_CURSOR },
        { MQGMO_LOCK,                     MQRC_OPTIONS_ERROR       },
        { MQGMO_UNLOCK,                   MQRC_OPTIONS_ERROR       },
        { MQGMO_ACCEPT_TRUNCATED_MSG,     MQRC_NO_MSG_AVAILABLE    },
        { MQGMO_SET_SIGNAL,               MQRC_OPTIONS_ERROR       },
        { MQGMO_FAIL_IF_QUIESCING,        MQRC_OPTIONS_ERROR       },
        { MQGMO_CONVERT,                  MQRC_OPTIONS_ERROR       },
        { MQGMO_LOGICAL_ORDER,            MQRC_OPTIONS_ERROR       },
        { MQGMO_COMPLETE_MSG,             MQRC_OPTIONS_ERROR       },
        { MQGMO_ALL_MSGS_AVAILABLE,       MQRC_OPTIONS_ERROR       },
        { MQGMO_ALL_SEGMENTS_AVAILABLE,   MQRC_OPTIONS_ERROR       }
       };

  const int nbrOptions = sizeof(optionList) / sizeof(sOptionList);

  for (int n = 0; n < nbrOptions; n++)
  {
    gmo.Options = MQGMO_WAIT + optionList[n].option;
    Get(optionList[n].rc);
  }
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 102=
   ===============
    Test getting a message using MQGMO_NONE option indicating no other
    options have been specified.  Its value is zero.  It is used when
    none of the other options are required.

    Expected Result: MQRC_NONE
----------------------------------------------------------------------*/
static void Case102()
{
  gmo.Options = MQGMO_NONE;

  Put("Test Case 102");
  Get("Test Case 102");
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 103=
   ===============
    Test specifying unlimited wait interval constant MQWI_UNLIMITED in
    the WaitInterval field.

    Expected Result: MQRC_NONE
----------------------------------------------------------------------*/
static void Case103()
{
  gmo.Options = MQGMO_WAIT;
  gmo.WaitInterval = MQWI_UNLIMITED;

  Put("Test Case 103");
  Get("Test Case 103");
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 104=
   ===============
    Test using an invalid wait interval value.  Use different invalid
    values like -9, -12, -5.

    Expected Result: MQRC_WAIT_INTERVAL_ERROR
----------------------------------------------------------------------*/
static void Case104()
{
  MQLONG waitIntervals[] = { -3, -9, -12, -5 };
  int    nbrOptions = sizeof(waitIntervals) / sizeof(MQLONG);

  gmo.Options = MQGMO_WAIT;

  for (int n = 0; n < nbrOptions; n++)
  {
    gmo.WaitInterval = waitIntervals[n];
    Get(MQRC_WAIT_INTERVAL_ERROR);
  }
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 200=
   ===============
    Test using various invalid browse options or combinations
    when the queue is opened for browse only.
----------------------------------------------------------------------*/
static void Case200()
{
  struct sOptionList
  {
    MQLONG option;                          // option
    MQLONG rc;                              // expected result
  };

  struct sOptionList optionList[] =
  {{ MQGMO_NONE,                                             MQRC_NOT_OPEN_FOR_INPUT  },
   { MQGMO_MSG_UNDER_CURSOR,                                 MQRC_NOT_OPEN_FOR_INPUT  },
   { MQGMO_LOCK | MQGMO_UNLOCK | MQGMO_BROWSE_NEXT,          MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_FIRST | MQGMO_BROWSE_NEXT,                 MQRC_OPTIONS_ERROR       },
   { MQGMO_LOCK,                                             MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_WAIT,                              MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_BROWSE_FIRST,                      MQRC_OPTIONS_ERROR       },
   { MQGMO_LOCK | MQGMO_ACCEPT_TRUNCATED_MSG,                MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_FIRST,                                     MQRC_NO_MSG_AVAILABLE    },
   { MQGMO_BROWSE_NEXT,                                      MQRC_NO_MSG_AVAILABLE    },
   { MQGMO_BROWSE_MSG_UNDER_CURSOR,                          MQRC_NO_MSG_UNDER_CURSOR },
   { MQGMO_LOCK | MQGMO_UNLOCK,                              MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_MSG_UNDER_CURSOR | MQGMO_MSG_UNDER_CURSOR, MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_MSG_UNDER_CURSOR,                  MQRC_OPTIONS_ERROR       }
  };

  const int nbrOptions = sizeof(optionList) / sizeof(sOptionList);

  for (int n = 0; n < nbrOptions; n++)
  {
    gmo.Options = optionList[n].option;
    Get(optionList[n].rc);
  }
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 201=
   ===============
    Test using various invalid browse options or combinations
    when the queue is opened for browse and input.
----------------------------------------------------------------------*/
static void Case201()
{
  struct sOptionList
  {
    MQLONG option;                          // option
    MQLONG rc;                              // expected result
  };

  struct sOptionList optionList[] =
  {{ 0,                                                      MQRC_NO_MSG_AVAILABLE    },
   { MQGMO_MSG_UNDER_CURSOR,                                 MQRC_NO_MSG_UNDER_CURSOR },
   { MQGMO_LOCK | MQGMO_UNLOCK | MQGMO_BROWSE_NEXT,          MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_FIRST | MQGMO_BROWSE_NEXT,                 MQRC_OPTIONS_ERROR       },
   { MQGMO_LOCK,                                             MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_WAIT,                              MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_BROWSE_FIRST,                      MQRC_OPTIONS_ERROR       },
   { MQGMO_LOCK | MQGMO_ACCEPT_TRUNCATED_MSG,                MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_FIRST,                                     MQRC_NO_MSG_AVAILABLE    },
   { MQGMO_BROWSE_NEXT,                                      MQRC_NO_MSG_AVAILABLE    },
   { MQGMO_BROWSE_MSG_UNDER_CURSOR,                          MQRC_NO_MSG_UNDER_CURSOR },
   { MQGMO_LOCK | MQGMO_UNLOCK,                              MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_MSG_UNDER_CURSOR | MQGMO_MSG_UNDER_CURSOR, MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_MSG_UNDER_CURSOR,                  MQRC_OPTIONS_ERROR       }
  };

  const int nbrOptions = sizeof(optionList) / sizeof(sOptionList);

  for (int n = 0; n < nbrOptions; n++)
  {
    gmo.Options = optionList[n].option;
    Get(optionList[n].rc);
  }
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 202=
   ===============
    Test using various browse options or combinations
    when the queue is opened for output only.
----------------------------------------------------------------------*/
static void Case202()
{
  struct sOptionList
  {
    MQLONG option;                          // option
    MQLONG rc;                              // expected result
  };

  struct sOptionList optionList[] =
  {{ 0,                                                      MQRC_NOT_OPEN_FOR_INPUT  },
   { MQGMO_MSG_UNDER_CURSOR,                                 MQRC_NOT_OPEN_FOR_BROWSE },
   { MQGMO_LOCK | MQGMO_UNLOCK | MQGMO_BROWSE_NEXT,          MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_FIRST | MQGMO_BROWSE_NEXT,                 MQRC_OPTIONS_ERROR       },
   { MQGMO_LOCK,                                             MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_WAIT,                              MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_BROWSE_FIRST,                      MQRC_OPTIONS_ERROR       },
   { MQGMO_LOCK | MQGMO_ACCEPT_TRUNCATED_MSG,                MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_FIRST,                                     MQRC_NOT_OPEN_FOR_BROWSE },
   { MQGMO_BROWSE_NEXT,                                      MQRC_NOT_OPEN_FOR_BROWSE },
   { MQGMO_BROWSE_MSG_UNDER_CURSOR,                          MQRC_NOT_OPEN_FOR_BROWSE },
   { MQGMO_LOCK | MQGMO_UNLOCK,                              MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_MSG_UNDER_CURSOR | MQGMO_MSG_UNDER_CURSOR, MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_MSG_UNDER_CURSOR,                  MQRC_OPTIONS_ERROR       }
  };

  const int nbrOptions = sizeof(optionList) / sizeof(sOptionList);

  for (int n = 0; n < nbrOptions; n++)
  {
    gmo.Options = optionList[n].option;
    Get(optionList[n].rc);
  }
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 203=
   ===============
    Test using various browse options or combinations
    when the queue is opened for input only.
----------------------------------------------------------------------*/
static void Case203()
{
  struct sOptionList
  {
    MQLONG option;                          // option
    MQLONG rc;                              // expected result
  };

  struct sOptionList optionList[] =
  {{ 0,                                                      MQRC_NO_MSG_AVAILABLE    },
   { MQGMO_MSG_UNDER_CURSOR,                                 MQRC_NOT_OPEN_FOR_BROWSE },
   { MQGMO_LOCK | MQGMO_UNLOCK | MQGMO_BROWSE_NEXT,          MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_FIRST | MQGMO_BROWSE_NEXT,                 MQRC_OPTIONS_ERROR       },
   { MQGMO_LOCK,                                             MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_WAIT,                              MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_BROWSE_FIRST,                      MQRC_OPTIONS_ERROR       },
   { MQGMO_LOCK | MQGMO_ACCEPT_TRUNCATED_MSG,                MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_FIRST,                                     MQRC_NOT_OPEN_FOR_BROWSE },
   { MQGMO_BROWSE_NEXT,                                      MQRC_NOT_OPEN_FOR_BROWSE },
   { MQGMO_BROWSE_MSG_UNDER_CURSOR,                          MQRC_NOT_OPEN_FOR_BROWSE },
   { MQGMO_LOCK | MQGMO_UNLOCK,                              MQRC_OPTIONS_ERROR       },
   { MQGMO_BROWSE_MSG_UNDER_CURSOR | MQGMO_MSG_UNDER_CURSOR, MQRC_OPTIONS_ERROR       },
   { MQGMO_UNLOCK | MQGMO_MSG_UNDER_CURSOR,                  MQRC_OPTIONS_ERROR       }
  };

  const int nbrOptions = sizeof(optionList) / sizeof(sOptionList);

  for (int n = 0; n < nbrOptions; n++)
  {
    gmo.Options = optionList[n].option;
    Get(optionList[n].rc);
  }
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 204=
   ===============
   Test rejecting and accepting an uncommitted truncated message.
----------------------------------------------------------------------*/
static void Case204()
{
  TX_Begin();
   Put(100,'A');
   gmo.Options = MQGMO_NONE;
   Get(MQRC_TRUNCATED_MSG_FAILED, 50, 'A');
   gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
   Get(MQRC_TRUNCATED_MSG_ACCEPTED, 50, 'A');
   Get(MQRC_NO_MSG_AVAILABLE);
  TX_Commit();
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 205=
   ===============
   Test a simple browse of three messages.
----------------------------------------------------------------------*/
static void Case205()
{
  TX_Begin();
   Put(100, 'A');
   Put(100, 'B');
   Put(100, 'C');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_NEXT;     // same as BROWSE_FIRST
  Get(MQRC_NONE, 100, 'A');
  Get(MQRC_NONE, 100, 'B');
  Get(MQRC_NONE, 100, 'C');
  Get(MQRC_NO_MSG_AVAILABLE);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 206=
   ===============
   Test rejecting and accepting a committed truncated message and
   verify that the cursor positioning is correct.
----------------------------------------------------------------------*/
static void Case206()
{
  TX_Begin();
   Put(100, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_TRUNCATED_MSG_FAILED, 50, 'A');
  gmo.Options = MQGMO_BROWSE_NEXT | MQGMO_ACCEPT_TRUNCATED_MSG;
  Get(MQRC_TRUNCATED_MSG_ACCEPTED, 50, 'A');
  Get(MQRC_NO_MSG_AVAILABLE);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 207=
   ===============
   Browse a queue with one committed and one uncommitted message.  The
   uncommitted message should not be visible to Browse.
----------------------------------------------------------------------*/
static void Case207()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  TX_Begin();
   Put(50, 'B');
   gmo.Options = MQGMO_BROWSE_FIRST;
   Get(MQRC_NONE, 50, 'A');
   gmo.Options = MQGMO_BROWSE_NEXT;
   Get(MQRC_NO_MSG_AVAILABLE);
  TX_Commit();
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 208=
   ===============
   Test MSG_UNDER_CURSOR and verify the correct return code when
   using BROWSE_MSG_UNDER_CURSOR after the message under the cursor
   is removed.
----------------------------------------------------------------------*/
static void Case208()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NO_MSG_UNDER_CURSOR);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 209=
   ===============
   Test browsing, then getting the message under the cursor.
----------------------------------------------------------------------*/
static void Case209()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 210=
   ===============
   Verify that attempting to browse past the end of the queue doesn't
   move the cursor.
----------------------------------------------------------------------*/
static void Case210()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NO_MSG_AVAILABLE);
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 211=
   ===============
   Verify that a locked message can only be read using the cursor
   that locked it, and that browsing without lock will unlock a
   message.
----------------------------------------------------------------------*/
static void Case211()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NO_MSG_AVAILABLE, 2);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 212=
   ===============
   Verify that a message that becomes locked after it's browsed can't
   be browsed even using the cursor that first browsed it, but it can
   after the locking cursor moves on.
----------------------------------------------------------------------*/
static void Case212()
{
  TX_Begin();
   Put(50, 'A');
   Put(50, 'B');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 2);
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NO_MSG_UNDER_CURSOR, 1);
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NONE, 50, 'B', 2);
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'A', 1);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 213=
   ===============
   Verify that two BROWSE_FIRSTs with LOCK keeps the message locked,
   and that a BROWSE_MSG_UNDER_CURSOR without lock will unlock it.
----------------------------------------------------------------------*/
static void Case213()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NO_MSG_AVAILABLE, 2);
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 214=
   ===============
   Verify that a locked message is unlocked when the Hobj that has
   it locked is closed.
----------------------------------------------------------------------*/
static void Case214()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 1);
  Close(1);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 215=
   ===============
   Verify that a locked message is unlocked when MQDISC is executed.
----------------------------------------------------------------------*/
static void Case215()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A');
  Disc();
  Conn();
  Open();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 216=
   ===============
   Verify correct reason code when BROWSE_MSG_UNDER_CURSOR is used
   before any other call and when it finds the message is gone.
   Also, try to unlock it.
----------------------------------------------------------------------*/
static void Case216()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NO_MSG_UNDER_CURSOR, 1);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A', 2);
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NO_MSG_UNDER_CURSOR, 1);
  gmo.Options = MQGMO_UNLOCK;
  Get(MQRC_NO_MSG_LOCKED, 1);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 217=
   ===============
   Try to unlock a message that isn't locked.
----------------------------------------------------------------------*/
static void Case217()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_UNLOCK;
  Get(MQRC_NO_MSG_LOCKED);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 218=
   ===============
   Try to unlock a message that was locked, but is now gone.
----------------------------------------------------------------------*/
static void Case218()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_UNLOCK;
  Get(MQRC_NO_MSG_LOCKED);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 219=
   ===============
   Verify that removing a message that was just browsed doesn't cause
   a problem browsing the next message.
----------------------------------------------------------------------*/
static void Case219()
{
  TX_Begin();
   Put(50, 'A');
   Put(50, 'B');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NONE, 50, 'B');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 220=
   ===============
   Verify that a message is unlocked if the end of the queue is reached.
----------------------------------------------------------------------*/
static void Case220()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NO_MSG_AVAILABLE, 1);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 221=
   ===============
   Try to get a message that was locked by another handle,
   then try again after the message is unlocked.
----------------------------------------------------------------------*/
static void Case221()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NO_MSG_AVAILABLE, 2);
  gmo.Options = MQGMO_UNLOCK;
  Get((int)MQRC_NONE, 1);
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A', 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 222=
   ===============
   Verify that a BROWSE_NEXT after reaching end-of-queue and after
   a message is added will succeed.
----------------------------------------------------------------------*/
static void Case222()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NO_MSG_AVAILABLE);
  TX_Begin();
   Put(50, 'B');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NONE, 50, 'B');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 223=
   ===============
   Verify that the same handle that locked a message can get it using
   a non-browse MQGET, but a different handle, only opened for input,
   cannot.
----------------------------------------------------------------------*/
static void Case223()
{
  TX_Begin();
   Put(50, 'A', 1);
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NO_MSG_AVAILABLE, 2);
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NO_MSG_UNDER_CURSOR, 1);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 224=
   ===============
   Verify that UNLOCK and BROWSE_MSG_UNDER_CURSOR don't advance the
   cursor.
----------------------------------------------------------------------*/
static void Case224()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_UNLOCK;
  Get((int)MQRC_NONE);
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'A');
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 225=
   ===============
   Test MSG_UNDER_CURSOR and verify the correct return code when
   using BROWSE_MSG_UNDER_CURSOR after the message under the cursor
   is removed.  This is the same as test case 208, except that the
   removal of the message is committed.
----------------------------------------------------------------------*/
static void Case225()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  TX_Begin();
   gmo.Options = MQGMO_BROWSE_FIRST;
   Get(MQRC_NONE, 50, 'A');
   gmo.Options = MQGMO_MSG_UNDER_CURSOR;
   Get(MQRC_NONE, 50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NO_MSG_UNDER_CURSOR);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 226=
   ===============
   - A message is browsed from the front message list, then
     test BROWSE_MSG_UNDER_CURSOR after the message is swept.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
----------------------------------------------------------------------*/
static void Case226()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A');
  QMQZ_Sleep(65);   // queue is swept after 60 seconds
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 227=
   ===============
   - A swept message is browsed, then
     test BROWSE_MSG_UNDER_CURSOR after the message is unswept.
   - The queue is then re-swept and the message is browsed again, but
     this time it's locked.
   - The queue is unswept again and the messaged is browsed one more
     time (and unlocked).
   - Another handle then gets the previously locked message.
   - The queue should now be empty.  Try to browse and read it.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
----------------------------------------------------------------------*/
static void Case227()
{
  TX_Begin();
   Put(50, 'A');
   Put(50, 'B');   // this is the message that gets browsed
   Put(50, 'C');
  TX_Commit();

  // Sleep for 65 seconds to allow the queue to sweep.
  QMQZ_Sleep(65);   // queue is swept after 60 seconds

  // Browse to message B to set the cursor.
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NONE, 50, 'B', 1);

  // Using a different handle, get message A.
  // This will cause the queue to unsweep.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A', 2);  // causes unsweep
  QMQZ_Sleep(10);   // time for unsweep to complete

  // Browse message B (now the first message).
  // This message has now been unswept.
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'B', 1);

  // Allow the queue to be re-swept.
  QMQZ_Sleep(65);

  // Browse and lock message B.
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'B', 1);

  // Get a message using the other handle.  Since
  // message B is locked, this will get message C.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'C', 2);  // causes unsweep

  // Browse and unlock message B.
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'B', 1);

  // Now the other handle gets message B, since it's unlocked.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'B', 2);

  // The queue should now be empty.
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NO_MSG_UNDER_CURSOR, 1);
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NO_MSG_AVAILABLE, 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 228=
   ===============
   - An unswept message is locked and the lock is tested.
   - Then the message is swept and the lock is tested again.
   - Finally, the message is unswept and the lock tested one more time.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
----------------------------------------------------------------------*/
static void Case228()
{
  TX_Begin();
   Put(50, 'A');
   Put(50, 'B');
   Put(50, 'C');
  TX_Commit();

  // Browse to message B and lock it.
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 1);
  gmo.Options = MQGMO_BROWSE_NEXT | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'B', 1);

  // Using a different handle, browse two messages
  // to verify that message B is locked.
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 2);
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NONE, 50, 'C', 2);  // gets C because B is locked.

  // Allow the queue to be swept.
  QMQZ_Sleep(65);

  // Now that the queue is swept, browse the messages
  // again to verify that message B is still locked.
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 2);
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NONE, 50, 'C', 2);  // gets C because B is locked.

  // Get message A using the other handle.
  // This will cause the queue to unsweep.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A', 2);  // causes unsweep

  // Using the second handle, browse one message
  // to verify that message B is locked.
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'C', 2);  // gets C because B is locked.

  // Browse and unlock message B.
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 50, 'B', 1);

  // Now the other handle gets message B, since it's unlocked.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'B', 2);

  // Since the message under handle 1's cursor is no
  // longer there, we should get an error return.
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NO_MSG_UNDER_CURSOR, 1);

  // Now take message C off the queue.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'C', 2);

  // The queue should now be empty.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NO_MSG_AVAILABLE, 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 229=
   ===============
   - Verify that a handle that locks a message can get it with a
     non-browse MQGET.
----------------------------------------------------------------------*/
static void Case229()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();

  // Lock message A.
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A');

  // Using a non-browse MQGET to read and remove the message.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A');

  // The queue should now be empty.
  gmo.Options = MQGMO_UNLOCK;
  Get(MQRC_NO_MSG_LOCKED);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 230=
   ===============
   - Lock a message and hold it until it ages.  This will cause it
     to be validated the next time it's encountered.
----------------------------------------------------------------------*/
static void Case230()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();

  // Lock message A.
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 1);

  // Wait until it ages (and, for memory queue, is swept).
  QMQZ_Sleep(MAX(65, OLD_LOCK + 5));                                            //26359

  // Using a browse MQGET to try to read the message.
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NO_MSG_AVAILABLE, 2);

  // Now unlock the message...
  gmo.Options = MQGMO_UNLOCK;
  Get((int)MQRC_NONE, 1);

  // ...and try a browse MQGET again.
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 50, 'A', 2);

  // Then remove the message.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A', 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 231=
   ===============
   - Same as test case 230, except a non-browse MQGET is used to
     validate the lock.
----------------------------------------------------------------------*/
static void Case231()
{
  TX_Begin();
   Put(50, 'A');
  TX_Commit();

  // Lock message A.
  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(MQRC_NONE, 50, 'A', 1);

  // Wait until it ages (and, for memory queue, is swept).
  QMQZ_Sleep(MAX(65, OLD_LOCK + 5));                                            //26359

  // Using a non-browse MQGET to try to read the message.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NO_MSG_AVAILABLE, 2);

  // Now unlock the message...
  gmo.Options = MQGMO_UNLOCK;
  Get((int)MQRC_NONE, 1);

  // ...and try a non-browse MQGET again.
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 50, 'A', 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 232=
   ===============
   - Browse a swept message that uses an overflow SWB.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
----------------------------------------------------------------------*/
static void Case232()
{
  TX_Begin();
   Put(1000, 'A', 1);
  TX_Commit();
  QMQZ_Sleep(65);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 1000, 'A', 1);
  gmo.Options = MQGMO_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 1000, 'A', 1);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 233=
   ===============
   - Put two messages with different MsgId's and verify that the
     correct ones are read when specifying the MsgId in the MQMD,
     first with MQGMO_BROWSE_FIRST and then with MQGMO_NONE.
----------------------------------------------------------------------*/
static void Case233()
{
  char MsgId1[MQ_MSG_ID_LENGTH], MsgId2[MQ_MSG_ID_LENGTH];

  blankPad(MsgId1, "MSGID1", MQ_MSG_ID_LENGTH);
  blankPad(MsgId2, "MSGID2", MQ_MSG_ID_LENGTH);

  TX_Begin();
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   Put(100, 'A');
   memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
   Put(100, 'B');
  TX_Commit();

  gmo.MatchOptions = MQMO_MATCH_MSG_ID;
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'B');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'A');

  gmo.Options = MQGMO_NONE;
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'B');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 234=
   ===============
   - Put two messages with different MsgId's and verify that the
     correct ones are read when specifying the MsgId in the MQMD,
     first with MQGMO_BROWSE_FIRST and then with MQGMO_NONE.
   - This is the same as case 233, except that the messages are
     swept.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
----------------------------------------------------------------------*/
static void Case234()
{
  char MsgId1[MQ_MSG_ID_LENGTH], MsgId2[MQ_MSG_ID_LENGTH];

  blankPad(MsgId1, "MSGID1", MQ_MSG_ID_LENGTH);
  blankPad(MsgId2, "MSGID2", MQ_MSG_ID_LENGTH);

  TX_Begin();
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   Put(100, 'A');
   memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
   Put(100, 'B');
  TX_Commit();

  // Wait for queue to be swept.

  QMQZ_Sleep(65);

  gmo.MatchOptions = MQMO_MATCH_MSG_ID;
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'B');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'A');

  gmo.Options = MQGMO_NONE;
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'B');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 235=
   ===============
   - Verify that a BROWSE_NEXT will find the second matching message
     if the first one has been passed.
   - Remove the messages using MsgID.
----------------------------------------------------------------------*/
static void Case235()
{
  char MsgId1[MQ_MSG_ID_LENGTH], MsgId2[MQ_MSG_ID_LENGTH];

  blankPad(MsgId1, "MSGID1", MQ_MSG_ID_LENGTH);
  blankPad(MsgId2, "MSGID2", MQ_MSG_ID_LENGTH);

  TX_Begin();
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   Put(100, 'A');
   memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
   Put(100, 'B');
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   Put(100, 'C');
  TX_Commit();

  gmo.MatchOptions = MQMO_MATCH_MSG_ID;
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'B');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NONE, 100, 'C');

  gmo.Options = MQGMO_NONE;
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'B');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'A');
  Get(MQRC_NONE, 100, 'C');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 236=
   ===============
   - Verify that a BROWSE_NEXT will find the second matching message
     if the first one has been passed.
   - Remove the messages using MsgID.
   - This is the same as case 235, except that the messages are
     swept.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
----------------------------------------------------------------------*/
static void Case236()
{
  char MsgId1[MQ_MSG_ID_LENGTH], MsgId2[MQ_MSG_ID_LENGTH];

  blankPad(MsgId1, "MSGID1", MQ_MSG_ID_LENGTH);
  blankPad(MsgId2, "MSGID2", MQ_MSG_ID_LENGTH);

  TX_Begin();
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   Put(100, 'A');
   memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
   Put(100, 'B');
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   Put(100, 'C');
  TX_Commit();

  // Wait for queue to be swept.

  QMQZ_Sleep(65);

  gmo.MatchOptions = MQMO_MATCH_MSG_ID;
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'B');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  gmo.Options = MQGMO_BROWSE_NEXT;
  Get(MQRC_NONE, 100, 'C');

  gmo.Options = MQGMO_NONE;
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'B');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  Get(MQRC_NONE, 100, 'A');
  Get(MQRC_NONE, 100, 'C');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 237=
   ===============
   - Put messages with different MsgId/CorrelId combinations and verify
     that the correct ones are read when specifying the MsgId and
     CorrelId in the MQMD, first with MQGMO_BROWSE_FIRST and then with
     MQGMO_NONE.
----------------------------------------------------------------------*/
static void Case237()
{
  char MsgId1[MQ_MSG_ID_LENGTH], MsgId2[MQ_MSG_ID_LENGTH];
  char CorrelId1[MQ_CORREL_ID_LENGTH], CorrelId2[MQ_CORREL_ID_LENGTH];

  blankPad(MsgId1, "MSGID1", MQ_MSG_ID_LENGTH);
  blankPad(MsgId2, "MSGID2", MQ_MSG_ID_LENGTH);
  blankPad(CorrelId1, "CORRELID1", MQ_CORREL_ID_LENGTH);
  blankPad(CorrelId2, "CORRELID2", MQ_CORREL_ID_LENGTH);

  TX_Begin();
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   memcpy(md.CorrelId, CorrelId1, MQ_CORREL_ID_LENGTH);
   Put(100, 'A');
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   memcpy(md.CorrelId, CorrelId2, MQ_CORREL_ID_LENGTH);
   Put(100, 'B');
   memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
   memcpy(md.CorrelId, CorrelId1, MQ_CORREL_ID_LENGTH);
   Put(100, 'C');
   memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
   memcpy(md.CorrelId, CorrelId2, MQ_CORREL_ID_LENGTH);
   Put(100, 'D');
  TX_Commit();

  gmo.MatchOptions = MQMO_MATCH_MSG_ID | MQMO_MATCH_CORREL_ID;
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'D');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId1, MQ_CORREL_ID_LENGTH);
  Get(MQRC_NONE, 100, 'A');
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId1, MQ_CORREL_ID_LENGTH);
  Get(MQRC_NONE, 100, 'C');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId2, MQ_CORREL_ID_LENGTH);
  Get(MQRC_NONE, 100, 'B');

  gmo.Options = MQGMO_NONE;
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId2, MQ_CORREL_ID_LENGTH);
  Get(MQRC_NONE, 100, 'D');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId1, MQ_CORREL_ID_LENGTH);
  Get(MQRC_NONE, 100, 'A');
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId1, MQ_CORREL_ID_LENGTH);
  Get(MQRC_NONE, 100, 'C');
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId2, MQ_CORREL_ID_LENGTH);
  Get(MQRC_NONE, 100, 'B');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 238=
   ===============
   - Verify that BROWSE_MSG_UNDER_CURSOR and MSG_UNDER_CURSOR don't
     require successful match of MsgId.
----------------------------------------------------------------------*/
static void Case238()
{
  char MsgId1[MQ_MSG_ID_LENGTH], MsgId2[MQ_MSG_ID_LENGTH];

  blankPad(MsgId1, "MSGID1", MQ_MSG_ID_LENGTH);
  blankPad(MsgId2, "MSGID2", MQ_MSG_ID_LENGTH);

  TX_Begin();
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   Put(100, 'A');
  TX_Commit();

  gmo.MatchOptions = MQMO_MATCH_MSG_ID;
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'A');
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 100, 'A');
  gmo.Options = MQGMO_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 100, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 239=
   ===============
   - Verify that BROWSE_MSG_UNDER_CURSOR and MSG_UNDER_CURSOR don't
     require successful match of MsgId.
   - This is the same as case 238 except that the messages are
     swept.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
----------------------------------------------------------------------*/
static void Case239()
{
  char MsgId1[MQ_MSG_ID_LENGTH], MsgId2[MQ_MSG_ID_LENGTH];

  blankPad(MsgId1, "MSGID1", MQ_MSG_ID_LENGTH);
  blankPad(MsgId2, "MSGID2", MQ_MSG_ID_LENGTH);

  TX_Begin();
   memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
   Put(100, 'A');
  TX_Commit();

  // Wait for queue to be swept.

  QMQZ_Sleep(65);

  gmo.MatchOptions = MQMO_MATCH_MSG_ID;
  memcpy(md.MsgId, MsgId1, MQ_MSG_ID_LENGTH);
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'A');
  memcpy(md.MsgId, MsgId2, MQ_MSG_ID_LENGTH);
  gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 100, 'A');
  gmo.Options = MQGMO_MSG_UNDER_CURSOR;
  Get(MQRC_NONE, 100, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 240=
   ===============
   - Verify that MQPMO_NEW_MSG_ID generates a new MsgId in MQMD.
   - Put a message with MsgId specified in MQMD,
     with MQPMO_NEW_MSG_ID also specified in MQPMO.
     The MsgID field in MQMD should be overwritten by the QMGR.
   - Get with the original MsgId should result in
     MQRC_NO_MSG_AVAILABLE.
----------------------------------------------------------------------*/
static void Case240()
{
  char MsgId[MQ_MSG_ID_LENGTH];

  blankPad(MsgId, "MYMSGID", MQ_MSG_ID_LENGTH);

  memcpy(md.MsgId, MsgId, MQ_MSG_ID_LENGTH);
  pmo.Options = MQPMO_NEW_MSG_ID;
  Put(100, 'A');

  // Get the message with match option.
  gmo.MatchOptions = MQMO_MATCH_MSG_ID;
  memcpy(md.MsgId, MsgId, MQ_MSG_ID_LENGTH);
  Get(MQRC_NO_MSG_AVAILABLE, 100, 'A');

  // Get the message with no match option.
  gmo.MatchOptions = MQMO_NONE;
  Get(MQRC_NONE, 100, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 241=
   ===============
   - Verify that MQPMO_NEW_CORREL_ID generates a new CorrelId in MQMD.
   - Put a message with CorrelId specified in MQMD, with
     MQPMO_NEW_CORREL_ID also specified in MQPMO.
     The CorrelId in MQMD should be overwritten by the QMGR.
   - Get with the original CorrelId should result in
     MQ_NO_MSG_AVAILABLE.
----------------------------------------------------------------------*/
static void Case241()
{
  char CorrelId[MQ_CORREL_ID_LENGTH];

  blankPad(CorrelId, "MYCORRELID", MQ_CORREL_ID_LENGTH);

  memcpy(md.CorrelId, CorrelId, MQ_CORREL_ID_LENGTH);
  pmo.Options = MQPMO_NEW_CORREL_ID;
  Put(100, 'A');

  // Get the message with match option.
  gmo.MatchOptions = MQMO_MATCH_CORREL_ID;
  memcpy(md.CorrelId, CorrelId, MQ_CORREL_ID_LENGTH);
  Get(MQRC_NO_MSG_AVAILABLE, 100, 'A');

  // Get the message with no match option.
  gmo.MatchOptions = MQMO_NONE;
  Get(MQRC_NONE, 100, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 242=
   ===============
   - Verify that MQPMO_NEW_MSG_ID and MQPMO_NEW_CORREL_ID
     generates both new MsgId and CorrelId in MQMD.
   - Put a message with MsgId and CorrelId specified in MQMD,
     with MQPMO_NEW_MSG_ID and MQPMO_NEW_CORREL_ID also specified
     in MQPMO.
     The MsgID and CorrelId in MQMD should be overwritten by the
     QMGR.
   - Get with the original MsgId and CorrelId should result in
     MQ_NO_MSG_AVAILABLE.
----------------------------------------------------------------------*/
static void Case242()
{
  char MsgId[MQ_MSG_ID_LENGTH];
  char CorrelId[MQ_CORREL_ID_LENGTH];

  blankPad(MsgId, "MYMSGID", MQ_MSG_ID_LENGTH);
  blankPad(CorrelId, "MYCORRELID", MQ_CORREL_ID_LENGTH);

  memcpy(md.MsgId, MsgId, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId, MQ_CORREL_ID_LENGTH);
  pmo.Options = MQPMO_NEW_MSG_ID | MQPMO_NEW_CORREL_ID;
  Put(100, 'A');

  // Get the message with no match option.
  gmo.MatchOptions = MQMO_MATCH_MSG_ID | MQMO_MATCH_CORREL_ID;
  memcpy(md.MsgId, MsgId, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId, MQ_CORREL_ID_LENGTH);
  Get(MQRC_NO_MSG_AVAILABLE, 100, 'A');

  // Get the message with match option.
  gmo.MatchOptions = MQMO_NONE;
  Get(MQRC_NONE, 100, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 243=
   ===============
   - Verify that messages on the Rear Message List can be
     searched by MsgID and CorrelID.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
----------------------------------------------------------------------*/
static void Case243()
{
  char MsgId[MQ_MSG_ID_LENGTH];
  char CorrelId[MQ_CORREL_ID_LENGTH];

  blankPad(MsgId, "MYMSGID", MQ_MSG_ID_LENGTH);
  blankPad(CorrelId, "MYCORRELID", MQ_CORREL_ID_LENGTH);

  memcpy(md.MsgId, MsgId, MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId, MQ_CORREL_ID_LENGTH);
  pmo.Options = MQPMO_NEW_MSG_ID | MQPMO_NEW_CORREL_ID;
  Put(100, 'A');

  // Wait for queue to be swept.

  QMQZ_Sleep(65);

  // Put two messages on the Rear Message List.

  pmo.Options = MQPMO_NONE;

  blankPad(MsgId, "MSGID-B", MQ_MSG_ID_LENGTH);
  memcpy(md.MsgId, MsgId, MQ_MSG_ID_LENGTH);
  memset(md.CorrelId, ' ', MQ_CORREL_ID_LENGTH);
  Put(100, 'B');

  blankPad(CorrelId, "CORRELID-C", MQ_CORREL_ID_LENGTH);
  memset(md.MsgId, ' ', MQ_MSG_ID_LENGTH);
  memcpy(md.CorrelId, CorrelId, MQ_CORREL_ID_LENGTH);
  Put(100, 'C');

  // Get the messages with match option.

  memcpy(md.MsgId, MsgId, MQ_MSG_ID_LENGTH);
  gmo.MatchOptions = MQMO_MATCH_MSG_ID;
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'B');       // browse it
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 100, 'B');       // get it
  Get(MQRC_NO_MSG_AVAILABLE);     // make sure it's gone.

  memcpy(md.CorrelId, CorrelId, MQ_CORREL_ID_LENGTH);
  gmo.MatchOptions = MQMO_MATCH_CORREL_ID;
  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(MQRC_NONE, 100, 'C');       // browse it
  gmo.Options = MQGMO_NONE;
  Get(MQRC_NONE, 100, 'C');       // get it
  Get(MQRC_NO_MSG_AVAILABLE);     // make sure it's gone.
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 244=
   ===============
   - Test handling of rolled back message.  A swept message (not the
     first swept message) is gotten and rolled back.  This puts it
     at the end of the Front Message List.  The message is then browsed
     and gotten.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
   - Set SWEEPDEPTH to about 30.
----------------------------------------------------------------------*/
static void Case244()
{
  char MsgId[] = "MSGIDnnn";

  // Put 100 messages on the queue.  The queue will be swept,
  // leaving SWEEPDEPTH messages on the Front Message List.

  TX_Begin();

  for (int n = 1; n <= 100; n++)
  {
    sprintf(MsgId, "MSGID%03d", n);
    memset(md.MsgId, ' ', MQ_MSG_ID_LENGTH);
    memcpy(md.MsgId, MsgId, strlen(MsgId));
    Put(1000, n == 60 ? 'B' : 'A');
  }

  TX_Commit();

  // Allow time for the queue to be swept.

  QMQZ_Sleep(20);

  // Get the 60th message and roll it back.

  memcpy(md.MsgId, "MSGID060", 8);
  gmo.MatchOptions = MQMO_MATCH_MSG_ID;
  TX_Begin();
  Get(1000, 'B');
  TX_Rollback();

  // The message is now on the end of the Front Message List.

  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(1000, 'B');
  gmo.Options = MQGMO_NONE;
  Get(1000, 'B');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 245=
   ===============
   - Allow a queue to be fully swept, then get the first message.
   - The queue should be empty before starting this test case,
     otherwise the getrate will prevent the queue from being swept.
   - This test is designed for memory queues.
   - Make sure SWEEPSWB is high enough to cause the queue to be swept.
   - Set SWEEPDEPTH to about 30.
----------------------------------------------------------------------*/
static void Case245()
{
  Put(100, 'A');

  for (int n = 1; n <= 100; n++)
    Put(100, 'B');

  // Allow time for the queue to be swept.

  QMQZ_Sleep(65);

  Get(100, 'A');
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 246=
   ===============
   - Hold a lock for more than 60 seconds and verify that the message
     is still locked.
----------------------------------------------------------------------*/
static void Case246()
{
  Put(100, 'A', 1);
  Put(100, 'B', 1);

  gmo.Options = MQGMO_BROWSE_FIRST | MQGMO_LOCK;
  Get(100, 'A', 1);  // handle 1 has msg locked

  // Give lock time to age.

  QMQZ_Sleep(65);

  // Should get B because A is locked.

  gmo.Options = MQGMO_BROWSE_FIRST;
  Get(100, 'B', 2);
  Get(100, 'B', 2);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 247=
   ===============
   - Test expiry values for valid and invalid codes
----------------------------------------------------------------------*/
static void Case247()
{
   // Valid Values:
   md.Expiry = 1;
   Put(100, 'A', 1);
   md.Expiry = 2;
   Put(100, 'A', 1);
   md.Expiry = 100;
   Put(100, 'A', 1);
   md.Expiry = 999999999;
   Put(100, 'A', 1);
   md.Expiry = -1;
   Put(100, 'A', 1);

   // Invalid Values:
   md.Expiry = 0;
   Put(MQRC_EXPIRY_ERROR, 1, 'A', 1);
   md.Expiry = -2;
   Put(MQRC_EXPIRY_ERROR, 1, 'A', 1);
   md.Expiry = -150;
   Put(MQRC_EXPIRY_ERROR, 1, 'A', 1);
   md.Expiry = -9999;
   Put(MQRC_EXPIRY_ERROR, 1, 'A', 1);
   md.Expiry = -999999999;
   Put(MQRC_EXPIRY_ERROR, 1, 'A', 1);
   md.Expiry = -499999999;
   Put(MQRC_EXPIRY_ERROR, 1, 'A', 1);
   md.Expiry = 1000000000;
   Put(MQRC_EXPIRY_ERROR, 1, 'A', 1);
   md.Expiry = -999999999;
   Put(MQRC_EXPIRY_ERROR, 1, 'A', 1);
}
/*----------------------------------------------------------------------
   ===============
   =Test Case 248=
   ===============
----------------------------------------------------------------------*/
static void Case248()
{
   struct timeval StartTime;                 // Starting Time
   struct timeval CurrentTime;               // Current Time
   unsigned long long TimeDiff = 0;          // Elapsed time

   // Check that the default is unlimited
   Put(100, 'A', 1);
   Get(100, 'A', 1);
   if (md.Expiry != MQEI_UNLIMITED)
   {
      char string[50];
      sprintf(string, "Expected MQEI_UNLIMITED, Got %d\n", MQEI_UNLIMITED, md.Expiry);
      Error(string);
   }

   // Put an expired message then attempt a browse
   md.Expiry = 1;
   Put(100, 'B', 1);

   /*----------------------------------------------------------------------*/
   /* Put in a defer loop for 0.1 seconds.  QMQZ_Sleep() was not accurate  */
   /* enough, so we have to do this.                                       */
   /*----------------------------------------------------------------------*/
   gettimeofday(&StartTime, NULL);           // Get initial starting time
   do
   {
     defrc();
     gettimeofday(&CurrentTime, NULL);            // Refresh current time
     TimeDiff =  (CurrentTime.tv_sec - StartTime.tv_sec) * 1000000 +
                   (CurrentTime.tv_usec - StartTime.tv_usec);

   } while(TimeDiff/1000 < 100);

   gmo.Options = MQGMO_BROWSE_FIRST;
   Get(MQRC_NO_MSG_AVAILABLE, 100, 'B');

   // Put an expired message then attempt a get
   md.Expiry = 1;
   Put(100, 'C', 1);

   /*----------------------------------------------------------------------*/
   /* Put in a defer loop for 0.1 seconds.  QMQZ_Sleep() was not accurate  */
   /* enough, so we have to do this.                                       */
   /*----------------------------------------------------------------------*/
   gettimeofday(&StartTime, NULL);           // Get initial starting time
   do
   {
     defrc();
     gettimeofday(&CurrentTime, NULL);            // Refresh current time
     TimeDiff =  (CurrentTime.tv_sec - StartTime.tv_sec) * 1000000 +
                   (CurrentTime.tv_usec - StartTime.tv_usec);

   } while(TimeDiff/1000 < 100);

   gmo.Options = MQGMO_NONE;
   Get(MQRC_NO_MSG_AVAILABLE, 100, 'C');

   // Verify that a single message expires
   md.Expiry = 250;
   Put(100, 'D', 1);
   QMQZ_Sleep(20);
   // Browse it to make sure it has not expired
   gmo.Options = MQGMO_BROWSE_FIRST;
   Get(MQRC_NONE, 100, 'D', 1);
   QMQZ_Sleep(10);
   // Try to get the expired message
   //gmo.Options = MQGMO_BROWSE_FIRS;
   Get(MQRC_NO_MSG_AVAILABLE, 100, 'D');

}
/*----------------------------------------------------------------------
   ===============
   =Test Case 249=
   ===============
----------------------------------------------------------------------*/
static void Case249()
{
   struct timeval StartTime;                 // Starting Time
   struct timeval CurrentTime;               // Current Time
   unsigned long long TimeDiff = 0;          // Elapsed time

   // Put a message, Browse message wait for expire, then attempt to
   // browse message under cursor which fails
   md.Expiry = 150;
   Put(100, 'A', 1);
   // Browse it to make sure it has not expired
   gmo.Options = MQGMO_BROWSE_FIRST;
   Get(100, 'A', 1);

   /*----------------------------------------------------------------------*/
   /* Put in a defer loop for 15 seconds.  QMQZ_Sleep() was not accurate   */
   /* enough, so we have to do this.                                       */
   /*----------------------------------------------------------------------*/
   gettimeofday(&StartTime, NULL);           // Get initial starting time
   do
   {
     defrc();
     gettimeofday(&CurrentTime, NULL);            // Refresh current time
     TimeDiff =  (CurrentTime.tv_sec - StartTime.tv_sec) * 1000000 +
                   (CurrentTime.tv_usec - StartTime.tv_usec);

   } while(TimeDiff/1000000 < 15);

   gmo.Options = MQGMO_BROWSE_MSG_UNDER_CURSOR;
   Get(MQRC_NO_MSG_UNDER_CURSOR, 100, 'A', 1);

   // Put a message, Browse message wait for expire, then get msg under
   // cursor which fails
   md.Expiry = 150;
   Put(100, 'B', 1);
   // Browse it to make sure it has not expired
   gmo.Options = MQGMO_BROWSE_FIRST;
   Get(100, 'B', 1);

   /*----------------------------------------------------------------------*/
   /* Put in a defer loop for 15 seconds.  QMQZ_Sleep() was not accurate   */
   /* enough, so we have to do this.                                       */
   /*----------------------------------------------------------------------*/
   gettimeofday(&StartTime, NULL);           // Get initial starting time
   do
   {
     defrc();
     gettimeofday(&CurrentTime, NULL);            // Refresh current time
     TimeDiff =  (CurrentTime.tv_sec - StartTime.tv_sec) * 1000000 +
                   (CurrentTime.tv_usec - StartTime.tv_usec);

   } while(TimeDiff/1000000 < 15);

   gmo.Options = MQGMO_MSG_UNDER_CURSOR;
   Get(MQRC_NO_MSG_UNDER_CURSOR, 100, 'B',1);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 250=
   ===============
----------------------------------------------------------------------*/
static void Case250()
{
   int i;
   char MsgId[25];

   // Put 5 messages with an expiry of 1 minute
   md.Expiry = 150;
   for (i = 0; i < 5; i++)
   {
      sprintf(MsgId, "%-24d", i);
      memcpy(md.MsgId, MsgId, 24);
      Put(100, 'A', 1);
   }

   // Put 10 messages with an expiry of 2 minutes
   md.Expiry = 300;
   for (; i < 15; i++)
   {
      sprintf(MsgId, "%-24d", i);
      memcpy(md.MsgId, MsgId, 24);
      Put(100, 'A', 1);
   }

   // Put 5 messages with an expiry of 3 minutes
   md.Expiry = 450;
   for (; i < 20; i++)
   {
      sprintf(MsgId, "%-24d", i);
      memcpy(md.MsgId, MsgId, 24);
      Put(100, 'A', 1);
   }

   // Put 1 message with an unlimited expiry
   md.Expiry = MQEI_UNLIMITED;
   sprintf(MsgId, "%-24d", i);
   memcpy(md.MsgId, MsgId, 24);
   Put(100, 'A', 1);

   // Browse all 20 messages by MsgId
   for (i = 0; i < 21; i++)
   {
      gmo.Options = MQGMO_BROWSE_FIRST;
      gmo.MatchOptions = MQMO_MATCH_MSG_ID;
      sprintf(MsgId, "%-24d", i);
      memcpy(md.MsgId, MsgId, 24);
      Get(100, 'A', 1);
   }

   // Sleep for 20 seconds and let first five expire
   QMQZ_Sleep(20);

   gmo.Options = MQGMO_BROWSE_FIRST;

   // Make sure first five expired by MsgId
   for (i = 0; i < 5; i++)
   {
      gmo.MatchOptions = MQMO_MATCH_MSG_ID;
      sprintf(MsgId, "%-24d", i);
      memcpy(md.MsgId, MsgId, 24);
      Get(MQRC_NO_MSG_AVAILABLE);
   }

   // Verify that other 15 are still there
   for (; i < 20; i++)
   {
      gmo.MatchOptions = MQMO_MATCH_MSG_ID;
      sprintf(MsgId, "%-24d", i);
      memcpy(md.MsgId, MsgId, 24);
      Get(100, 'A', 1);
   }

   // Sleep for another 15 seconds and let next 10 expire
   QMQZ_Sleep(15);

   // Make sure next 10 expired by MsgId
   for (i = 5; i < 15; i++)
   {
      gmo.MatchOptions = MQMO_MATCH_MSG_ID;
      sprintf(MsgId, "%-24d", i);
      memcpy(md.MsgId, MsgId, 24);
      Get(MQRC_NO_MSG_AVAILABLE);
   }

   // Verify that last 5 are still there
   for (; i < 20; i++)
   {
      gmo.MatchOptions = MQMO_MATCH_MSG_ID;
      sprintf(MsgId, "%-24d", i);
      memcpy(md.MsgId, MsgId, 24);
      Get(100, 'A', 1);
   }

   // Sleep for another 15 seconds and let next 5 expire
   QMQZ_Sleep(15);

   // Make sure next 5 expired by MsgId
   for (i = 15; i < 20; i++)
   {
      gmo.MatchOptions = MQMO_MATCH_MSG_ID;
      sprintf(MsgId, "%-24d", i);
      memcpy(md.MsgId, MsgId, 24);
      Get(MQRC_NO_MSG_AVAILABLE);
   }

   // Verify that Unlimited message is still there
    gmo.MatchOptions = MQMO_MATCH_MSG_ID;
    sprintf(MsgId, "%-24d", i);
    memcpy(md.MsgId, MsgId, 24);
    Get(100, 'A', 1);
}

/*----------------------------------------------------------------------
   ===============
   =Test Case 251=
   ===============
   - Verify that messages with a portion of a second of an EXPIRY value
     remaining are able to be gotten. For example, a message with .3
     seconds remaining should be able to be gotten.  We try this by
     putting a message on a queue with an EXPIRY time of just over 30
     seconds, and then we sleep for 30 seconds and try to GET it.
----------------------------------------------------------------------*/
static void Case251()
{
  MQLONG ExpiryValue;                       // Expiry Time remaining on msg
  struct timeval StartTime;                 // Starting Time
  struct timeval CurrentTime;               // Current Time
  char   output[250];                       // String to send to terminal
  long   TimeDiff;

  /*----------------------------------------------------------------------*/
  /* This is a table of attempts made to get a message with various       */
  /* EXPIRY times.  The times at the right are in tenths of seconds.  For */
  /* example, with an expiry time of 300 (30 seconds), I made 8 attempts  */
  /* at getting a message.  6 of them were able to get the message, and 2 */
  /* of them failed.  The 6 successful attempts had remaining EXPIRY times*/
  /* of 6, 6, 5, 2, 8 and 4 tenths of a second.                           */
  /* The loop with the defrc() should sleep for exactly 30 seconds, but I */
  /* imagine the inconsistency in the results stem from the actual amount */
  /* of time the ecb sleeps between the PUT and GET.                      */
  /*----------------------------------------------------------------------*/
  /*  295    GOT 2 OUT OF 8 ATTEMPTS    NA, NA, NA, NA, NA,  3,  3, NA    */
  /*  299    GOT 6 OUT OF 8 ATTEMPTS     4,  6, NA,  3,  6,  7,  7, NA    */
  /*  300    GOT 6 OUT OF 8 ATTEMPTS    NA, NA,  6,  6,  5,  2,  8,  4    */
  /*  301    GOT 7 OUT OF 8 ATTEMPTS     9, NA, 10,  6,  6,  5,  1,  1    */
  /*  302    GOT 6 OUT OF 6 ATTEMPTS     5,  5,  5, 10,  6, 11            */
  /*  305    GOT 6 OUT OF 6 ATTEMPTS     9, 13, 10, 15,  7,  8            */
  /*  310    GOT 6 OUT OF 6 ATTEMPTS    20,  5, 15, 18, 15, 19            */
  /*----------------------------------------------------------------------*/
  md.Expiry = 305;
  ExpiryValue = md.Expiry;

  Put(100, 'A');                            // Put a message
  gettimeofday(&StartTime, NULL);         // Get initial starting time

  /*----------------------------------------------------------------------*/
  /* Put in a defer loop for 30 seconds.  QMQZ_Sleep(30) was not accurate */
  /* enough, so we have to do this.                                       */
  /*----------------------------------------------------------------------*/
  do
  {
    defrc();
    gettimeofday(&CurrentTime, NULL);            // Refresh current time
    TimeDiff =  (CurrentTime.tv_sec - StartTime.tv_sec) * 1000000 +
                  (CurrentTime.tv_usec - StartTime.tv_usec);

  }while(TimeDiff/1000000 < 30);

  Get(MQRC_NONE, 100, 'A');                 // Get the message
  gettimeofday(&CurrentTime, NULL);           // Get the End time

  TimeDiff = (CurrentTime.tv_sec - StartTime.tv_sec) * 1000000 +
                (CurrentTime.tv_usec - StartTime.tv_usec);

  /*----------------------------------------------------------------------*/
  /* Display the remaining time left before the message expires.          */
  /*----------------------------------------------------------------------*/
  sprintf(output, "Expiry value Before: %d, After: %d, Time Elapsed: %ld.%ld",
          ExpiryValue, md.Expiry, TimeDiff / 1000000, TimeDiff % 1000000);
  out(output);

}

/*----------------------------------------------------------------------
   ===============
   =Test Case 252=
   ===============
   - Put a message onto a queue, get the message, then issue an MQCLOSE
     and disconnect. Reconnect and loop through this process 100 times,
     then verify the memory in use has not changed.
----------------------------------------------------------------------*/
static void Case252()                                                           //d26725
{                                                                               //d26725
  unsigned long memBefore = GetMemoryInUse();                                   //d26725
  unsigned long memAfter;                                                       //d26725
  char output[100];                                                             //d26725

  for (int i = 0; i < 100; i++) {                                               //d26725
    Put("Message 1");                                                           //d26725
    Get("Message 1");                                                           //d26725
    Close(1);                                                                   //d26725
    Disc();                                                                     //d26725
    Conn();                                                                     //d26725
    Open();                                                                     //d26725
  }

  memAfter = GetMemoryInUse();                                                  //d26725
  if (memAfter > memBefore) {                                                   //d26725
    sprintf(output,"memory after MQPUT loop %d larger than expected %d",        //d26725
            memAfter, memBefore);                                               //d26725
    CaseError(output);                                                          //d26725
  }                                                                             //d26725
}                                                                               //d26725

/*----------------------------------------------------------------------
   ===============
   =Test Case 253=
   ===============
   - Verify memory in use before an MQCONN is equal to the memory in use
     after an MQDISC.
----------------------------------------------------------------------*/

static void Case253()                                                           //d26725
{                                                                               //d26725
  unsigned long memAfter;                                                       //d26725
  char output[100];                                                             //d26725
  Put("Message 1");                                                             //d26725
  Get("Message 1");                                                             //d26725
  Close(1);                                                                     //d26725
  Disc();                                                                       //d26725

  memAfter = GetMemoryInUse();                                                  //d26725
  if ((memAfter - memBeforeConn) != 0) {                                        //d26725
    sprintf(output,"memory after DISC() %d different than expected %d",         //d26725
            memAfter, memBeforeConn);                                           //d26725
    CaseError(output);
  }

  Conn();                                                                       //d26725
  Open();                                                                       //d26725
}                                                                               //d26725

/*  ___________________________________________________________

          The RIGOR test cases take more computing time,
          look in MQDriverDocs.lwp for an explanation of each.
    ___________________________________________________________
*/


/*
  RIGOR1 does a MQPUT and corresponding MQGET of messages from min_size to max_size
  This is used to test that messages of all sizes can properly fit into SWBs
*/

void Rigor1(int min_size, int max_size)                                         //26359
{
  for (int i = min_size; i <= max_size; i++)                                    //26359
  {
    if (i % 50 == 0)
    {
      QMQZ_Sleep(1);
      out(sf("Size %d",i));
    }

    Put(i, 'A');
    Get(i, 'A');
  }
}


/*
  RIGOR2 builds all possible combinations of 2 tx_begins, 2 tx_commits/tx_rollbacks,
  3 MQPUTs, and 3 MQGETs.  Each combination is stored in an array A, of structure
  sA (sA consists of an int "numCommands", and a byte array "Command[]").  The
  array of commands is then sent to function RigorRun, which verifies that MQ
  correctly executes the commands.  Only cases from min_case to max_case are sent to
  RigorRun.  This tests transactions scope combinations with 2 scopes, nested or
  not nested.  StartArrayBackup and EndArrayBackup find heap memory for which
  RigorRun will back up the command array, in case something bad happens.
*/

void Rigor2(int min_case, int max_case)                                         //26359
{
  int  b1, b2;
  int  r1, r2;
  int  g1, p2, g2;
  sA   A;

  testcase = 0;

  StartArrayBackup();

  // Set up tx_begins.

  for (b1 = 1; b1 <= 7; b1++)
  {
    for (b2 = b1 + 1; b2 <= 7; b2++)
    {
      // Set up tx_commits or tx_rollbacks.

      for (r1 = b1 + 1; r1 <= 7; r1++)
      {
        for (r2 = b2 + 1; r2 <= 6; r2++)
        {
          // Locate places for puts and gets.

          for (g1 = 1; g1 <= 5; g1++)
          {
            for (p2 = 1; p2 <= 3; p2++)
            {
              for (g2 = p2; g2 <= 3; g2++)
              {
                // Combinations of commits and rollbacks.

                for (BYTE COR = 0; COR <= 3; COR++)
                {
                  testcase++;

                  if ((testcase >= min_case) && ((testcase % 10) == 0))         //26359
                    QMQZ_Sleep(1);     // slow down processing

                  if ((testcase >= min_case) && ((testcase % 100) == 0))        //26359
                    out(sf("case %d run",testcase));

                  // Run a case.

                  if ((testcase >= min_case) && (testcase <= max_case))         //26359
                  {
                    memset(&A, 0, sizeof(A));
                    A.numCommands = 10;
                    A.Command[b1] = c74BEGIN;
                    A.Command[RigorFindFree(&A, b2)] = c74BEGIN;

                    if (COR & 0x01)
                      A.Command[RigorFindFree(&A, r1)] = c74COMMIT;
                    else
                      A.Command[RigorFindFree(&A, r1)] = c74ROLLBACK;

                    if (COR & 0x02)
                      A.Command[RigorFindFree(&A, r2)] = c74COMMIT;
                    else
                      A.Command[RigorFindFree(&A, r2)] = c74ROLLBACK;

                    A.Command[RigorFindFree(&A, 1)] = c74PUT;
                    A.Command[RigorFindFree(&A, g1)] = c74GET;
                    A.Command[RigorFindFree(&A, p2)] = c74PUT;
                    A.Command[RigorFindFree(&A, g2)] = c74GET;
                    A.Command[RigorFindFree(&A, 1) ] = c74PUT;
                    A.Command[RigorFindFree(&A, 1) ] = c74GET;

                    if ((HelpVar != 1) && ((max_case - min_case) < 50))         //26359
                      out(sf("Running case %d",testcase));

                    RigorRun(&A);

                    if (testcase >= max_case)                                   //26359
                    {
                      EndArrayBackup();
                      return;
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}


/*
  RIGOR3 builds a list of commands much like RIGOR2, except the command list is
  randomized, to model possible customer transactions.  I used the following rules
  in building the random list:
  - Each list contains from 10 to 30 commands
  - From 1 to 1/3 of the commands are tx_begins, and each tx_begin is placed
    anywhere in the list except the end
  - For each tx_begin, a tx_commit or a tx_rollback is placed somewhere in the
    list after that tx_begin
  - Half of the remaining commands become MQPUTs
  - The other half become MQGETs
  - Additional MQGETs are executed after the list is run to empty the queue

  So it is possible to create deeply nested transaction scopes, as well as many
  unnested scopes.  Like RIGOR2, the list is sent to RigorRun for testing.  max_time
  different random lists are tested.  StartArrayBackup and EndArrayBackup find
  heap memory for which RigorRun will back up the command array, in case something
  bad happens.
*/

void Rigor3(int max_time)                                                       //26359
{
  int putgetcount = 0;                 // number of MQPUT, MQGET
  sA  A;                               // command array
  int i, NumScopes, Place;             // counters

  srand(time(NULL));                   // start random generator

  StartArrayBackup();

  testcase = 0;                        // counter
  out("Beginning to run randomized test cases");

  while (++testcase <= max_time)            // run max times                    //26359
  {
    memset(&A, 0, sizeof(A));          // initialize command array
    // 10 to SIZEOFA commands
    A.numCommands = (rand() % ((SIZEOFA - 10) / 2)) * 2 + 10;
    // number of tx_begins, up to 1/3 of commands
    NumScopes = (rand() % (A.numCommands / 3)) + 1;

    for (i = 0; i < NumScopes * 2; i += 2)  // place transaction pairs
    {
      Place = 1 + rand() % (A.numCommands - i - 3);   // point for tx_begin
      A.Command[RigorFindFree(&A, Place)] = c74BEGIN; // place the tx_begin
      A.Command[RigorFindFree(&A,
                Place + 1 + rand() % (A.numCommands - i - Place - 1))] =
        (rand() % 2) ? c74ROLLBACK : c74COMMIT;
    }

    for (; i < A.numCommands; i += 2)  // place PUT/GET pairs
    {
      Place = 1 + rand() % (A.numCommands - i - 1);   // point for MQPUT
      A.Command[RigorFindFree(&A, Place)] = c74PUT;   // place the put
      A.Command[RigorFindFree                         // corresponding get
           (&A, Place + rand() % (A.numCommands - i - Place))] = c74GET;
    }

    for (i = 1; i < A.numCommands; i++)  // search command array
    {
      if (A.Command[i] == c74BEGIN &&
          (A.Command[i + 1] == c74COMMIT ||
           A.Command[i + 1] == c74ROLLBACK))  // find transactions with no scope
      {
        // convert to MQPUT/MQGET
        A.Command[i] = c74PUT;  A.Command[i + 1] = c74GET;
      }
    }

    if ((putgetcount + A.numCommands - NumScopes * 2) > 400)
      QMQZ_Sleep(1);                        // slow down transactions

    putgetcount += A.numCommands - NumScopes * 2;

    if ((max_time < 40) || (testcase % (max_time / 25) == 0)) // output every case if few
      out(sf("Running case %d", testcase));                                     //26359

    RigorRun(&A);
  }                                         // run commands

  EndArrayBackup();
}

/*----------------------------------------------------------------------
   ==================
   =Test Case Rigor4=
   ==================
   - Cause 10,000 'Unknown Object Name' errors to test putting a large
     number of event messages to an event queue.  Local events must be
     enabled and SYSTEM.ADMIN.QMGR.EVENT queue must be defined.  The
     queue specified in the ZTEST MQS1 CASE command must not exist.

     ZTEST MQS1 CASE RIGOR4 xxx (xxx is undefined queue name)
----------------------------------------------------------------------*/
static void Rigor4()
{
  MQOD   ObjDesc = {MQOD_DEFAULT};    // Object Descriptor
  MQLONG CompCode, Reason;

  // Set the queue name.

  strcpy(ObjDesc.ObjectName, TrgtQName[0]);

  // Open the queue 10,000 times.

  for (int n = 0; n < 10000; n++)
  {
    MQOPEN(Hconn, &ObjDesc, MQOO_INPUT_AS_Q_DEF,
           &Hobj[0], &CompCode, &Reason);
    dlayc();
  }
}


/*
  Given a command array and a place to find, RigorFindFree will find the
  'place'th entry in the command array which is empty.
*/

int RigorFindFree(sA* pA, int place)
{
  int i = -1;

  while (place)
  {
    i++;

    if (pA->Command[i] == c74EMPTY)
      place--;
  }

  return i;
}


/*
  RigorRun will run a list of commands from one of the RIGOR functions.  It
  executes each command in the array and verifies that the command executed
  correctly.  The prediction works like this:

    If RigorRun encounters a tx_begin, it executes the tx_begin, and then
    increments "nests" to store all new transactions (gets and puts) on a new
    level.

    If an MQPUT is found, it increments the highest message put (0 to start) and
    MQPUTs a unique message (message contains number of highest message put).  It
    also adds one to getatscope[nests], to record the number of transactions at
    the current nesting level.

    If an MQGET is found, it increments the highest message got and MQGETs a
    message and verifies that the message is the unique message for the highest
    message got.  Then it increments the number of messages got at the current
    scope.  It is possible that no message is on the queue (like if a bunch of
    MQPUTs got rolled back) and mputhi == mputlow.  In that case nothing should be
    incremented and it should verify that the queue is empty.

    If a tx_rollback is found, all transactions on the current scope level are
    thrown out and "nests" is decremented.  This means that mputhi and mputlow are
    lowered, because the unique message that was put or got must be done again.

    If a tx_commit is found, "nests" is decremented, and all recorded transactions
    on the old nesting level must be moved in to the current nesting level,
    because a tx_rollback at the new level would undo all nested transactions.

    If, when all commands in the list have been executed, more messages have been
    put to the queue than gotten, additional MQGETs must be done and the message
    verified, so that the next test case running has an empty queue.  Also,
    RigorRun calls ArrayBackup before executing any commands so that, if something
    bad happens, the list of commands being run can be recovered.
*/

void RigorRun(sA* pA)                  // runs an array of transactions
{
  char buff[100];                      // output buffer
  int  nests = 0;                      // nesting levels
  int  mputhi  = 0;                    // total messages put
  int  mputlow = 0;                    // total messages got
  int  putatscope[SIZEOFA / 2] = {0};  // messages put at each nest level
  int  getatscope[SIZEOFA / 2] = {0};  // messages got at each nest level
  int i;

  ArrayBackup(pA);

  for (i = 0; i < pA->numCommands; i++)  // go through the commands
  {
    ArrayUpdate(pA);

    switch (pA->Command[i])
    {
      case c74BEGIN:                   // tx_begin
        TX_Begin();
        nests++;                       // nested more
        putatscope[nests] = 0;         // reset nest counters
        getatscope[nests] = 0;
        break;

      case c74COMMIT:                  // tx_commit
        TX_Commit();
        putatscope[nests - 1] += putatscope[nests];  // move nest counters
        getatscope[nests - 1] += getatscope[nests];  //   into lower nest
        nests--;                                     // less nesting
        break;

      case c74ROLLBACK:                // tx_rollback
        TX_Rollback();
        mputhi -= putatscope[nests];   // undo transactions expected
        mputlow -= getatscope[nests];  //   to be done at this level
        nests--;
        break;

      case c74PUT:                     // MQPUT
        mputhi++;                      // more messages put
        putatscope[nests]++;           // messages put at this scope
        sprintf(buff, "Message %d", mputhi); // unique message
        Put(buff, mputhi % queues + 1);
        break;

      case c74GET:                     // MQGET
        if (mputhi > mputlow)          // Message on queue
        {
          mputlow++;                   // more messages got
          getatscope[nests]++;         // message got at this scope
          sprintf(buff, "Message %d", mputlow); // get the unique message
          Get(buff, mputlow % queues + 1);
        }
        else                           // No message on queue
          Get(NOMSG, mputlow % queues + 1); // verify no message

        break;

      default:
        Error("Unknown command in A");
    }
  }

  for (; mputlow < mputhi; i++)        // remove any messages left on queue
  {                                    // (from rollback on gets)
    mputlow++;
    sprintf(buff, "Message %d", mputlow);
    Get(buff, mputlow % queues + 1);
  }
}


/*
  StartArrayBackup finds some heap space to store the command array and creates
  another ECB to watch the heap space (through RigorWatchA).
*/

void StartArrayBackup()
{
  pBackupArray = (sABACK *) gsysc(1, "RIGORXX");
  memset(pBackupArray, 0, 500);
  CreateECBs(1, sf("ZTEST MQS1 CASE RIGORBACKUP %p", pBackupArray));
}


/*
  ArrayBackup is called before each command list is executed.  It copies the
  command array into heap memory (pBackupArray), so that another ECB can output
  the commands if anything goes wrong.
*/

void ArrayBackup(sA* pA)
{
  if (pA == &pBackupArray->A) return;  // A and heap share same space

  memset(pBackupArray, 0, sizeof(*pBackupArray));
  pBackupArray->testCase = testcase;
  memcpy(&pBackupArray->A, pA, sizeof(sA));
}


/*
  ArrayUpdate copies the command that is currently being run by RigorRun to heap
  memory (pBackupArray)
*/

void ArrayUpdate(sA* pA)
{
  if (pA == &pBackupArray->A) return;  // A and heap share same space
  pBackupArray->commandNum += 1;
}


/*
  When the testing ECB has completed execution, it puts a -1 in the heap for the
  test case number, so that the ECB watching the heap knows everything completed
  successfully.
*/

void EndArrayBackup()
{
  pBackupArray->testCase = -1;
}


/*
  RigorWatchA is called by the ECB created to watch heap and make sure that
  testing has not ended abnormally.  It does this by looking at the current test
  case and command number that is being executed by the testing ECB (it is stored
  in heap).  If the numbers don't change for awhile (I use 4 seconds), the testing
  ECB has encountered an error and the commands run must be output.  Output is
  done by turning on HelpVar, so MQ APIs are output instead of executed, and then
  by sending RigorRun the command list that was running.  Heap is also left as
  used so that the user can look at the command array directly.  If RigorWatchA
  encounters a test case of -1, the testing ECB has signaled its exit, and heap
  memory can be released.

  All this is necessary because we need to know the commands that were run for a
  randomly generated command list.  It is not enough to know the test number in
  RIGOR3, because it cannot be recreated.
*/

void RigorWatchA()
{
  int currentcase = 0;
  int currentcommand = 0;
  int delaycount = 0;

  out(sf("RigorWatchA start: heap at %p", pBackupArray));

  while ((delaycount < 4) &&
         (pBackupArray->testCase != -1))
  {
    if ((currentcase == pBackupArray->testCase) &&
        (currentcommand == pBackupArray->commandNum))
    {
      delaycount++;
    }
    else
    {
      currentcase = pBackupArray->testCase;
      currentcommand = pBackupArray->commandNum;
      delaycount = 0;
    }

    QMQZ_Sleep(1);
  }

  if (pBackupArray->testCase == -1)
  {
    if (rsysc((void*) pBackupArray, 1, "RIGORXX") == RSYSC_ERROR)
      Error("Error releasing heap storage");
    else
      out("Heap storage released");
  }
  else
  {
    outc(sf("RigorWatchA Timeout: heap at %p", pBackupArray));
    outc(sf("Testing failed on command %d", pBackupArray->commandNum));
    HelpVar = 1;
    RigorRun(&pBackupArray->A);
    oute("END OF CODE");
  }

  exit(0);
}

/*********************************************************************/
/* Function: GetProtectKey                                           */
/* ------------------------                                          */
/* This function will return the current protection key.             */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:                                                 */
/*                                                                   */
/*   None                                                            */
/*                                                                   */
/* Return Value:                                                     */
/*                                                                   */
/*    unsigned char key       - protection key                       */
/*********************************************************************/

unsigned char GetProtectKey()                                                   //d26725
{                                                                               //d26725
  //get the protect key
  cinfc(CINFC_WRITE, CINFC_CM2000);                                             //d26725
  unsigned char key = ecbp2()->ce2okey;                                         //d26725

  //reset the protect key
  accessMQPreset(&key);                                                         //d26725

  return key;                                                                   //d26725
}                                                                               //d26725

/*********************************************************************/
/* Function: GetMemoryInUse                                          */
/* ------------------------                                          */
/* This function will return the memory in use.                      */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:                                                 */
/*                                                                   */
/*   None                                                            */
/*                                                                   */
/* Return Value:                                                     */
/*                                                                   */
/*    unsigned long inUseAtStart   - memory in use                   */
/*********************************************************************/
unsigned long GetMemoryInUse()                                                  //d26725
{                                                                               //d26725
  struct mallinfo mallocStart;                                                  //d26725
  unsigned long inUseAtStart = 0;                                               //d26725

  mallocStart = mallinfo();                                                     //d26725
  inUseAtStart = mallocStart.usmblks + mallocStart.uordblks;                    //d26725

  return inUseAtStart;                                                          //d26725
}                                                                               //d26725

/*********************************************************************/
/*                        Automated zdevo tests helper functions     */
/*********************************************************************/


/*********************************************************************/
/* Function: SetUpQueues                                             */
/* ------------------------                                          */
/* This function will define all queues required by the zdevo        */
/* automated tests.  It will first establish a connection to the     */
/* queue manager and alter the MAXMSGL of queue manager if necessary.*/
/* Then all required queues will be defined if they do not already   */
/* exist.  We verify the queues were created and prep the queues for */
/* puts and gets. If we are unable to create the queues,             */
/* TPF_TC_ERROR is issued to indicate a test case failure.           */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:                                                 */
/*                                                                   */
/*   None                                                            */
/*                                                                   */
/* Return Value:                                                     */
/*                                                                   */
/*    MQ2RC_OK                - normal return, queues were set up    */
/*    MQ2RC_BREAK             - could not finish setting up queues,  */
/*                              due to either an MQOPEN error, an    */
/*                              MQINQ error, MQCLOSE error or        */
/*                              because the queues could not be      */
/*                              defined.                             */
/*                                                                   */
/*********************************************************************/
MQ2RC SetUpQueues()                                                             //d26725
{                                                                               //d26725
  int          i;                                                               //d26725
  int          rc = 0;                                                          //d26725
  unsigned int queuesDefined = 0;                                               //d26725
  MQLONG       Selectors[] = {MQCA_Q_MGR_NAME};                                 //d26725
  MQLONG       QmOptions = MQOO_INQUIRE;                                        //d26725
  MQLONG       QmCurMaxMsgLength[1] = {0};                                      //d26725
  MQOD         ObjDesc = {MQOD_DEFAULT};                                        //d26725
  MQLONG       Reason = MQRC_NONE;                                              //d26725
  MQLONG       CompCode = MQCC_OK;                                              //d26725
  MQHOBJ       QmHobj=MQHO_UNUSABLE_HOBJ;                                       //d26725
  MQOD         QmDesc;                                                          //d26725
  char         errorMsg[30];                                                    //d26725
  char         *defQCommand[3];                                                 //d26725
  char         **qNames;                                                        //d26725
  char         command[100];                                                    //d26725

  SetAutomatedTestCase(TRUE);                                                   //d26725

  // Define queues required for this test case
  if (!common && !compsize)     //memory q, no compression                      //d26725
  {                                                                             //d26725
    defQCommand[0] = "ZMQSC DEFINE QL-MQ_TEST_MEMORY_UNCOMPRESSED_1 "           //d26725
                       "COMPSIZE-0 COMMON-NO MAXMSGL-4000000";                  //d26725
    defQCommand[1] = "ZMQSC DEFINE QL-MQ_TEST_MEMORY_UNCOMPRESSED_2 "           //d26725
                       "COMPSIZE-0 COMMON-NO MAXMSGL-4000000";                  //d26725
    defQCommand[2] = "ZMQSC DEFINE QL-MQ_TEST_MEMORY_UNCOMPRESSED_3 "           //d26725
                       "COMPSIZE-0 COMMON-NO MAXMSGL-4000000";                  //d26725
    qNames = qNamesMemoryUncompressed;                                          //d26725
  }                                                                             //d26725
  else if (!common && compsize) //memory q, compression                         //d26725
  {                                                                             //d26725
    defQCommand[0] = "ZMQSC DEFINE QL-MQ_TEST_MEMORY_COMPRESSED_1 "             //d26725
                       "COMPSIZE-1 COMMON-NO MAXMSGL-4000000";                  //d26725
    defQCommand[1] = "ZMQSC DEFINE QL-MQ_TEST_MEMORY_COMPRESSED_2 "             //d26725
                       "COMPSIZE-1 COMMON-NO MAXMSGL-4000000";                  //d26725
    defQCommand[2] = "ZMQSC DEFINE QL-MQ_TEST_MEMORY_COMPRESSED_3 "             //d26725
                       "COMPSIZE-1 COMMON-NO MAXMSGL-4000000";                  //d26725
    qNames = qNamesMemoryCompressed;                                            //d26725
  }                                                                             //d26725
  else if (common && !compsize) //common q, no compression                      //d26725
  {                                                                             //d26725
    defQCommand[0] = "ZMQSC DEFINE QL-MQ_TEST_COMMON_UNCOMPRESSED_1 "           //d26725
                       "COMPSIZE-0 COMMON-YES MAXMSGL-4000000";                 //d26725
    defQCommand[1] = "ZMQSC DEFINE QL-MQ_TEST_COMMON_UNCOMPRESSED_2 "           //d26725
                       "COMPSIZE-0 COMMON-YES MAXMSGL-4000000";                 //d26725
    defQCommand[2] = "ZMQSC DEFINE QL-MQ_TEST_COMMON_UNCOMPRESSED_3 "           //d26725
                       "COMPSIZE-0 COMMON-YES MAXMSGL-4000000";                 //d26725
    qNames = qNamesCommonUncompressed;                                          //d26725
  }                                                                             //d26725
  else                          //common q, compression                         //d26725
  {                                                                             //d26725
    defQCommand[0] = "ZMQSC DEFINE QL-MQ_TEST_COMMON_COMPRESSED_1 "             //d26725
                       "COMPSIZE-1 COMMON-YES MAXMSGL-4000000";                 //d26725
    defQCommand[1] = "ZMQSC DEFINE QL-MQ_TEST_COMMON_COMPRESSED_2 "             //d26725
                       "COMPSIZE-1 COMMON-YES MAXMSGL-4000000";                 //d26725
    defQCommand[2] = "ZMQSC DEFINE QL-MQ_TEST_COMMON_COMPRESSED_3 "             //d26725
                       "COMPSIZE-1 COMMON-YES MAXMSGL-4000000";                 //d26725
    qNames = qNamesCommonCompressed;                                            //d26725
  }                                                                             //d26725

  queues = 3;                                                                   //d26725

  //set default test case timeout, this may be overriden by individual test cases
  TPF_TC_TIMEOUT(60);                                                           //d26725

  //Store memory in use prior to connecting to the queue manager
  memBeforeConn = GetMemoryInUse();                                             //d26725

  // Connect to Queue Manager.
  MQCONN(QMgrName, &Hconn, &CompCode, &Reason);                                 //d26725
  CheckError("MQCONN", Reason);                                                 //d26725

  // We need to determine if the maxmsgl of the QMGR needs to be modified.
  // Before calling MQINQ to check this, we first need to open the queue manager
  memcpy(&QmDesc, &ObjDesc, sizeof(ObjDesc));                                   //d26725
  QmDesc.ObjectName[0] = '\0';                                                  //d26725
  QmDesc.ObjectType = MQOT_Q_MGR;                                               //d26725
  memcpy(QmDesc.ObjectQMgrName, QMgrName, MQ_Q_MGR_NAME_LENGTH);                //d26725

  MQOPEN(Hconn, &QmDesc, QmOptions, &QmHobj, &CompCode, &Reason);               //d26725

  if (CompCode != MQCC_OK)                                                      //d26725
  {                                                                             //d26725
    sprintf(errorMsg, "MQOPEN on QMGR failed, %d", Reason);                     //d26725
    ZdevoError(errorMsg);                                                       //d26725
    return MQ2RC_BREAK;                                                         //d26725
  }                                                                             //d26725

  // Get name of local queue manager
  MQINQ(Hconn, QmHobj, 1, Selectors, 0, NULL, MQ_Q_MGR_NAME_LENGTH, QMgrName,   //d26725
        &CompCode, &Reason);                                                    //d26725
  Trace("MQINQ");                                                               //d26725

  if (CompCode != MQCC_OK)                                                      //d26725
  {                                                                             //d26725
    sprintf(errorMsg, "MQINQ on QMGR failed, %d", Reason);                      //d26725
    ZdevoError(errorMsg);                                                       //d26725
    return MQ2RC_BREAK;                                                         //d26725
  }                                                                             //d26725

  // if MQOPEN was successful, issue an MQINQ to check value of maxmsgl
  Selectors[0] = MQIA_MAX_MSG_LENGTH;                                           //d26725
  MQINQ(Hconn, QmHobj, 1, Selectors, 1, QmCurMaxMsgLength, 0, NULL,             //d26725
        &CompCode, &Reason);                                                    //d26725
  Trace("MQINQ");                                                               //d26725

  if (CompCode != MQCC_OK)                                                      //d26725
  {                                                                             //d26725
    sprintf(errorMsg, "MQINQ on QMGR failed, %d", Reason);                      //d26725
    ZdevoError(errorMsg);                                                       //d26725
    return MQ2RC_BREAK;                                                         //d26725
  }                                                                             //d26725

  //Close QMGR
  MQCLOSE(Hconn, &QmHobj, 0, &CompCode, &Reason);                               //d26725

  if (CompCode != MQCC_OK)                                                      //d26725
  {                                                                             //d26725
    sprintf(errorMsg, "MQCLOSE on QMGR failed, %d", Reason);                    //d26725
    ZdevoError(errorMsg);                                                       //d26725
    return MQ2RC_BREAK;                                                         //d26725
  }                                                                             //d26725

  if (QmCurMaxMsgLength[0] != 4000000)                                          //d26725
  {                                                                             //d26725
    //if needed, set maxmsgl of qmgr to 4000000
    strcpy (command,"ZMQSC ALT QMGR-");                                         //d26725
    strcat (command,trunc(QMgrName));                                           //d26725
    strcat (command," MAXMSGL-4000000");                                        //d26725
    ExecuteCommand(command);                                                    //d26725
    QMQZ_Sleep(5); //give MQ time to alter qmgr                                 //d26725
  }                                                                             //d26725

  TPF_TC_INFO("Defining Queues");                                               //d26725
  //define queues that do not already exist
  for (i=0;i<queues;i++)                                                        //d26725
  {                                                                             //d26725
    if (rc == 0) {                                                              //d26725
      if (QueueExists(qNames[i]) != MQ2RC_OK)                                   //d26725
      {                                                                         //d26725
        queuesDefined++;                                                        //d26725
        ExecuteCommand(defQCommand[i]);                                         //d26725
      }                                                                         //d26725
      strcpy(TrgtQName[i], qNames[i]);                                          //d26725
    }                                                                           //d26725
  }                                                                             //d26725

  // prep queues for puts or gets
  memcpy(md.Format, MQFMT_STRING, MQ_FORMAT_LENGTH);                            //d26725
  md.CodedCharSetId = CCSID_EBCDIC;                                             //d26725

  if (queuesDefined > 0) //if any new queues were defined                       //d26725
  {                                                                             //d26725
    QMQZ_Sleep(15); //give MQ time to define queues                             //d26725
    //verify all required queues exist, if they do not, issue an error
    for (i=0;i<queues;i++)                                                      //d26725
    {                                                                           //d26725
      if (QueueExists(qNames[i]) != MQ2RC_OK)                                   //d26725
      {                                                                         //d26725
        sprintf(errorMsg, "%s could not be defined.", qNames[i]);               //d26725
        ZdevoError(errorMsg);                                                   //d26725
        return MQ2RC_BREAK;                                                     //d26725
      }                                                                         //d26725
    }                                                                           //d26725
  }                                                                             //d26725
  return MQ2RC_OK;                                                              //d26725
}                                                                               //d26725


/*********************************************************************/
/* Function: QueueExists                                             */
/* ------------------------                                          */
/* This function will check to see if a queue definition exists.  If */
/* it does exist, it will verify that the queue is not put inhibited */
/* and hasn't been deleted.                                          */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:                                                 */
/*                                                                   */
/*   qName                    - name of a queue                      */
/*                                                                   */
/* Return Value:                                                     */
/*                                                                   */
/*    MQ2RC_OK                - specified queue exists               */
/*    MQ2RC_Q_DOES_NOT_EXIST  - specified queue was not found in the */
/*                              queue hash table                     */
/*    MQ2RC_Q_MGR_DOES_NOT_EXIST                                     */
/*                            - queue manager has not been defined   */
/*    MQ2RC_Q_MGR_NOT_STARTED - queue table not loaded into memory   */
/*    MQ2RC_BREAK             - specified queue was deleted or is    */
/*                              put inhibited                        */
/*                                                                   */
/*********************************************************************/
MQ2RC QueueExists(char * qName)                                                 //d26725
{                                                                               //d26725
  cQDEF* pQDef = NULL;                                                          //d26725
  MQ2RC rc = locateQDef(qName, &pQDef, FALSE);                                  //d26725

  if (rc == MQ2RC_OK)                                                           //d26725
  {                                                                             //d26725
    if (pQDef->isDeleted() || pQDef->isPutInhibited())                          //d26725
    {                                                                           //d26725
      rc = MQ2RC_BREAK;                                                         //d26725
    }                                                                           //d26725
  }                                                                             //d26725
  return rc;                                                                    //d26725
}                                                                               //d26725

/*********************************************************************/
/* Function: ExecuteCommand                                          */
/* ------------------------                                          */
/* This function will execute a TPF command.                         */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:                                                 */
/*                                                                   */
/*   char * command           - command to be executed               */
/*                                                                   */
/* Return Value:                                                     */
/*                                                                   */
/*   None                                                            */
/*********************************************************************/
void ExecuteCommand(char * command)                                             //d26725
{                                                                               //d26725
  struct swisc_immediate_input swisc_is;                                        //d26725
  void     *temp;                                                               //d26725
  struct   mi0mi* message;                                                      //d26725

  swisc_is.istream = SWISC_IS_MAIN;                                             //d26725
  swisc_immediate(&swisc_is);                                                   //d26725
  temp = getcc (D2,GETCC_TYPE,L0);                                              //d26725
  message = (struct mi0mi*) temp;                                               //d26725
  memset (message->mi0acc,0,sizeof(message->mi0acc));                           //d26725
  memcpy (message->mi0acc,command,strlen(command));                             //d26725
  message->mi0adr=PRIME_CRAS;                                                   //d26725
  message->mi0cct = (strlen(command) + ROUTE_LEN + 1);                          //d26725
  creec(0,0,CVAA,D2,CREEC_IMMEDIATE);                                           //d26725
}                                                                               //d26725

/*********************************************************************/
/* Function: IsTestCaseValid                                         */
/* ------------------------                                          */
/* If a test case requires compression support, this function will   */
/* check to see if compression is supported by the processor.        */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:                                                 */
/*                                                                   */
/* None                                                              */
/*                                                                   */
/* Return Value:                                                     */
/*                                                                   */
/*   TRUE  - if compression is supported or if the test case does    */
/*           not use compression.                                    */
/*   FALSE - if compression is not supported and the test case does  */
/*           use compression.                                        */
/*********************************************************************/
boolean IsTestCaseValid()                                                       //d26725
{                                                                               //d26725
  boolean   rc   = TRUE;                                                        //d26725
  cMQP_MEM* pMQP = (cMQP_MEM*)accessMQPforRead();                               //d26725

  if (compsize && !pMQP->isCompressionEnabled())                                //d26725
  {                                                                             //d26725
    //if queue compression is not supported by processor, skip these test cases
    rc = FALSE;                                                                 //d26725
  }                                                                             //d26725
  return rc;                                                                    //d26725
}                                                                               //d26725

/*********************************************************************/
/* Function: ZdevoError                                              */
/* ------------------------                                          */
/* When an error is encountered in an automated test case,           */
/* ZdevoError() will be called.  This function will call             */
/* TPF_TC_ERROR() and undo all transactions.                         */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:                                                 */
/*                                                                   */
/* char *ErrorText - text of error message                           */
/*                                                                   */
/* Return Value:                                                     */
/*                                                                   */
/* None                                                              */
/*********************************************************************/
void ZdevoError(char *ErrorText)                                                //d26725
{                                                                               //d26725
  TPF_TC_ERROR(ErrorText);                                                      //d26725
  while (!tx_rollback()) {}            // undo all transactions                 //d26725
}                                                                               //d26725


/*********************************************************************/
/*                        Automated zdevo test cases                 */
/*********************************************************************/

TPF_TESTSUITE("ibm.mq.localqueues",                                             //d26725
              "CMQS,CMQO,CMQI,CMQU,CMQR,CUIB,CUIF,CUIX,CUIY,CUSC")              //d26725

//test case 1
TPF_TESTCASE(mq_put_in_root_get_in_root_memory_uncompressed_1,                  //d26725
             "Put uncompressed message on memory queue and get the message "    //d26725
             "within a commit scope.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725
  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 1;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_uncompressed_1,                  //d26725
             "Put uncompressed message on common queue and get the message "    //d26725
             "within a commit scope.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 1;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_memory_compressed_1,                    //d26725
             "Put compressed message on memory queue and get the message "      //d26725
             "within a commit scope.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 1;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_compressed_1,                    //d26725
             "Put compressed message on common queue and get the message "      //d26725
             "within a commit scope.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 1;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 2
TPF_TESTCASE(mq_put_in_root_get_in_root_memory_uncompressed_2,                  //d26725
             "Put uncompressed message on memory queue in commit scope, get "   //d26725
             "in nested commit scope, tx_commit, then verify another mqget "    //d26725
             "fails with no message.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 2;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_uncompressed_2,                  //d26725
             "Put uncompressed message on common queue in commit scope, get "   //d26725
             "in nested commit scope, tx_commit, then verify another mqget "    //d26725
             "fails with no message.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 2;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_memory_compressed_2,                    //d26725
             "Put compressed message on memory queue in commit scope, get "     //d26725
             "in nested commit scope, tx_commit, then verify another mqget "    //d26725
             "fails with no message.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 2;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_compressed_2,                    //d26725
             "Put compressed message on common queue in commit scope, get "     //d26725
             "in nested commit scope, tx_commit, then verify another mqget "    //d26725
             "fails with no message.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 2;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 3
TPF_TESTCASE(mq_put_in_root_get_in_root_memory_uncompressed_3,                  //d26725
             "Put uncompressed message on memory queue in commit scope, get "   //d26725
             "in nested commit scope, tx_rollback, then verify another mqget "  //d26725
             "retrieves message.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 3;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_uncompressed_3,                  //d26725
             "Put uncompressed message on common queue in commit scope, get "   //d26725
             "in nested commit scope, tx_rollback, then verify another mqget "  //d26725
             "retrieves message.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 3;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_memory_compressed_3,                    //d26725
             "Put compressed message on memory queue in commit scope, get "     //d26725
             "in nested commit scope, tx_rollback, then verify another mqget "  //d26725
             "retrieves message.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 3;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_compressed_3,                    //d26725
             "Put compressed message on common queue in commit scope, get "     //d26725
             "in nested commit scope, tx_rollback, then verify another mqget "  //d26725
             "retrieves message.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 3;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 4
TPF_TESTCASE(mq_put_in_root_get_in_root_memory_uncompressed_4,                  //d26725
             "Put uncompressed message on memory queue in commit scope, put "   //d26725
             "second message in nested commit scope, get in nested commit "     //d26725
             "scope, then verify another mqget retrieves message in root "      //d26725
             "commit scope.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 4;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_uncompressed_4,                  //d26725
             "Put uncompressed message on common queue in commit scope, put "   //d26725
             "second message in nested commit scope, get in nested commit "     //d26725
             "scope, then verify another mqget retrieves message in root "      //d26725
             "commit scope.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 4;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_memory_compressed_4,                    //d26725
             "Put compressed message on memory queue in commit scope, put "     //d26725
             "second message in nested commit scope, get in nested commit "     //d26725
             "scope, then verify another mqget retrieves message in root "      //d26725
             "commit scope.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 4;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_compressed_4,                    //d26725
             "Put compressed message on common queue in commit scope, put "     //d26725
             "second message in nested commit scope, get in nested commit "     //d26725
             "scope, then verify another mqget retrieves message in root "      //d26725
             "commit scope.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 4;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 5
TPF_TESTCASE(mq_put_in_root_get_in_root_memory_uncompressed_5,                  //d26725
             "Put uncompressed message on memory queue in commit scope, put "   //d26725
             "second message in nested commit scope, get in nested commit "     //d26725
             "scope, tx_rollback, then verify another mqget retrieves message " //d26725
             "in root commit scope.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 5;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_uncompressed_5,                  //d26725
             "Put uncompressed message on common queue in commit scope, put "   //d26725
             "second message in nested commit scope, get in nested commit "     //d26725
             "scope, tx_rollback, then verify another mqget retrieves message " //d26725
             "in root commit scope.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 5;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_memory_compressed_5,                    //d26725
             "Put compressed message on memory queue in commit scope, put "     //d26725
             "second message in nested commit scope, get in nested commit "     //d26725
             "scope, tx_rollback, then verify another mqget retrieves message " //d26725
             "in root commit scope.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 5;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_compressed_5,                    //d26725
             "Put compressed message on common queue in commit scope, put "     //d26725
             "second message in nested commit scope, get in nested commit "     //d26725
             "scope, tx_rollback, then verify another mqget retrieves message " //d26725
             "in root commit scope.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 5;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 6
TPF_TESTCASE(mq_put_in_root_get_in_root_memory_uncompressed_6,                  //d26725
             "Put uncompressed message on memory queue in commit scope, put "   //d26725
             "second message in nested commit scope, then verify two mqgets "   //d26725
             "in the root commit scope retrieve both messages.")                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 6;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_uncompressed_6,                  //d26725
             "Put uncompressed message on common queue in commit scope, put "   //d26725
             "second message in nested commit scope, then verify two mqgets "   //d26725
             "in the root commit scope retrieve both messages.")                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 6;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_memory_compressed_6,                    //d26725
             "Put compressed message on memory queue in commit scope, put "     //d26725
             "second message in nested commit scope, then verify two mqgets "   //d26725
             "in the root commit scope retrieve both messages.")                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 6;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_compressed_6,                    //d26725
             "Put compressed message on common queue in commit scope, put "     //d26725
             "second message in nested commit scope, then verify two mqgets "   //d26725
             "in the root commit scope retrieve both messages.")                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 6;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 7
TPF_TESTCASE(mq_put_in_root_get_in_root_memory_uncompressed_7,                  //d26725
             "Put uncompressed message on memory queue in commit scope, put "   //d26725
             "second message in nested commit scope, tx_rollback, verify we "   //d26725
             "can get the first message from the queue in the root commit "     //d26725
             "scope, then verify a second get will fail with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 7;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_uncompressed_7,                  //d26725
             "Put uncompressed message on common queue in commit scope, put "   //d26725
             "second message in nested commit scope, tx_rollback, verify we "   //d26725
             "can get the first message from the queue in the root commit "     //d26725
             "scope, then verify a second get will fail with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 7;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_memory_compressed_7,                    //d26725
             "Put compressed message on memory queue in commit scope, put "     //d26725
             "second message in nested commit scope, tx_rollback, verify we "   //d26725
             "can get the first message from the queue in the root commit "     //d26725
             "scope, then verify a second get will fail with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 7;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_root_common_compressed_7,                    //d26725
             "Put compressed message on common queue in commit scope, put "     //d26725
             "second message in nested commit scope, tx_rollback, verify we "   //d26725
             "can get the first message from the queue in the root commit "     //d26725
             "scope, then verify a second get will fail with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 7;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 8
TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_uncompressed_8,                  //d26725
             "Put an uncompressed message on a memory queue in root commit "    //d26725
             "scope, get in nested commit scope")                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 8;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_uncompressed_8,                  //d26725
             "Put an uncompressed message on a common queue in root commit "    //d26725
             "scope, get in nested commit scope")                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 8;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_compressed_8,                    //d26725
             "Put a compressed message on a memory queue in root commit "       //d26725
             "scope, get in nested commit scope")                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 8;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_compressed_8,                    //d26725
             "Put a compressed message on a common queue in root commit "       //d26725
             "scope, get in nested commit scope")                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 8;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 9
TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_uncompressed_9,                  //d26725
             "Put uncompressed message on a memory queue in root commit "       //d26725
             "scope, put another message in third level of nested commit "      //d26725
             "scope, get both messages in second level of nested commit scope.")//d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 9;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_uncompressed_9,                  //d26725
             "Put uncompressed message on a common queue in root commit "       //d26725
             "scope, put another message in third level of nested commit "      //d26725
             "scope, get both messages in second level of nested commit scope.")//d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 9;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_compressed_9,                    //d26725
             "Put compressed message on a memory queue in root commit "         //d26725
             "scope, put another message in third level of nested commit "      //d26725
             "scope, get both messages in second level of nested commit scope.")//d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 9;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_compressed_9,                    //d26725
             "Put compressed message on a common queue in root commit "         //d26725
             "scope, put another message in third level of nested commit "      //d26725
             "scope, get both messages in second level of nested commit scope.")//d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 9;                                                             //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 10
TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_uncompressed_10,                 //d26725
             "Put uncompressed message on a memory queue in root commit "       //d26725
             "scope, put another message in third level of nested commit "      //d26725
             "scope, tx_rollback, get message in second level of nested "       //d26725
             "commit scope, then verify second get fails with no message.")     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 10;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_uncompressed_10,                 //d26725
             "Put uncompressed message on a common queue in root commit "       //d26725
             "scope, put another message in third level of nested commit "      //d26725
             "scope, tx_rollback, get message in second level of nested "       //d26725
             "commit scope, then verify second get fails with no message.")     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 10;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_compressed_10,                   //d26725
             "Put compressed message on a memory queue in root commit "         //d26725
             "scope, put another message in third level of nested commit "      //d26725
             "scope, tx_rollback, get message in second level of nested "       //d26725
             "commit scope, then verify second get fails with no message.")     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 10;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_compressed_10,                   //d26725
             "Put compressed message on a common queue in root commit "         //d26725
             "scope, put another message in third level of nested commit "      //d26725
             "scope, tx_rollback, get message in second level of nested "       //d26725
             "commit scope, then verify second get fails with no message.")     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 10;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 11
TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_uncompressed_11,                 //d26725
             "Put uncompressed message on a memory queue in root commit "       //d26725
             "scope, get in third level of nested commit scope, tx_rollback, "  //d26725
             "get message in second level of nested commit scope, then verify " //d26725
             "second get fails with no message")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 11;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_uncompressed_11,                 //d26725
             "Put uncompressed message on a common queue in root commit "       //d26725
             "scope, get in third level of nested commit scope, tx_rollback, "  //d26725
             "get message in second level of nested commit scope, then verify " //d26725
             "second get fails with no message")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 11;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_compressed_11,                   //d26725
             "Put compressed message on a memory queue in root commit "         //d26725
             "scope, get in third level of nested commit scope, tx_rollback, "  //d26725
             "get message in second level of nested commit scope, then verify " //d26725
             "second get fails with no message")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 11;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_compressed_11,                   //d26725
             "Put compressed message on a common queue in root commit "         //d26725
             "scope, get in third level of nested commit scope, tx_rollback, "  //d26725
             "get message in second level of nested commit scope, then verify " //d26725
             "second get fails with no message")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 11;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 12
TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_uncompressed_12,                 //d26725
             "Put uncompressed message on a memory queue in root commit "       //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "then verify second get fails with no message in second level "    //d26725
             "of nested commit scope.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 12;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_uncompressed_12,                 //d26725
             "Put uncompressed message on a common queue in root commit "       //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "then verify second get fails with no message in second level "    //d26725
             "of nested commit scope.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 12;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_memory_compressed_12,                   //d26725
             "Put compressed message on a memory queue in root commit "         //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "then verify second get fails with no message in second level "    //d26725
             "of nested commit scope.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 12;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_in_nest_common_compressed_12,                   //d26725
             "Put compressed message on a common queue in root commit "         //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "then verify second get fails with no message in second level "    //d26725
             "of nested commit scope.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 12;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 13
TPF_TESTCASE(mq_put_in_nest_get_in_nest_memory_uncompressed_13,                 //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope, put second message in third level of nested commit scope, "//d26725
             "then verify we can get both messages in second level of nested "  //d26725
             "commit scope.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 13;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_common_uncompressed_13,                 //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope, put second message in third level of nested commit scope, "//d26725
             "then verify we can get both messages in second level of nested "  //d26725
             "commit scope.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 13;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_memory_compressed_13,                   //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, put second message in third level of nested commit scope, "//d26725
             "then verify we can get both messages in second level of nested "  //d26725
             "commit scope.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 13;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_common_compressed_13,                   //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, put second message in third level of nested commit scope, "//d26725
             "then verify we can get both messages in second level of nested "  //d26725
             "commit scope.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 13;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 14
TPF_TESTCASE(mq_put_in_nest_get_in_nest_memory_uncompressed_14,                 //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope, put second message in third level of nested commit scope, "//d26725
             "tx_rollback, verify we can get the first message in second level "//d26725
             "of nested commit scope, and verify a second get fails with no "   //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 14;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_common_uncompressed_14,                 //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope, put second message in third level of nested commit scope, "//d26725
             "tx_rollback, verify we can get the first message in second level "//d26725
             "of nested commit scope, and verify a second get fails with no "   //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 14;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_memory_compressed_14,                   //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, put second message in third level of nested commit scope, "//d26725
             "tx_rollback, verify we can get the first message in second level "//d26725
             "of nested commit scope, and verify a second get fails with no "   //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 14;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_common_compressed_14,                   //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope, put second message in third level of nested commit scope, "//d26725
             "tx_rollback, verify we can get the first message in second level "//d26725
             "of nested commit scope, and verify a second get fails with no "   //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 14;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725


//test case 15
TPF_TESTCASE(mq_put_in_nest_get_in_nest_memory_uncompressed_15,                 //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "verify a second get fails with no message in second level of "    //d26725
             "nested commit scope.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 15;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_common_uncompressed_15,                 //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "verify a second get fails with no message in second level of "    //d26725
             "nested commit scope.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 15;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_memory_compressed_15,                   //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "verify a second get fails with no message in second level of "    //d26725
             "nested commit scope.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 15;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_common_compressed_15,                   //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "verify a second get fails with no message in second level of "    //d26725
             "nested commit scope.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 15;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 16
TPF_TESTCASE(mq_put_in_nest_get_in_nest_memory_uncompressed_16,                 //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "tx_rollback, get the message again in the second level of "       //d26725
             "nested commit scope.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 16;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_common_uncompressed_16,                 //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "tx_rollback, get the message again in the second level of "       //d26725
             "nested commit scope.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 16;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_memory_compressed_16,                   //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "tx_rollback, get the message again in the second level of "       //d26725
             "nested commit scope.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 16;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_nest_common_compressed_16,                   //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope, get message in third level of nested commit scope, "       //d26725
             "tx_rollback, get the message again in the second level of "       //d26725
             "nested commit scope.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 16;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 17
TPF_TESTCASE(mq_put_in_nest_get_in_root_memory_uncompressed_17,                 //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope, get in root commit scope.")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 17;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_common_uncompressed_17,                 //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope, get in root commit scope.")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 17;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_memory_compressed_17,                   //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, get in root commit scope.")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 17;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_common_compressed_17,                   //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope, get in root commit scope.")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 17;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 18
TPF_TESTCASE(mq_put_in_nest_get_in_root_memory_uncompressed_18,                 //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope, tx_rollback, verify get in root commit scope results "     //d26725
             "in no msg available.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 18;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_common_uncompressed_18,                 //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope, tx_rollback, verify get in root commit scope results "     //d26725
             "in no msg available.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 18;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_memory_compressed_18,                   //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, tx_rollback, verify get in root commit scope results "     //d26725
             "in no msg available.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 18;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_common_compressed_18,                   //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope, tx_rollback, verify get in root commit scope results "     //d26725
             "in no msg available.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 18;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 19
TPF_TESTCASE(mq_put_in_nest_get_in_root_memory_uncompressed_19,                 //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope, put another message in the root commit scope, then "       //d26725
             "verify we can get both messages in root commit scope.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 19;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_common_uncompressed_19,                 //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope, put another message in the root commit scope, then "       //d26725
             "verify we can get both messages in root commit scope.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 19;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_memory_compressed_19,                   //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, put another message in the root commit scope, then "       //d26725
             "verify we can get both messages in root commit scope.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 19;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_common_compressed_19,                   //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope, put another message in the root commit scope, then "       //d26725
             "verify we can get both messages in root commit scope.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 19;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 20
TPF_TESTCASE(mq_put_in_nest_get_in_root_memory_uncompressed_20,                 //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope, tx_rollback, put another message in the root commit "      //d26725
             "scope, then verify we can get second message in root commit "     //d26725
             "scope.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 20;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_common_uncompressed_20,                 //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope, tx_rollback, put another message in the root commit "      //d26725
             "scope, then verify we can get second message in root commit "     //d26725
             "scope.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 20;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_memory_compressed_20,                   //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, tx_rollback, put another message in the root commit "      //d26725
             "scope, then verify we can get second message in root commit "     //d26725
             "scope.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 20;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_in_root_common_compressed_20,                   //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope, tx_rollback, put another message in the root commit "      //d26725
             "scope, then verify we can get second message in root commit "     //d26725
             "scope.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 20;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 21
TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_uncompressed_21,               //d26725
             "Put an uncompressed message on a memory queue out of commit "     //d26725
             "scope, get in root commit scope")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 21;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_uncompressed_21,               //d26725
             "Put an uncompressed message on a common queue out of commit "     //d26725
             "scope, get in root commit scope")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 21;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_compressed_21,                 //d26725
             "Put a compressed message on a memory queue out of commit "        //d26725
             "scope, get in root commit scope")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 21;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_compressed_21,                 //d26725
             "Put a compressed message on a common queue out of commit "        //d26725
             "scope, get in root commit scope")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 21;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 22
TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_uncompressed_22,               //d26725
             "Put uncompressed message on a memory queue out of commit "        //d26725
             "scope, put another message on second level of nested commit "     //d26725
             "scope,  get both messages in root commit scope.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 22;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_uncompressed_22,               //d26725
             "Put uncompressed message on a common queue out of commit "        //d26725
             "scope, put another message on second level of nested commit "     //d26725
             "scope,  get both messages in root commit scope.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 22;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_compressed_22,                 //d26725
             "Put compressed message on a memory queue out of commit scope, "   //d26725
             "put another message on second level of nested commit scope,  "    //d26725
             "get both messages in root commit scope.")                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 22;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_compressed_22,                 //d26725
             "Put compressed message on a common queue out of commit scope, "   //d26725
             "put another message on second level of nested commit scope,  "    //d26725
             "get both messages in root commit scope.")                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 22;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 23
TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_uncompressed_23,               //d26725
             "Put uncompressed message on a memory queue out of commit "        //d26725
             "scope, put another message on second level of nested commit "     //d26725
             "scope, tx_rollback, get first messages in root commit scope, "    //d26725
             "then verify second get fails with no message.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 23;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_uncompressed_23,               //d26725
             "Put uncompressed message on a common queue out of commit "        //d26725
             "scope, put another message on second level of nested commit "     //d26725
             "scope, tx_rollback, get first messages in root commit scope, "    //d26725
             "then verify second get fails with no message.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 23;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_compressed_23,                 //d26725
             "Put compressed message on a memory queue out of commit scope, "   //d26725
             "put another message on second level of nested commit scope, "     //d26725
             "tx_rollback, get first messages in root commit scope, then "      //d26725
             "verify second get fails with no message.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 23;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_compressed_23,                 //d26725
             "Put compressed message on a common queue out of commit scope, "   //d26725
             "put another message on second level of nested commit scope, "     //d26725
             "tx_rollback, get first messages in root commit scope, then "      //d26725
             "verify second get fails with no message.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 23;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 24
TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_uncompressed_24,               //d26725
             "Put uncompressed message on a memory queue out of commit "        //d26725
             "scope, get message in second level of nested commit scope, "      //d26725
             "tx_rollback, get message again in root commit scope, then "       //d26725
             "verify another get fails with no message.")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 24;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_uncompressed_24,               //d26725
             "Put uncompressed message on a common queue out of commit "        //d26725
             "scope, get message in second level of nested commit scope, "      //d26725
             "tx_rollback, get message again in root commit scope, then "       //d26725
             "verify another get fails with no message.")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 24;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_compressed_24,                 //d26725
             "Put compressed message on a memory queue out of commit scope, "   //d26725
             "get message in second level of nested commit scope, "             //d26725
             "tx_rollback, get message again in root commit scope, then "       //d26725
             "verify another get fails with no message.")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 24;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_compressed_24,                 //d26725
             "Put compressed message on a common queue out of commit scope, "   //d26725
             "get message in second level of nested commit scope, "             //d26725
             "tx_rollback, get message again in root commit scope, then "       //d26725
             "verify another get fails with no message.")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 24;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 25
TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_uncompressed_25,               //d26725
             "Put uncompressed message on a memory queue out of commit "        //d26725
             "scope, get message in second level of nested commit scope, "      //d26725
             "then verify another get in the root commit scope fails with "     //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 25;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_uncompressed_25,               //d26725
             "Put uncompressed message on a common queue out of commit "        //d26725
             "scope, get message in second level of nested commit scope, "      //d26725
             "then verify another get in the root commit scope fails with "     //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 25;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_memory_compressed_25,                 //d26725
             "Put compressed message on a memory queue out of commit scope, "   //d26725
             "get message in second level of nested commit scope, then "        //d26725
             "verify another get in the root commit scope fails with no "       //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 25;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_in_root_common_compressed_25,                 //d26725
             "Put compressed message on a common queue out of commit scope, "   //d26725
             "get message in second level of nested commit scope, then "        //d26725
             "verify another get in the root commit scope fails with no "       //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 25;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 26
TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_uncompressed_26,               //d26725
             "Put uncompressed message on a memory queue out of commit "        //d26725
             "scope, get in second level of nested commit scope.")              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 26;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_uncompressed_26,               //d26725
             "Put uncompressed message on a common queue out of commit "        //d26725
             "scope, get in second level of nested commit scope.")              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 26;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_compressed_26,                 //d26725
             "Put compressed message on a memory queue out of commit scope, "   //d26725
             "get in second level of nested commit scope.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 26;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_compressed_26,                 //d26725
             "Put compressed message on a common queue out of commit scope, "   //d26725
             "get in second level of nested commit scope.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 26;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 27
TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_uncompressed_27,               //d26725
             "Put uncompressed message on a memory queue out of commit "        //d26725
             "scope, put another message in root commit scope, get both "       //d26725
             "messages in second level of nested commit scope.")                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 27;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_uncompressed_27,               //d26725
             "Put uncompressed message on a common queue out of commit "        //d26725
             "scope, put another message in root commit scope, get both "       //d26725
             "messages in second level of nested commit scope.")                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 27;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_compressed_27,                 //d26725
             "Put compressed message on a memory queue out of commit scope, "   //d26725
             "put another message in root commit scope, get both messages "     //d26725
             "in second level of nested commit scope.")                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 27;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_compressed_27,                 //d26725
             "Put compressed message on a common queue out of commit scope, "   //d26725
             "put another message in root commit scope, get both messages "     //d26725
             "in second level of nested commit scope.")                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 27;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 28
TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_uncompressed_28,               //d26725
             "Put uncompressed message on a memory queue in root commit "       //d26725
             "scope, get out of commit scope.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 28;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_uncompressed_28,               //d26725
             "Put uncompressed message on a common queue in root commit "       //d26725
             "scope, get out of commit scope.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 28;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_compressed_28,                 //d26725
             "Put compressed message on a memory queue in root commit "         //d26725
             "scope, get out of commit scope.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 28;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_compressed_28,                 //d26725
             "Put compressed message on a common queue in root commit "         //d26725
             "scope, get out of commit scope.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 28;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 29
TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_uncompressed_29,               //d26725
             "Put uncompressed message on a memory queue in root commit "       //d26725
             "scope, tx_rollback, verify get out of commit scope fails with "   //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 29;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_uncompressed_29,               //d26725
             "Put uncompressed message on a common queue in root commit "       //d26725
             "scope, tx_rollback, verify get out of commit scope fails with "   //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 29;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_compressed_29,                 //d26725
             "Put compressed message on a memory queue in root commit "         //d26725
             "scope, tx_rollback, verify get out of commit scope fails with "   //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 29;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_compressed_29,                 //d26725
             "Put compressed message on a common queue in root commit "         //d26725
             "scope, tx_rollback, verify get out of commit scope fails with "   //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 29;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 30
TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_uncompressed_30,               //d26725
             "Put uncompressed message on a memory queue in root commit "       //d26725
             "scope, put another message out of commit scope, verify two "      //d26725
             "gets out of commit scope will retrieve both messages.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 30;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_uncompressed_30,               //d26725
             "Put uncompressed message on a common queue in root commit "       //d26725
             "scope, put another message out of commit scope, verify two "      //d26725
             "gets out of commit scope will retrieve both messages.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 30;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_compressed_30,                 //d26725
             "Put compressed message on a memory queue in root commit "         //d26725
             "scope, put another message out of commit scope, verify two "      //d26725
             "gets out of commit scope will retrieve both messages.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 30;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_compressed_30,                 //d26725
             "Put compressed message on a common queue in root commit "         //d26725
             "scope, put another message out of commit scope, verify two "      //d26725
             "gets out of commit scope will retrieve both messages.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 30;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 31
TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_uncompressed_31,               //d26725
             "Put uncompressed message on a memory queue in root commit "       //d26725
             "scope, tx_rollback, put another message out of commit scope,  "   //d26725
             "verify get out of commit scope will retrieve message.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 31;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_uncompressed_31,               //d26725
             "Put uncompressed message on a common queue in root commit "       //d26725
             "scope, tx_rollback, put another message out of commit scope,  "   //d26725
             "verify get out of commit scope will retrieve message.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 31;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_memory_compressed_31,                 //d26725
             "Put compressed message on a memory queue in root commit "         //d26725
             "scope, tx_rollback, put another message out of commit scope,  "   //d26725
             "verify get out of commit scope will retrieve message.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 31;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_root_get_out_scope_common_compressed_31,                 //d26725
             "Put compressed message on a common queue in root commit "         //d26725
             "scope, tx_rollback, put another message out of commit scope,  "   //d26725
             "verify get out of commit scope will retrieve message.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 31;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 32
TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_uncompressed_32,               //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope, get out of commit scope.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 32;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_uncompressed_32,               //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope, get out of commit scope.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 32;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_compressed_32,                 //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope, get out of commit scope.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 32;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_compressed_32,                 //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope, get out of commit scope.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 32;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725


//test case 33
TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_uncompressed_33,               //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope.  Commit the message, tx_rollback, then verify a get "      //d26725
             "out of commit scope will fail with no message.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 33;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_uncompressed_33,               //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope.  Commit the message, tx_rollback, then verify a get "      //d26725
             "out of commit scope will fail with no message.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 33;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_compressed_33,                 //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope.  Commit the message, tx_rollback, then verify a get "      //d26725
             "out of commit scope will fail with no message.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 33;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_compressed_33,                 //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope.  Commit the message, tx_rollback, then verify a get "      //d26725
             "out of commit scope will fail with no message.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 33;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725


//test case 34
TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_uncompressed_34,               //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope.  Rollback, then verify a get out of commit scope will "    //d26725
             "fail with no message.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 34;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_uncompressed_34,               //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope.  Rollback, then verify a get out of commit scope will "    //d26725
             "fail with no message.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 34;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_compressed_34,                 //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope.  Rollback, then verify a get out of commit scope will "    //d26725
             "fail with no message.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 34;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_compressed_34,                 //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope.  Rollback, then verify a get out of commit scope will "    //d26725
             "fail with no message.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 34;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725


//test case 35
TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_uncompressed_35,               //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope.  tx_rollback twice, then verify a get out of commit "      //d26725
             "scope will fail with no message.")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 35;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_uncompressed_35,               //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope.  tx_rollback twice, then verify a get out of commit "      //d26725
             "scope will fail with no message.")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 35;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_compressed_35,                 //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope.  tx_rollback twice, then verify a get out of commit "      //d26725
             "scope will fail with no message.")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 35;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_compressed_35,                 //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope.  tx_rollback twice, then verify a get out of commit "      //d26725
             "scope will fail with no message.")                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 35;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725


//test case 36
TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_uncompressed_36,               //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope.  Put another message in the root commit scope.  Verify "   //d26725
             "two gets out of the commit scope will retrieve both messages.")   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 36;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_uncompressed_36,               //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope.  Put another message in the root commit scope.  Verify "   //d26725
             "two gets out of the commit scope will retrieve both messages.")   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 36;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_compressed_36,                 //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope.  Put another message in the root commit scope.  Verify "   //d26725
             "two gets out of the commit scope will retrieve both messages.")   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 36;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_compressed_36,                 //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope.  Put another message in the root commit scope.  Verify "   //d26725
             "two gets out of the commit scope will retrieve both messages.")   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 36;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725


//test case 37
TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_uncompressed_37,               //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope.  Put another message in the root commit scope.  "          //d26725
             "tx_rollback, then verify a get out of the commit scope will "     //d26725
             "fail with no message.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 37;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_uncompressed_37,               //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope.  Put another message in the root commit scope.  "          //d26725
             "tx_rollback, then verify a get out of the commit scope will "     //d26725
             "fail with no message.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 37;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_compressed_37,                 //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope.  Put another message in the root commit scope.  "          //d26725
             "tx_rollback, then verify a get out of the commit scope will "     //d26725
             "fail with no message.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 37;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_compressed_37,                 //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope.  Put another message in the root commit scope.  "          //d26725
             "tx_rollback, then verify a get out of the commit scope will "     //d26725
             "fail with no message.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 37;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 38
TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_uncompressed_38,               //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope. tx_rollback, then put another message in the root "        //d26725
             "commit scope. Get the first message.  Verify a second get "       //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 38;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_uncompressed_38,               //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope. tx_rollback, then put another message in the root "        //d26725
             "commit scope. Get the first message.  Verify a second get "       //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 38;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_compressed_38,                 //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope. tx_rollback, then put another message in the root "        //d26725
             "commit scope. Get the first message.  Verify a second get "       //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 38;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_compressed_38,                 //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope. tx_rollback, then put another message in the root "        //d26725
             "commit scope. Get the first message.  Verify a second get "       //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 38;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725


//test case 39
TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_uncompressed_39,               //d26725
             "Put uncompressed message on a memory queue in nested commit "     //d26725
             "scope. tx_rollback, then put another message in the root "        //d26725
             "commit scope. tx_rollback, then verify a get will fail with "     //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 39;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_uncompressed_39,               //d26725
             "Put uncompressed message on a common queue in nested commit "     //d26725
             "scope. tx_rollback, then put another message in the root "        //d26725
             "commit scope. tx_rollback, then verify a get will fail with "     //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 39;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_memory_compressed_39,                 //d26725
             "Put compressed message on a memory queue in nested commit "       //d26725
             "scope. tx_rollback, then put another message in the root "        //d26725
             "commit scope. tx_rollback, then verify a get will fail with "     //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 39;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_in_nest_get_out_scope_common_compressed_39,                 //d26725
             "Put compressed message on a common queue in nested commit "       //d26725
             "scope. tx_rollback, then put another message in the root "        //d26725
             "commit scope. tx_rollback, then verify a get will fail with "     //d26725
             "no message.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 39;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725


//test case 40
TPF_TESTCASE(mq_put_out_scope_get_out_scope_memory_uncompressed_40,             //d26725
             "Put an uncompressed message on a memory queue out of commit  "    //d26725
             "scope, get in commit scope. Verify a second get out of commit "   //d26725
             "scope fails with no message.")                                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 40;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_common_uncompressed_40,             //d26725
             "Put an uncompressed message on a common queue out of commit  "    //d26725
             "scope, get in commit scope. Verify a second get out of commit "   //d26725
             "scope fails with no message.")                                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 40;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_memory_compressed_40,               //d26725
             "Put an compressed message on a memory queue out of commit  "      //d26725
             "scope, get in commit scope. Verify a second get out of commit "   //d26725
             "scope fails with no message.")                                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 40;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_common_compressed_40,               //d26725
             "Put an compressed message on a common queue out of commit  "      //d26725
             "scope, get in commit scope. Verify a second get out of commit "   //d26725
             "scope fails with no message.")                                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 40;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 41
TPF_TESTCASE(mq_put_out_scope_get_out_scope_memory_uncompressed_41,             //d26725
             "Put an uncompressed message on a memory queue out of commit  "    //d26725
             "scope, get in commit scope. tx_rollback, then verify a second "   //d26725
             "get out of commit scope fails with no message.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 41;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_common_uncompressed_41,             //d26725
             "Put an uncompressed message on a common queue out of commit  "    //d26725
             "scope, get in commit scope. tx_rollback, then verify a second "   //d26725
             "get out of commit scope fails with no message.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 41;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_memory_compressed_41,               //d26725
             "Put an compressed message on a memory queue out of commit  "      //d26725
             "scope, get in commit scope. tx_rollback, then verify a second "   //d26725
             "get out of commit scope fails with no message.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 41;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_common_compressed_41,               //d26725
             "Put an compressed message on a common queue out of commit  "      //d26725
             "scope, get in commit scope. tx_rollback, then verify a second "   //d26725
             "get out of commit scope fails with no message.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 41;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 42
TPF_TESTCASE(mq_put_out_scope_get_out_scope_memory_uncompressed_42,             //d26725
             "Put an uncompressed message on a memory queue out of commit  "    //d26725
             "scope, put another message in commit scope. Verify two gets "     //d26725
             "out of commit scope will get both messages.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 42;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_common_uncompressed_42,             //d26725
             "Put an uncompressed message on a common queue out of commit  "    //d26725
             "scope, put another message in commit scope. Verify two gets "     //d26725
             "out of commit scope will get both messages.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 42;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_memory_compressed_42,               //d26725
             "Put an compressed message on a memory queue out of commit  "      //d26725
             "scope, put another message in commit scope. Verify two gets "     //d26725
             "out of commit scope will get both messages.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 42;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_common_compressed_42,               //d26725
             "Put an compressed message on a common queue out of commit  "      //d26725
             "scope, put another message in commit scope. Verify two gets "     //d26725
             "out of commit scope will get both messages.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 42;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 43
TPF_TESTCASE(mq_put_out_scope_get_out_scope_memory_uncompressed_43,             //d26725
             "Put an uncompressed message on a memory queue out of commit  "    //d26725
             "scope, put another message in commit scope. Tx_rollback, get "    //d26725
             "first message out of commit scope, then verify a second get "     //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 43;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_common_uncompressed_43,             //d26725
             "Put an uncompressed message on a common queue out of commit  "    //d26725
             "scope, put another message in commit scope. Tx_rollback, get "    //d26725
             "first message out of commit scope, then verify a second get "     //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 43;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_memory_compressed_43,               //d26725
             "Put an compressed message on a memory queue out of commit  "      //d26725
             "scope, put another message in commit scope. Tx_rollback, get "    //d26725
             "first message out of commit scope, then verify a second get "     //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 43;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_out_scope_get_out_scope_common_compressed_43,               //d26725
             "Put an compressed message on a common queue out of commit  "      //d26725
             "scope, put another message in commit scope. Tx_rollback, get "    //d26725
             "first message out of commit scope, then verify a second get "     //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 43;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 44
TPF_TESTCASE(mq_put_get_storage_block_memory_uncompressed_44,                   //d26725
             "Put and get an uncompressed message from a memory queue, with "   //d26725
             "a storage block size of 0.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 44;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_uncompressed_44,                   //d26725
             "Put and get an uncompressed message from a common queue, with "   //d26725
             "a storage block size of 0.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 44;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_memory_compressed_44,                     //d26725
             "Put and get an compressed message from a memory queue, with a "   //d26725
             "storage block size of 0.")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 44;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_compressed_44,                     //d26725
             "Put and get an compressed message from a common queue, with a "   //d26725
             "storage block size of 0.")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 44;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 45
TPF_TESTCASE(mq_put_get_storage_block_memory_uncompressed_45,                   //d26725
             "Put and get an uncompressed message from a memory queue, with "   //d26725
             "a storage block size of 1.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 45;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_uncompressed_45,                   //d26725
             "Put and get an uncompressed message from a common queue, with "   //d26725
             "a storage block size of 1.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 45;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_memory_compressed_45,                     //d26725
             "Put and get an compressed message from a memory queue, with a "   //d26725
             "storage block size of 1.")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 45;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_compressed_45,                     //d26725
             "Put and get an compressed message from a common queue, with a "   //d26725
             "storage block size of 1.")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 45;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 46
TPF_TESTCASE(mq_put_get_storage_block_memory_uncompressed_46,                   //d26725
             "Put and get an uncompressed message from a memory queue, with "   //d26725
             "a storage block size of the first segment.")                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 46;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_uncompressed_46,                   //d26725
             "Put and get an uncompressed message from a common queue, with "   //d26725
             "a storage block size of the first segment.")                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 46;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_memory_compressed_46,                     //d26725
             "Put and get an compressed message from a memory queue, with a "   //d26725
             "storage block size of the first segment.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 46;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_compressed_46,                     //d26725
             "Put and get an compressed message from a common queue, with a "   //d26725
             "storage block size of the first segment.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 46;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 47
TPF_TESTCASE(mq_put_get_storage_block_memory_uncompressed_47,                   //d26725
             "Put and get an uncompressed message from a memory queue, with "   //d26725
             "a storage block size of the first segment + 1.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 47;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_uncompressed_47,                   //d26725
             "Put and get an uncompressed message from a common queue, with "   //d26725
             "a storage block size of the first segment + 1.")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 46;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_memory_compressed_47,                     //d26725
             "Put and get an compressed message from a memory queue, with a "   //d26725
             "storage block size of the first segment + 1.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 47;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_compressed_47,                     //d26725
             "Put and get an compressed message from a common queue, with a "   //d26725
             "storage block size of the first segment + 1.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 47;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 48
TPF_TESTCASE(mq_put_get_storage_block_memory_uncompressed_48,                   //d26725
             "Put and get an uncompressed message from a memory queue, with "   //d26725
             "a storage block size of the first two segments.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 48;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_uncompressed_48,                   //d26725
             "Put and get an uncompressed message from a common queue, with "   //d26725
             "a storage block size of the first two segments.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 48;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_memory_compressed_48,                     //d26725
             "Put and get an compressed message from a memory queue, with a "   //d26725
             "storage block size of the first two segments.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 48;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_compressed_48,                     //d26725
             "Put and get an compressed message from a common queue, with a "   //d26725
             "storage block size of the first two segments.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 48;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 49
TPF_TESTCASE(mq_put_get_storage_block_memory_uncompressed_49,                   //d26725
             "Put and get an uncompressed message from a memory queue, with "   //d26725
             "a storage block size of the first two segments + 1.")             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 49;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_uncompressed_49,                   //d26725
             "Put and get an uncompressed message from a common queue, with "   //d26725
             "a storage block size of the first two segments + 1.")             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 49;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_memory_compressed_49,                     //d26725
             "Put and get an compressed message from a memory queue, with a "   //d26725
             "storage block size of the first two segments + 1.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 49;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_compressed_49,                     //d26725
             "Put and get an compressed message from a common queue, with a "   //d26725
             "storage block size of the first two segments + 1.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 49;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 50
TPF_TESTCASE(mq_put_get_storage_block_memory_uncompressed_50,                   //d26725
             "Put and get an uncompressed message from a memory queue, with a " //d26725
             "storage block size of the first three segments.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 50;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_uncompressed_50,                   //d26725
             "Put and get an uncompressed message from a common queue with a "  //d26725
             "storage block size of the first three segments.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 50;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_memory_compressed_50,                     //d26725
             "Put and get a compressed message from a memory queue with a "     //d26725
             "storage block size of the first three segments.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 50;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_storage_block_common_compressed_50,                     //d26725
             "Put and get a compressed message from a common queue with a "     //d26725
             "storage block size of the first three segments.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 50;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 51
TPF_TESTCASE(mq_put_get_multiple_queues_memory_uncompressed_51,                 //d26725
             "Put an uncompressed message on each queue, put a second "         //d26725
             "message on each queue, get first message from each queue, get "   //d26725
             "second message from each queue.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 51;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_uncompressed_51,                 //d26725
             "Put an uncompressed message on each queue, put a second "         //d26725
             "message on each queue, get first message from each queue, get "   //d26725
             "second message from each queue.")                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 51;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_memory_compressed_51,                   //d26725
             "Put an compressed message on each queue, put a second message "   //d26725
             "on each queue, get first message from each queue, get second "    //d26725
             "message from each queue.")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 51;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_compressed_51,                   //d26725
             "Put an compressed message on each queue, put a second message "   //d26725
             "on each queue, get first message from each queue, get second "    //d26725
             "message from each queue.")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 51;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 52
TPF_TESTCASE(mq_put_get_multiple_queues_memory_uncompressed_52,                 //d26725
             "In root commit scope, put an uncompressed message on each "       //d26725
             "queue, put a second message on each queue, get first message "    //d26725
             "from each queue, get second message from each queue.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 52;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_uncompressed_52,                 //d26725
             "In root commit scope, put an uncompressed message on each "       //d26725
             "queue, put a second message on each queue, get first message "    //d26725
             "from each queue, get second message from each queue.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 52;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_memory_compressed_52,                   //d26725
             "In root commit scope, put an compressed message on each "         //d26725
             "queue, put a second message on each queue, get first message "    //d26725
             "from each queue, get second message from each queue.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 52;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_compressed_52,                   //d26725
             "In root commit scope, put an compressed message on each "         //d26725
             "queue, put a second message on each queue, get first message "    //d26725
             "from each queue, get second message from each queue.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 52;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 53
TPF_TESTCASE(mq_put_get_multiple_queues_memory_uncompressed_53,                 //d26725
             "In root commit scope, put an uncompressed message on each "       //d26725
             "queue, put a second message on each queue.  Tx_rollback, then "   //d26725
             "verify that a get on each queue will fail with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 53;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_uncompressed_53,                 //d26725
             "In root commit scope, put an uncompressed message on each "       //d26725
             "queue, put a second message on each queue.  Tx_rollback, then "   //d26725
             "verify that a get on each queue will fail with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 53;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_memory_compressed_53,                   //d26725
             "In root commit scope, put an compressed message on each "         //d26725
             "queue, put a second message on each queue.  Tx_rollback, then "   //d26725
             "verify that a get on each queue will fail with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 53;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_compressed_53,                   //d26725
             "In root commit scope, put an compressed message on each "         //d26725
             "queue, put a second message on each queue.  Tx_rollback, then "   //d26725
             "verify that a get on each queue will fail with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 53;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 54
TPF_TESTCASE(mq_put_get_multiple_queues_memory_uncompressed_54,                 //d26725
             "In nested commit scope, put an uncompressed message on each "     //d26725
             "queue.  In root commit scope, put a second message on each "      //d26725
             "queue. Out of commit scope, get both messages from each "         //d26725
             "queue.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 54;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_uncompressed_54,                 //d26725
             "In nested commit scope, put an uncompressed message on each "     //d26725
             "queue.  In root commit scope, put a second message on each "      //d26725
             "queue. Out of commit scope, get both messages from each "         //d26725
             "queue.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 54;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_memory_compressed_54,                   //d26725
             "In nested commit scope, put an compressed message on each "       //d26725
             "queue.  In root commit scope, put a second message on each "      //d26725
             "queue. Out of commit scope, get both messages from each "         //d26725
             "queue.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 54;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_compressed_54,                   //d26725
             "In nested commit scope, put an compressed message on each "       //d26725
             "queue.  In root commit scope, put a second message on each "      //d26725
             "queue. Out of commit scope, get both messages from each "         //d26725
             "queue.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 54;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 55
TPF_TESTCASE(mq_put_get_multiple_queues_memory_uncompressed_55,                 //d26725
             "In nested commit scope, put an uncompressed message on each "     //d26725
             "queue. tx_rollback, then put a second message on each queue. "    //d26725
             "Out of commit scope, get message from each queue.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 55;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_uncompressed_55,                 //d26725
             "In nested commit scope, put an uncompressed message on each "     //d26725
             "queue. tx_rollback, then put a second message on each queue. "    //d26725
             "Out of commit scope, get message from each queue.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 55;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_memory_compressed_55,                   //d26725
             "In nested commit scope, put an compressed message on each "       //d26725
             "queue. tx_rollback, then put a second message on each queue. "    //d26725
             "Out of commit scope, get message from each queue.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 55;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_compressed_55,                   //d26725
             "In nested commit scope, put an compressed message on each "       //d26725
             "queue. tx_rollback, then put a second message on each queue. "    //d26725
             "Out of commit scope, get message from each queue.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 55;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 56
TPF_TESTCASE(mq_put_get_multiple_queues_memory_uncompressed_56,                 //d26725
             "In root commit scope, put an uncompressed message on each "       //d26725
             "queue. In nested commit scope, iterate through loop putting "     //d26725
             "messages on each queue. Get messages from each queue.  "          //d26725
             "Tx_rollback, then put message on each queue.  Out of commit "     //d26725
             "scope, get messages from each queue, then verify the last get "   //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 56;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_uncompressed_56,                 //d26725
             "In root commit scope, put an uncompressed message on each "       //d26725
             "queue. In nested commit scope, iterate through loop putting "     //d26725
             "messages on each queue. Get messages from each queue.  "          //d26725
             "Tx_rollback, then put message on each queue.  Out of commit "     //d26725
             "scope, get messages from each queue, then verify the last get "   //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 56;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_memory_compressed_56,                   //d26725
             "In root commit scope, put an compressed message on each "         //d26725
             "queue. In nested commit scope, iterate through loop putting "     //d26725
             "messages on each queue. Get messages from each queue.  "          //d26725
             "Tx_rollback, then put message on each queue.  Out of commit "     //d26725
             "scope, get messages from each queue, then verify the last get "   //d26725
             "will fail with no message.")                                      //d26725

{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 56;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_queues_common_compressed_56,                   //d26725
             "In root commit scope, put an compressed message on each "         //d26725
             "queue. In nested commit scope, iterate through loop putting "     //d26725
             "messages on each queue. Get messages from each queue.  "          //d26725
             "Tx_rollback, then put message on each queue.  Out of commit "     //d26725
             "scope, get messages from each queue, then verify the last get "   //d26725
             "will fail with no message.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 56;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 57
TPF_TESTCASE(mq_put_get_large_queue_memory_uncompressed_57,                     //d26725
             "Put and get large uncompressed message to and from memory "       //d26725
             "queue.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 57;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_uncompressed_57,                     //d26725
             "Put and get large uncompressed message to and from common "       //d26725
             "queue.")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 57;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_memory_compressed_57,                       //d26725
             "Put and get large compressed message to and from memory queue.")  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 57;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_compressed_57,                       //d26725
             "Put and get large compressed message to and from common queue.")  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 57;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 58
TPF_TESTCASE(mq_put_get_large_queue_memory_uncompressed_58,                     //d26725
             "In commit scope, put and get large uncompressed message to "      //d26725
             "and from memory queue")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 58;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_uncompressed_58,                     //d26725
             "In commit scope, put and get large uncompressed message to "      //d26725
             "and from common queue")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 58;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_memory_compressed_58,                       //d26725
             "In commit scope, put and get large compressed message to and "    //d26725
             "from memory queue")                                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 58;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_compressed_58,                       //d26725
             "In commit scope, put and get large compressed message to and "    //d26725
             "from common queue")                                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 58;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 59
TPF_TESTCASE(mq_put_get_large_queue_memory_uncompressed_59,                     //d26725
             "In commit scope, put large uncompressed message to memory "       //d26725
             "queue, then get message out of commit scope.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 59;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_uncompressed_59,                     //d26725
             "In commit scope, put large uncompressed message to common "       //d26725
             "queue, then get message out of commit scope.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 59;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_memory_compressed_59,                       //d26725
             "In commit scope, put large compressed message to memory "         //d26725
             "queue, then get message out of commit scope.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 59;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_compressed_59,                       //d26725
             "In commit scope, put large compressed message to common "         //d26725
             "queue, then get message out of commit scope.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 59;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 60
TPF_TESTCASE(mq_put_get_large_queue_memory_uncompressed_60,                     //d26725
             "In commit scope, put large uncompressed message to memory "       //d26725
             "queue, tx_rollback, then verify get fails with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 60;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_uncompressed_60,                     //d26725
             "In commit scope, put large uncompressed message to common "       //d26725
             "queue, tx_rollback, then verify get fails with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 60;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_memory_compressed_60,                       //d26725
             "In commit scope, put large compressed message to memory "         //d26725
             "queue, tx_rollback, then verify get fails with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 60;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_compressed_60,                       //d26725
             "In commit scope, put large compressed message to common "         //d26725
             "queue, tx_rollback, then verify get fails with no message.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 60;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 61
TPF_TESTCASE(mq_put_get_large_queue_memory_uncompressed_61,                     //d26725
             "Out of commit scope, put large uncompressed message to memory "   //d26725
             "queue, then get message in commit scope.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 61;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_uncompressed_61,                     //d26725
             "Out of commit scope, put large uncompressed message to common "   //d26725
             "queue, then get message in commit scope.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 61;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_memory_compressed_61,                       //d26725
             "Out of commit scope, put large compressed message to memory "     //d26725
             "queue, then get message in commit scope.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 61;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_compressed_61,                       //d26725
             "Out of commit scope, put large compressed message to common "     //d26725
             "queue, then get message in commit scope.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 61;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 62
TPF_TESTCASE(mq_put_get_large_queue_memory_uncompressed_62,                     //d26725
             "In commit scope, put two large uncompressed message to memory "   //d26725
             "queue, then get two messages out of commit scope.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 62;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_uncompressed_62,                     //d26725
             "In commit scope, put two large uncompressed message to common "   //d26725
             "queue, then get two messages out of commit scope.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 62;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_memory_compressed_62,                       //d26725
             "In commit scope, put two large compressed message to memory "     //d26725
             "queue, then get two messages out of commit scope.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 62;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_compressed_62,                       //d26725
             "In commit scope, put two large compressed message to common "     //d26725
             "queue, then get two messages out of commit scope.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 62;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 63
TPF_TESTCASE(mq_put_get_large_queue_memory_uncompressed_63,                     //d26725
             "In commit scope, put large uncompressed message to memory "       //d26725
             "queue, then get message out of commit scope.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 63;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_uncompressed_63,                     //d26725
             "In commit scope, put large uncompressed message to common "       //d26725
             "queue, then get message out of commit scope.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 63;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_memory_compressed_63,                       //d26725
             "In commit scope, put large compressed message to memory "         //d26725
             "queue, then get message out of commit scope.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 63;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_large_queue_common_compressed_63,                       //d26725
             "In commit scope, put large compressed message to common "         //d26725
             "queue, then get message out of commit scope.")                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 63;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 64
TPF_TESTCASE(mq_put_get_multiple_scopes_memory_uncompressed_64,                 //d26725
             "In first commit scope put uncompressed message to memory "        //d26725
             "queue. In second commit scope, get message.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 64;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_uncompressed_64,                 //d26725
             "In first commit scope put uncompressed message to common "        //d26725
             "queue. In second commit scope, get message.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 64;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_memory_compressed_64,                   //d26725
             "In first commit scope put compressed message to memory queue. "   //d26725
             "In second commit scope, get message.")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 64;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_compressed_64,                   //d26725
             "In first commit scope put compressed message to common queue. "   //d26725
             "In second commit scope, get message.")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 64;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 65
TPF_TESTCASE(mq_put_get_multiple_scopes_memory_uncompressed_65,                 //d26725
             "In first commit scope put uncompressed message to memory "        //d26725
             "queue.  In second commit scope, get message, tx_rollback, "       //d26725
             "then get message again.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 65;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_uncompressed_65,                 //d26725
             "In first commit scope put uncompressed message to common "        //d26725
             "queue.  In second commit scope, get message, tx_rollback, "       //d26725
             "then get message again.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 65;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_memory_compressed_65,                   //d26725
             "In first commit scope put compressed message to memory queue. "   //d26725
             " In second commit scope, get message, tx_rollback, then get "     //d26725
             "message again.")                                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 65;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_compressed_65,                   //d26725
             "In first commit scope put compressed message to common queue. "   //d26725
             " In second commit scope, get message, tx_rollback, then get "     //d26725
             "message again.")                                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 65;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 66
TPF_TESTCASE(mq_put_get_multiple_scopes_memory_uncompressed_66,                 //d26725
             "In first commit scope put uncompressed message to memory "        //d26725
             "queue.  In second commit scope, put another message and get "     //d26725
             "the first message.  In third commit scope, get the second "       //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 66;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_uncompressed_66,                 //d26725
             "In first commit scope put uncompressed message to common "        //d26725
             "queue.  In second commit scope, put another message and get "     //d26725
             "the first message.  In third commit scope, get the second "       //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 66;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_memory_compressed_66,                   //d26725
             "In first commit scope put compressed message to memory queue. "   //d26725
             " In second commit scope, put another message and get the "        //d26725
             "first message.  In third commit scope, get the second "           //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 66;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_compressed_66,                   //d26725
             "In first commit scope put compressed message to common queue. "   //d26725
             " In second commit scope, put another message and get the "        //d26725
             "first message.  In third commit scope, get the second "           //d26725
             "message.")                                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 66;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 67
TPF_TESTCASE(mq_put_get_multiple_scopes_memory_uncompressed_67,                 //d26725
             "In first commit scope put uncompressed message to memory "        //d26725
             "queue.  In second commit scope, put another message and get "     //d26725
             "the first message.  tx_rollback, then in third commit scope, "    //d26725
             "get the first message.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 67;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_uncompressed_67,                 //d26725
             "In first commit scope put uncompressed message to common "        //d26725
             "queue.  In second commit scope, put another message and get "     //d26725
             "the first message.  tx_rollback, then in third commit scope, "    //d26725
             "get the first message.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 67;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_memory_compressed_67,                   //d26725
             "In first commit scope put compressed message to memory queue. "   //d26725
             " In second commit scope, put another message and get the "        //d26725
             "first message.  tx_rollback, then in third commit scope, get "    //d26725
             "the first message.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 67;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_compressed_67,                   //d26725
             "In first commit scope put compressed message to common queue. "   //d26725
             " In second commit scope, put another message and get the "        //d26725
             "first message.  tx_rollback, then in third commit scope, get "    //d26725
             "the first message.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 67;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 68
TPF_TESTCASE(mq_put_get_truncated_memory_uncompressed_68,                       //d26725
             "Out of commit scope, put and get truncated uncompressed "         //d26725
             "message to and from memory queue.")                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 68;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_uncompressed_68,                       //d26725
             "Out of commit scope, put and get truncated uncompressed "         //d26725
             "message to and from common queue.")                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 68;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_memory_compressed_68,                         //d26725
             "Out of commit scope, put and get truncated compressed message "   //d26725
             "to and from memory queue.")                                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 68;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_compressed_68,                         //d26725
             "Out of commit scope, put and get truncated compressed message "   //d26725
             "to and from common queue.")                                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 68;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 69
TPF_TESTCASE(mq_put_get_truncated_memory_uncompressed_69,                       //d26725
             "In commit scope, put and get truncated uncompressed message "     //d26725
             "to and from memory queue.")                                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 69;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_uncompressed_69,                       //d26725
             "In commit scope, put and get truncated uncompressed message "     //d26725
             "to and from common queue.")                                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 69;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_memory_compressed_69,                         //d26725
             "In commit scope, put and get truncated compressed message to "    //d26725
             "and from memory queue.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 69;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_compressed_69,                         //d26725
             "In commit scope, put and get truncated compressed message to "    //d26725
             "and from common queue.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 69;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 70
TPF_TESTCASE(mq_put_get_truncated_memory_uncompressed_70,                       //d26725
             "In commit scope, put and get truncated uncompressed message "     //d26725
             "to and from memory queue, then tx_rollback.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 70;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_uncompressed_70,                       //d26725
             "In commit scope, put and get truncated uncompressed message "     //d26725
             "to and from common queue, then tx_rollback.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 70;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_memory_compressed_70,                         //d26725
             "In commit scope, put and get truncated compressed message to "    //d26725
             "and from memory queue, then tx_rollback.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 70;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_compressed_70,                         //d26725
             "In commit scope, put and get truncated compressed message to "    //d26725
             "and from common queue, then tx_rollback.")                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 70;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 71
TPF_TESTCASE(mq_put_get_truncated_memory_uncompressed_71,                       //d26725
             "In commit scope, put truncated uncompressed message to memory "   //d26725
             "queue, then get out of commit scope.")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 71;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_uncompressed_71,                       //d26725
             "In commit scope, put truncated uncompressed message to common "   //d26725
             "queue, then get out of commit scope.")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 71;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_memory_compressed_71,                         //d26725
             "In commit scope, put truncated compressed message to memory "     //d26725
             "queue, then get out of commit scope.")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 71;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_compressed_71,                         //d26725
             "In commit scope, put truncated compressed message to common "     //d26725
             "queue, then get out of commit scope.")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 71;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 72
TPF_TESTCASE(mq_put_get_truncated_memory_uncompressed_72,                       //d26725
             "In commit scope, put truncated uncompressed message to memory "   //d26725
             "queue, tx_rollback, then get out of commit scope.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 72;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_uncompressed_72,                       //d26725
             "In commit scope, put truncated uncompressed message to common "   //d26725
             "queue, tx_rollback, then get out of commit scope.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 72;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_memory_compressed_72,                         //d26725
             "In commit scope, put truncated compressed message to memory "     //d26725
             "queue, tx_rollback, then get out of commit scope.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 72;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_compressed_72,                         //d26725
             "In commit scope, put truncated compressed message to common "     //d26725
             "queue, tx_rollback, then get out of commit scope.")               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 72;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 73
TPF_TESTCASE(mq_put_get_multiple_scopes_memory_uncompressed_73,                 //d26725
             "Out of commit scope, put three uncompressed message to memory "   //d26725
             "queue.  In commit scope, get two of the messages and put "        //d26725
             "another one.   In a nested commit scope, get another message, "   //d26725
             "tx_rollback, then put and get several more messages. In the "     //d26725
             "root commit scope, and outside the commit scope, get the "        //d26725
             "remaining messages.")                                             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 73;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_uncompressed_73,                 //d26725
             "Out of commit scope, put three uncompressed message to common "   //d26725
             "queue.  In commit scope, get two of the messages and put "        //d26725
             "another one.   In a nested commit scope, get another message, "   //d26725
             "tx_rollback, then put and get several more messages. In the "     //d26725
             "root commit scope, and outside the commit scope, get the "        //d26725
             "remaining messages.")                                             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 73;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_memory_compressed_73,                   //d26725
             "Out of commit scope, put three compressed message to memory "     //d26725
             "queue.  In commit scope, get two of the messages and put "        //d26725
             "another one.   In a nested commit scope, get another message, "   //d26725
             "tx_rollback, then put and get several more messages. In the "     //d26725
             "root commit scope, and outside the commit scope, get the "        //d26725
             "remaining messages.")                                             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 73;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_multiple_scopes_common_compressed_73,                   //d26725
             "Out of commit scope, put three compressed message to common "     //d26725
             "queue.  In commit scope, get two of the messages and put "        //d26725
             "another one.   In a nested commit scope, get another message, "   //d26725
             "tx_rollback, then put and get several more messages. In the "     //d26725
             "root commit scope, and outside the commit scope, get the "        //d26725
             "remaining messages.")                                             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 73;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 74
TPF_TESTCASE(mq_put_get_truncated_memory_uncompressed_74,                       //d26725
             "In commit scope, put truncated uncompressed message to memory "   //d26725
             "queue.  Out of the commit scope, verify on a get, the message "   //d26725
             "will be rejected due to unsufficient buffer size,  then "         //d26725
             "verify the message is read correctly on a get with the "          //d26725
             "correct buffer size.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 74;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_uncompressed_74,                       //d26725
             "In commit scope, put truncated uncompressed message to common "   //d26725
             "queue.  Out of the commit scope, verify on a get, the message "   //d26725
             "will be rejected due to unsufficient buffer size,  then "         //d26725
             "verify the message is read correctly on a get with the "          //d26725
             "correct buffer size.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 74;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_memory_compressed_74,                         //d26725
             "In commit scope, put truncated compressed message to memory "     //d26725
             "queue.  Out of the commit scope, verify on a get, the message "   //d26725
             "will be rejected due to unsufficient buffer size,  then "         //d26725
             "verify the message is read correctly on a get with the "          //d26725
             "correct buffer size.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 74;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_truncated_common_compressed_74,                         //d26725
             "In commit scope, put truncated compressed message to common "     //d26725
             "queue.  Out of the commit scope, verify on a get, the message "   //d26725
             "will be rejected due to unsufficient buffer size,  then "         //d26725
             "verify the message is read correctly on a get with the "          //d26725
             "correct buffer size.")                                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 74;                                                            //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 101
TPF_TESTCASE(mq_put_get_wait_option_memory_uncompressed_101,                    //d26725
             "Put and get uncompressed message to and from memory queue "       //d26725
             "with MQGMO_NONE option.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 101;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_common_uncompressed_101,                    //d26725
             "Put and get uncompressed message to and from common queue "       //d26725
             "with MQGMO_NONE option.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 101;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_memory_compressed_101,                      //d26725
             "Put and get compressed message to and from memory queue with "    //d26725
             "MQGMO_NONE option.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 101;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_common_compressed_101,                      //d26725
             "Put and get compressed message to and from common queue with "    //d26725
             "MQGMO_NONE option.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 101;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 102
TPF_TESTCASE(mq_put_get_wait_option_memory_uncompressed_102,                    //d26725
             "Put and get uncompressed message to and from memory queue "       //d26725
             "with MQGMO_NONE option and an interval of MQWI_UNLIMITED.")       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 102;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_common_uncompressed_102,                    //d26725
             "Put and get uncompressed message to and from common queue "       //d26725
             "with MQGMO_NONE option and an interval of MQWI_UNLIMITED.")       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 102;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_memory_compressed_102,                      //d26725
             "Put and get compressed message to and from memory queue with "    //d26725
             "MQGMO_NONE option and an interval of MQWI_UNLIMITED.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 102;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_common_compressed_102,                      //d26725
             "Put and get compressed message to and from common queue with "    //d26725
             "MQGMO_NONE option and an interval of MQWI_UNLIMITED.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 102;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 103
TPF_TESTCASE(mq_put_get_wait_option_memory_uncompressed_103,                    //d26725
             "Put and get uncompressed message to and from memory queue "       //d26725
             "with MQGMO_NONE option and invalid wait intervals.")              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 103;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_common_uncompressed_103,                    //d26725
             "Put and get uncompressed message to and from common queue "       //d26725
             "with MQGMO_NONE option and invalid wait intervals.")              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 103;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_memory_compressed_103,                      //d26725
             "Put and get compressed message to and from memory queue with "    //d26725
             "MQGMO_NONE option and invalid wait intervals.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 103;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_common_compressed_103,                      //d26725
             "Put and get compressed message to and from common queue with "    //d26725
             "MQGMO_NONE option and invalid wait intervals.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 103;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 104
TPF_TESTCASE(mq_put_get_wait_option_memory_uncompressed_104,                    //d26725
             "Put and get uncompressed message to and from memory queue "       //d26725
             "with invalid browse options.")                                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 104;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_common_uncompressed_104,                    //d26725
             "Put and get uncompressed message to and from common queue "       //d26725
             "with invalid browse options.")                                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 104;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_memory_compressed_104,                      //d26725
             "Put and get compressed message to and from memory queue with "    //d26725
             "invalid browse options.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 104;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_wait_option_common_compressed_104,                      //d26725
             "Put and get compressed message to and from common queue with "    //d26725
             "invalid browse options.")                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 104;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 200
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_200,                  //d26725
             "For uncompressed messages on memory queues, test using "          //d26725
             "various invalid browse options or combinations when the queue "   //d26725
             "is open for browse only.")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 200;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_200,                  //d26725
             "For uncompressed messages on common queues, test using "          //d26725
             "various invalid browse options or combinations when the queue "   //d26725
             "is open for browse only.")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 200;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_200,                    //d26725
             "For compressed messages on memory queues, test using various "    //d26725
             "invalid browse options or combinations when the queue is open "   //d26725
             "for browse only.")                                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 200;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_200,                    //d26725
             "For compressed messages on common queues, test using various "    //d26725
             "invalid browse options or combinations when the queue is open "   //d26725
             "for browse only.")                                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 200;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 201
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_201,                  //d26725
             "For uncompressed messages on memory queues, test using "          //d26725
             "various invalid browse options or combinations when the queue "   //d26725
             "is open for browse and input.")                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 201;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_201,                  //d26725
             "For uncompressed messages on common queues, test using "          //d26725
             "various invalid browse options or combinations when the queue "   //d26725
             "is open for browse and input.")                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 201;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_201,                    //d26725
             "For compressed messages on memory queues, test using various "    //d26725
             "invalid browse options or combinations when the queue is open "   //d26725
             "for browse and input.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 201;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_201,                    //d26725
             "For compressed messages on common queues, test using various "    //d26725
             "invalid browse options or combinations when the queue is open "   //d26725
             "for browse and input.")                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 201;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 202
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_202,                  //d26725
             "For uncompressed messages on memory queues, test using "          //d26725
             "various browse options or combinations when the queue is open "   //d26725
             "for output only.")                                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 202;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_202,                  //d26725
             "For uncompressed messages on common queues, test using "          //d26725
             "various browse options or combinations when the queue is open "   //d26725
             "for output only.")                                                //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 202;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_202,                    //d26725
             "For compressed messages on memory queues, test using various "    //d26725
             "browse options or combinations when the queue is open for "       //d26725
             "output only.")                                                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 202;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_202,                    //d26725
             "For compressed messages on common queues, test using various "    //d26725
             "browse options or combinations when the queue is open for "       //d26725
             "output only.")                                                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 202;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 203
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_203,                  //d26725
             "For uncompressed messages on memory queues, test using "          //d26725
             "various browse options or combinations when the queue is open "   //d26725
             "for input only.")                                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 203;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_203,                  //d26725
             "For uncompressed messages on common queues, test using "          //d26725
             "various browse options or combinations when the queue is open "   //d26725
             "for input only.")                                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 203;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_203,                    //d26725
             "For compressed messages on memory queues, test using various "    //d26725
             "browse options or combinations when the queue is open for "       //d26725
             "input only.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 203;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_203,                    //d26725
             "For compressed messages on common queues, test using various "    //d26725
             "browse options or combinations when the queue is open for "       //d26725
             "input only.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 203;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 204
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_204,                  //d26725
             "For uncompressed messages on memory queues, test rejecting "      //d26725
             "and accepting an uncommitted truncated message.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 204;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_204,                  //d26725
             "For uncompressed messages on common queues, test rejecting "      //d26725
             "and accepting an uncommitted truncated message.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 204;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_204,                    //d26725
             "For compressed messages on memory queues, test rejecting and "    //d26725
             "accepting an uncommitted truncated message.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 204;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_204,                    //d26725
             "For compressed messages on common queues, test rejecting and "    //d26725
             "accepting an uncommitted truncated message.")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 204;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 205
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_205,                  //d26725
             "For uncompressed messages on memory queues, test a simple "       //d26725
             "browse of three messages.")                                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 205;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_205,                  //d26725
             "For uncompressed messages on common queues, test a simple "       //d26725
             "browse of three messages.")                                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 205;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_205,                    //d26725
             "For compressed messages on memory queues, test a simple "         //d26725
             "browse of three messages.")                                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 205;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_205,                    //d26725
             "For compressed messages on common queues, test a simple "         //d26725
             "browse of three messages.")                                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 205;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 206
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_206,                  //d26725
             "For uncompressed messages on memory queues, test rejecting "      //d26725
             "and accepting a committed truncated message and verify that "     //d26725
             "the cursor positioning is correct.")                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 206;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_206,                  //d26725
             "For uncompressed messages on common queues, test rejecting "      //d26725
             "and accepting a committed truncated message and verify that "     //d26725
             "the cursor positioning is correct.")                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 206;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_206,                    //d26725
             "For compressed messages on memory queues, test rejecting and "    //d26725
             "accepting a committed truncated message and verify that the "     //d26725
             "cursor positioning is correct.")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 206;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_206,                    //d26725
             "For compressed messages on common queues, test rejecting and "    //d26725
             "accepting a committed truncated message and verify that the "     //d26725
             "cursor positioning is correct.")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 206;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 207
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_207,                  //d26725
             "For uncompressed messages on memory queues, browse a queue "      //d26725
             "with one committed and one uncommitted message.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 207;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_207,                  //d26725
             "For uncompressed messages on common queues, browse a queue "      //d26725
             "with one committed and one uncommitted message.")                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 207;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_207,                    //d26725
             "For compressed messages on memory queues, browse a queue with "   //d26725
             "one committed and one uncommitted message.")                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 207;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_207,                    //d26725
             "For compressed messages on common queues, browse a queue with "   //d26725
             "one committed and one uncommitted message.")                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 207;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 208
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_208,                  //d26725
             "For uncompressed messages on memory queues, Test "                //d26725
             "MSG_UNDER_CURSOR and verify the correct return code when "        //d26725
             "using BROWSE_MSG_UNDER_CURSOR after the message under the "       //d26725
             "cursor is removed.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 208;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_208,                  //d26725
             "For uncompressed messages on common queues, test "                //d26725
             "MSG_UNDER_CURSOR and verify the correct return code when "        //d26725
             "using BROWSE_MSG_UNDER_CURSOR after the message under the "       //d26725
             "cursor is removed.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 208;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_208,                    //d26725
             "For compressed messages on memory queues, test "                  //d26725
             "MSG_UNDER_CURSOR and verify the correct return code when "        //d26725
             "using BROWSE_MSG_UNDER_CURSOR after the message under the "       //d26725
             "cursor is removed.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 208;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_208,                    //d26725
             "For compressed messages on common queues, test "                  //d26725
             "MSG_UNDER_CURSOR and verify the correct return code when "        //d26725
             "using BROWSE_MSG_UNDER_CURSOR after the message under the "       //d26725
             "cursor is removed.")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 208;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 209
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_209,                  //d26725
             "For uncompressed messages on memory queues, test browsing, "      //d26725
             "then get the message under the cursor.")                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 209;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_209,                  //d26725
             "For uncompressed messages on common queues, test browsing, "      //d26725
             "then get the message under the cursor.")                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 209;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_209,                    //d26725
             "For compressed messages on memory queues, test browsing, then "   //d26725
             "get the message under the cursor.")                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 209;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_209,                    //d26725
             "For compressed messages on common queues, test browsing, then "   //d26725
             "get the message under the cursor.")                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 209;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 210
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_210,                  //d26725
             "For uncompressed messages on memory queues, verify that "         //d26725
             "attempting to browse past the end of the queue doesn't move "     //d26725
             "the cursor.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 210;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_210,                  //d26725
             "For uncompressed messages on common queues, verify that "         //d26725
             "attempting to browse past the end of the queue doesn't move "     //d26725
             "the cursor.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 210;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_210,                    //d26725
             "For compressed messages on memory queues, verify that "           //d26725
             "attempting to browse past the end of the queue doesn't move "     //d26725
             "the cursor.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 210;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_210,                    //d26725
             "For compressed messages on common queues, verify that "           //d26725
             "attempting to browse past the end of the queue doesn't move "     //d26725
             "the cursor.")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 210;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 211
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_211,                  //d26725
             "For uncompressed messages on memory queues, verify that a "       //d26725
             "locked message can only be read using the cursor that locked "    //d26725
             "it, and that browsing without lock will unlock a message.")       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 211;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_211,                  //d26725
             "For uncompressed messages on common queues, verify that a "       //d26725
             "locked message can only be read using the cursor that locked "    //d26725
             "it, and that browsing without lock will unlock a message.")       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 211;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_211,                    //d26725
             "For compressed messages on memory queues, verify that a "         //d26725
             "locked message can only be read using the cursor that locked "    //d26725
             "it, and that browsing without lock will unlock a message.")       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 211;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_211,                    //d26725
             "For compressed messages on common queues, verify that a "         //d26725
             "locked message can only be read using the cursor that locked "    //d26725
             "it, and that browsing without lock will unlock a message.")       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 211;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 212
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_212,                  //d26725
             "For uncompressed messages on memory queues, verify that a "       //d26725
             "message that becomes locked after it's browsed can't be "         //d26725
             "browsed even using the cursor that first browsed it, but it "     //d26725
             "can after the locking cursor moves on.")                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 212;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_212,                  //d26725
             "For uncompressed messages on common queues, verify that a "       //d26725
             "message that becomes locked after it's browsed can't be "         //d26725
             "browsed even using the cursor that first browsed it, but it "     //d26725
             "can after the locking cursor moves on.")                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 212;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_212,                    //d26725
             "For compressed messages on memory queues, verify that a "         //d26725
             "message that becomes locked after it's browsed can't be "         //d26725
             "browsed even using the cursor that first browsed it, but it "     //d26725
             "can after the locking cursor moves on.")                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 212;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_212,                    //d26725
             "For compressed messages on common queues, verify that a "         //d26725
             "message that becomes locked after it's browsed can't be "         //d26725
             "browsed even using the cursor that first browsed it, but it "     //d26725
             "can after the locking cursor moves on.")                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 212;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 213
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_213,                  //d26725
             "For uncompressed messages on memory queues, verify that two "     //d26725
             "BROWSE_FIRSTs with LOCK keeps the message locked, and that a "    //d26725
             "BROWSE_MSG_UNDER_CURSOR without lock will unlock it.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 213;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_213,                  //d26725
             "For uncompressed messages on common queues, verify that two "     //d26725
             "BROWSE_FIRSTs with LOCK keeps the message locked, and that a "    //d26725
             "BROWSE_MSG_UNDER_CURSOR without lock will unlock it.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 213;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_213,                    //d26725
             "For compressed messages on memory queues, verify that two "       //d26725
             "BROWSE_FIRSTs with LOCK keeps the message locked, and that a "    //d26725
             "BROWSE_MSG_UNDER_CURSOR without lock will unlock it.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 213;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_213,                    //d26725
             "For compressed messages on common queues, verify that two "       //d26725
             "BROWSE_FIRSTs with LOCK keeps the message locked, and that a "    //d26725
             "BROWSE_MSG_UNDER_CURSOR without lock will unlock it.")            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 213;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 214
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_214,                  //d26725
             "For uncompressed messages on memory queues, verify that a "       //d26725
             "locked message is unlocked when the Hobj that has it locked "     //d26725
             "is closed.")                                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 214;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_214,                  //d26725
             "For uncompressed messages on common queues, verify that a "       //d26725
             "locked message is unlocked when the Hobj that has it locked "     //d26725
             "is closed.")                                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 214;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_214,                    //d26725
             "For compressed messages on memory queues, verify that a "         //d26725
             "locked message is unlocked when the Hobj that has it locked "     //d26725
             "is closed.")                                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 214;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_214,                    //d26725
             "For compressed messages on common queues, verify that a "         //d26725
             "locked message is unlocked when the Hobj that has it locked "     //d26725
             "is closed.")                                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 214;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 215
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_215,                  //d26725
             "For uncompressed messages on memory queues, verify that a "       //d26725
             "locked message is unlocked when MQDISC is executed.")             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 215;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_215,                  //d26725
             "For uncompressed messages on common queues, verify that a "       //d26725
             "locked message is unlocked when MQDISC is executed.")             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 215;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_215,                    //d26725
             "For compressed messages on memory queues, verify that a "         //d26725
             "locked message is unlocked when MQDISC is executed.")             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 215;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_215,                    //d26725
             "For compressed messages on common queues, verify that a "         //d26725
             "locked message is unlocked when MQDISC is executed.")             //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 215;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 216
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_216,                  //d26725
             "For uncompressed messages on memory queues, verify correct "      //d26725
             "reason code when BROWSE_MSG_UNDER_CURSOR is used before any "     //d26725
             "other call and when it finds the message is gone. Also, try "     //d26725
             "to unlock it.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 216;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_216,                  //d26725
             "For uncompressed messages on common queues, verify correct "      //d26725
             "reason code when BROWSE_MSG_UNDER_CURSOR is used before any "     //d26725
             "other call and when it finds the message is gone. Also, try "     //d26725
             "to unlock it.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 216;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_216,                    //d26725
             "For compressed messages on memory queues, verify correct "        //d26725
             "reason code when BROWSE_MSG_UNDER_CURSOR is used before any "     //d26725
             "other call and when it finds the message is gone. Also, try "     //d26725
             "to unlock it.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 216;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_216,                    //d26725
             "For compressed messages on common queues, verify correct "        //d26725
             "reason code when BROWSE_MSG_UNDER_CURSOR is used before any "     //d26725
             "other call and when it finds the message is gone. Also, try "     //d26725
             "to unlock it.")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 216;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 217
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_217,                  //d26725
             "For uncompressed messages on memory queues, try to unlock a "     //d26725
             "message that isn't locked.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 217;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_217,                  //d26725
             "For uncompressed messages on common queues, try to unlock a "     //d26725
             "message that isn't locked.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 217;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_217,                    //d26725
             "For compressed messages on memory queues, try to unlock a "       //d26725
             "message that isn't locked.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 217;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_217,                    //d26725
             "For compressed messages on common queues, try to unlock a "       //d26725
             "message that isn't locked.")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 217;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 218
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_218,                  //d26725
             "For uncompressed messages on memory queues, try to unlock a "     //d26725
             "message that was locked, but is now gone.")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 218;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_218,                  //d26725
             "For uncompressed messages on common queues, try to unlock a "     //d26725
             "message that was locked, but is now gone.")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 218;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_218,                    //d26725
             "For compressed messages on memory queues, try to unlock a "       //d26725
             "message that was locked, but is now gone.")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 218;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_218,                    //d26725
             "For compressed messages on common queues, try to unlock a "       //d26725
             "message that was locked, but is now gone.")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 218;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 219
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_219,                  //d26725
             "For uncompressed messages on memory queues, verify that "         //d26725
             "removing a message that was just browsed doesn't cause a "        //d26725
             "problem browsing the next message.")                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 219;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_219,                  //d26725
             "For uncompressed messages on common queues, verify that "         //d26725
             "removing a message that was just browsed doesn't cause a "        //d26725
             "problem browsing the next message.")                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 219;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_219,                    //d26725
             "For compressed messages on memory queues, verify that "           //d26725
             "removing a message that was just browsed doesn't cause a "        //d26725
             "problem browsing the next message.")                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 219;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_219,                    //d26725
             "For compressed messages on common queues, verify that "           //d26725
             "removing a message that was just browsed doesn't cause a "        //d26725
             "problem browsing the next message.")                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 219;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 220
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_220,                  //d26725
             "For uncompressed messages on memory queues, verify that a "       //d26725
             "message is unlocked if the end of the queue is reached.")         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 220;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_220,                  //d26725
             "For uncompressed messages on common queues, verify that a "       //d26725
             "message is unlocked if the end of the queue is reached.")         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 220;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_220,                    //d26725
             "For compressed messages on memory queues, verify that a "         //d26725
             "message is unlocked if the end of the queue is reached.")         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 220;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_220,                    //d26725
             "For compressed messages on common queues, verify that a "         //d26725
             "message is unlocked if the end of the queue is reached.")         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 220;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 221
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_221,                  //d26725
             "For uncompressed messages on memory queues, try to get a "        //d26725
             "message that was locked by another handle, then try again "       //d26725
             "after the message is unlocked.")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 221;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_221,                  //d26725
             "For uncompressed messages on common queues, try to get a "        //d26725
             "message that was locked by another handle, then try again "       //d26725
             "after the message is unlocked.")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 221;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_221,                    //d26725
             "For compressed messages on memory queues, try to get a "          //d26725
             "message that was locked by another handle, then try again "       //d26725
             "after the message is unlocked.")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 221;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_221,                    //d26725
             "For compressed messages on common queues, try to get a "          //d26725
             "message that was locked by another handle, then try again "       //d26725
             "after the message is unlocked.")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 221;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 222
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_222,                  //d26725
             "For uncompressed messages on memory queues, verify that a "       //d26725
             "BROWSE_NEXT after reaching end-of-queue and after a message "     //d26725
             "is added will succeed.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 222;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_222,                  //d26725
             "For uncompressed messages on common queues, verify that a "       //d26725
             "BROWSE_NEXT after reaching end-of-queue and after a message "     //d26725
             "is added will succeed.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 222;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_222,                    //d26725
             "For compressed messages on memory queues, verify that a "         //d26725
             "BROWSE_NEXT after reaching end-of-queue and after a message "     //d26725
             "is added will succeed.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 222;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_222,                    //d26725
             "For compressed messages on common queues, verify that a "         //d26725
             "BROWSE_NEXT after reaching end-of-queue and after a message "     //d26725
             "is added will succeed.")                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 222;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 223
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_223,                  //d26725
             "For uncompressed messages on memory queues, verify that the "     //d26725
             "same handle that locked a message can get it using a "            //d26725
             "non-browse MQGET, but a different handle, only opened for "       //d26725
             "input, cannot.")                                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 223;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_223,                  //d26725
             "For uncompressed messages on common queues, verify that the "     //d26725
             "same handle that locked a message can get it using a "            //d26725
             "non-browse MQGET, but a different handle, only opened for "       //d26725
             "input, cannot.")                                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 223;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_223,                    //d26725
             "For compressed messages on memory queues, verify that the "       //d26725
             "same handle that locked a message can get it using a "            //d26725
             "non-browse MQGET, but a different handle, only opened for "       //d26725
             "input, cannot.")                                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 223;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_223,                    //d26725
             "For compressed messages on common queues, verify that the "       //d26725
             "same handle that locked a message can get it using a "            //d26725
             "non-browse MQGET, but a different handle, only opened for "       //d26725
             "input, cannot.")                                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 223;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 224
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_224,                  //d26725
             "For uncompressed messages on memory queues, verify that "         //d26725
             "UNLOCK and BROWSE_MSG_UNDER_CURSOR don't advance the cursor.")    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 224;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_224,                  //d26725
             "For uncompressed messages on common queues, verify that "         //d26725
             "UNLOCK and BROWSE_MSG_UNDER_CURSOR don't advance the cursor.")    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 224;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_224,                    //d26725
             "For compressed messages on memory queues, verify that UNLOCK "    //d26725
             "and BROWSE_MSG_UNDER_CURSOR don't advance the cursor.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 224;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_224,                    //d26725
             "For compressed messages on common queues, verify that UNLOCK "    //d26725
             "and BROWSE_MSG_UNDER_CURSOR don't advance the cursor.")           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 224;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 225
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_225,                  //d26725
             "For uncompressed messages on memory queues, test "                //d26725
             "MSG_UNDER_CURSOR and verify the correct return code when "        //d26725
             "using BROWSE_MSG_UNDER_CURSOR after the message under the "       //d26725
             "cursor is removed.  This is the same as test case 208, except "   //d26725
             "that the removal of the message is committed.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 225;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_225,                  //d26725
             "For uncompressed messages on common queues, test "                //d26725
             "MSG_UNDER_CURSOR and verify the correct return code when "        //d26725
             "using BROWSE_MSG_UNDER_CURSOR after the message under the "       //d26725
             "cursor is removed.  This is the same as test case 208, except "   //d26725
             "that the removal of the message is committed.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 225;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_225,                    //d26725
             "For compressed messages on memory queues, test "                  //d26725
             "MSG_UNDER_CURSOR and verify the correct return code when "        //d26725
             "using BROWSE_MSG_UNDER_CURSOR after the message under the "       //d26725
             "cursor is removed.  This is the same as test case 208, except "   //d26725
             "that the removal of the message is committed.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 225;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_225,                    //d26725
             "For compressed messages on common queues, test "                  //d26725
             "MSG_UNDER_CURSOR and verify the correct return code when "        //d26725
             "using BROWSE_MSG_UNDER_CURSOR after the message under the "       //d26725
             "cursor is removed.  This is the same as test case 208, except "   //d26725
             "that the removal of the message is committed.")                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 225;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 226
TPF_TESTCASE(mq_browse_msg_under_cursor_memory_uncompressed_226,                //d26725
             "An uncompressed message is browsed from the front message "       //d26725
             "list, the message is swept, then BROWSE_MSG_UNDER_CURSOR is "     //d26725
             "tested")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 226;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_browse_msg_under_cursor_memory_compressed_226,                  //d26725
             "An compressed message is browsed from the front message list, "   //d26725
             "the message is swept, then BROWSE_MSG_UNDER_CURSOR is tested")    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 226;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 227
TPF_TESTCASE(mq_browse_msg_under_cursor_memory_uncompressed_227,                //d26725
             "A swept uncompressed message is browsed, the message is unswept, "//d26725
             "then BROWSE_MSG_UNDER_CURSOR is tested.  The queue is then "      //d26725
             "re-swept and the message is browsed again, but this time it's "   //d26725
             "locked.  The queue is unswept again and the messaged is browsed " //d26725
             "once more and unlocked.  Another handle then gets the previously "//d26725
             "locked message.  The queue should now be empty.  We then try to " //d26725
             "browse and read it")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(200);                                                      //d26725
      testcase = 227;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_browse_msg_under_cursor_memory_compressed_227,                  //d26725
             "A swept compressed message is browsed, the message is unswept, "  //d26725
             "then BROWSE_MSG_UNDER_CURSOR is tested.  The queue is then "      //d26725
             "re-swept and the message is browsed again, but this time it's "   //d26725
             "locked.  The queue is unswept again and the messaged is browsed " //d26725
             "once more and unlocked.  Another handle then gets the previously "//d26725
             "locked message.  The queue should now be empty.  We then try to " //d26725
             "browse and read it")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(200);                                                      //d26725
      testcase = 227;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 228
TPF_TESTCASE(mq_locks_memory_uncompressed_228,                                  //d26725
             "An unswept uncompressed message is locked and the lock is "       //d26725
             "tested.  Then the message is swept and the lock is tested again. "//d26725
             "Finally, the message is unswept and the lock is tested one more " //d26725
             "time.  Then BROWSE_MSG_UNDER_CURSOR is tested. The queue is then "//d26725
             "re-swept and the message is browsed again, but this time it's "   //d26725
             "locked.  The queue is unswept again and the messaged is browsed " //d26725
             "once more and unlocked.  Another handle then gets the previously "//d26725
             "locked message.  The queue should now be empty.  We then try to " //d26725
             "browse and read it")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(200);                                                      //d26725
      testcase = 228;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_memory_compressed_228,                                    //d26725
             "An unswept compressed message is locked and the lock is "         //d26725
             "tested.  Then the message is swept and the lock is tested again. "//d26725
             "Finally, the message is unswept and the lock is tested one more " //d26725
             "time.  Then BROWSE_MSG_UNDER_CURSOR is tested. The queue is then "//d26725
             "re-swept and the message is browsed again, but this time it's "   //d26725
             "locked.  The queue is unswept again and the messaged is browsed " //d26725
             "once more and unlocked.  Another handle then gets the previously "//d26725
             "locked message.  The queue should now be empty.  We then try to " //d26725
             "browse and read it")                                              //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(200);                                                      //d26725
      testcase = 228;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 229
TPF_TESTCASE(mq_locks_memory_uncompressed_229,                                  //d26725
             "Verify that a handle that locks an uncompressed message can get " //d26725
             "it from a memory queue with a non-browse MQGET")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 229;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_common_uncompressed_229,                                  //d26725
             "Verify that a handle that locks an uncompressed message can get " //d26725
             "it from a common queue with a non-browse MQGET")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 229;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_memory_compressed_229,                                    //d26725
             "Verify that a handle that locks a compressed message can get "    //d26725
             "it from a memory queue with a non-browse MQGET")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 229;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_common_compressed_229,                                    //d26725
             "Verify that a handle that locks a compressed message can get "    //d26725
             "it from a common queue with a non-browse MQGET")                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 229;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 230
TPF_TESTCASE(mq_locks_memory_uncompressed_230,                                  //d26725
             "Lock an uncompressed message and hold it on a memory queue until "//d26725
             "it ages.  This will cause it to be validated the next time it's " //d26725
             "encountered")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 230;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_common_uncompressed_230,                                  //d26725
             "Lock an uncompressed message and hold it on a common queue until "//d26725
             "it ages.  This will cause it to be validated the next time it's " //d26725
             "encountered")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 230;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_memory_compressed_230,                                    //d26725
             "Lock a compressed message and hold it on a memory queue until "   //d26725
             "it ages.  This will cause it to be validated the next time it's " //d26725
             "encountered")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 230;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_common_compressed_230,                                    //d26725
             "Lock a compressed message and hold it on a common queue until "   //d26725
             "it ages.  This will cause it to be validated the next time it's " //d26725
             "encountered")                                                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 230;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 231
TPF_TESTCASE(mq_locks_memory_uncompressed_231,                                  //d26725
             "Lock an uncompressed message and hold it on a memory queue until "//d26725
             "it ages.  This will cause it to be validated by a non-browse "    //d26725
             "MQGET the next time it's encountered")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 231;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_common_uncompressed_231,                                  //d26725
             "Lock an uncompressed message and hold it on a common queue until "//d26725
             "it ages.  This will cause it to be validated by a non-browse "    //d26725
             "MQGET the next time it's encountered")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 231;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_memory_compressed_231,                                    //d26725
             "Lock a compressed message and hold it on a memory queue until "   //d26725
             "it ages.  This will cause it to be validated by a non-browse "    //d26725
             "MQGET the next time it's encountered")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 231;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_common_compressed_231,                                    //d26725
             "Lock a compressed message and hold it on a common queue until "   //d26725
             "it ages.  This will cause it to be validated by a non-browse "    //d26725
             "MQGET the next time it's encountered")                            //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 231;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 232
TPF_TESTCASE(mq_overflow_SWB_memory_uncompressed_232,                           //d26725
             "Browse a swept uncompressed message that uses an overflow SWB")   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 232;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_overflow_SWB_memory_compressed_232,                             //d26725
             "Browse a swept compressed message that uses an overflow SWB")     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 232;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 233
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_233,                  //d26725
             "Put two uncompressed messages with different MsgId's on a "       //d26725
             "memory queue and verify that the correct ones are read when "     //d26725
             "specifying the MsgId in the MQMD, first with MQGMO_BROWSE_FIRST " //d26725
             "and then with MQGMO_NONE")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 233;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_233,                  //d26725
             "Put two uncompressed messages with different MsgId's on a "       //d26725
             "common queue and verify that the correct ones are read when "     //d26725
             "specifying the MsgId in the MQMD, first with MQGMO_BROWSE_FIRST " //d26725
             "and then with MQGMO_NONE")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 233;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_233,                    //d26725
             "Put two compressed messages with different MsgId's on a "         //d26725
             "memory queue and verify that the correct ones are read when "     //d26725
             "specifying the MsgId in the MQMD, first with MQGMO_BROWSE_FIRST " //d26725
             "and then with MQGMO_NONE")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 233;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_233,                    //d26725
             "Put two compressed messages with different MsgId's on a "         //d26725
             "common queue and verify that the correct ones are read when "     //d26725
             "specifying the MsgId in the MQMD, first with MQGMO_BROWSE_FIRST " //d26725
             "and then with MQGMO_NONE")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 233;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 234
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_234,                  //d26725
             "Put two swept uncompressed messages with different MsgId's on a " //d26725
             "memory queue and verify that the correct ones are read when "     //d26725
             "specifying the MsgId in the MQMD, first with MQGMO_BROWSE_FIRST " //d26725
             "and then with MQGMO_NONE")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 234;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_234,                    //d26725
             "Put two swept compressed messages with different MsgId's on a "   //d26725
             "memory queue and verify that the correct ones are read when "     //d26725
             "specifying the MsgId in the MQMD, first with MQGMO_BROWSE_FIRST " //d26725
             "and then with MQGMO_NONE")                                        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 234;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725                                                            //d26725
//test case 235
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_235,                  //d26725
             "Verify that a BROWSE_NEXT will find the second matching "         //d26725
             "uncompressed message on a memory queue if the first one has been "//d26725
             "passed")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 235;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_235,                  //d26725
             "Verify that a BROWSE_NEXT will find the second matching "         //d26725
             "uncompressed message on a common queue if the first one has been "//d26725
             "passed")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 235;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_235,                    //d26725
             "Verify that a BROWSE_NEXT will find the second matching "         //d26725
             "compressed message on a memory queue if the first one has been "  //d26725
             "passed")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 235;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
TPF_TESTCASE(mq_put_get_browse_option_common_compressed_235,                    //d26725
             "Verify that a BROWSE_NEXT will find the second matching "         //d26725
             "compressed message on a common queue if the first one has been "  //d26725
             "passed")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 235;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 236
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_236,                  //d26725
             "Verify that a BROWSE_NEXT will find the second matching "         //d26725
             "swept uncompressed message if the first one has been passed")     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 236;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_236,                    //d26725
             "Verify that a BROWSE_NEXT will find the second matching "         //d26725
             "swept compressed message if the first one has been passed")       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 236;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 237
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_237,                  //d26725
             "Put uncompressed messages with different MsgId/CorrelId "         //d26725
             "combinations on a memory queue and verify that the correct "      //d26725
             "ones are read when specifying the MsgId and CorrelId in the "     //d26725
             "MQMD, first with MQGMO_BROWSE_FIRST and then with MQGMO_NONE")    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 237;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_237,                  //d26725
             "Put uncompressed messages with different MsgId/CorrelId "         //d26725
             "combinations on a common queue and verify that the correct "      //d26725
             "ones are read when specifying the MsgId and CorrelId in the "     //d26725
             "MQMD, first with MQGMO_BROWSE_FIRST and then with MQGMO_NONE")    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 237;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_237,                    //d26725
             "Put compressed messages with different MsgId/CorrelId "           //d26725
             "combinations on a memory queue and verify that the correct "      //d26725
             "ones are read when specifying the MsgId and CorrelId in the "     //d26725
             "MQMD, first with MQGMO_BROWSE_FIRST and then with MQGMO_NONE")    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 237;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_237,                    //d26725
             "Put compressed messages with different MsgId/CorrelId "           //d26725
             "combinations on a common queue and verify that the correct "      //d26725
             "ones are read when specifying the MsgId and CorrelId in the "     //d26725
             "MQMD, first with MQGMO_BROWSE_FIRST and then with MQGMO_NONE")    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 237;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 238
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_238,                  //d26725
             "Verify that BROWSE_MSG_UNDER_CURSOR and MSG_UNDER_CURSOR don't "  //d26725
             "require successful match of MsgId for uncompressed messages on "  //d26725
             "memory queues")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 238;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_uncompressed_238,                  //d26725
             "Verify that BROWSE_MSG_UNDER_CURSOR and MSG_UNDER_CURSOR don't "  //d26725
             "require successful match of MsgId for uncompressed messages on "  //d26725
             "common queues")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 238;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_238,                    //d26725
            "Verify that BROWSE_MSG_UNDER_CURSOR and MSG_UNDER_CURSOR don't "   //d26725
            "require successful match of MsgId for compressed messages on "     //d26725
            "memory queues")                                                    //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 238;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_common_compressed_238,                    //d26725
             "Verify that BROWSE_MSG_UNDER_CURSOR and MSG_UNDER_CURSOR don't "  //d26725
             "require successful match of MsgId for compressed messages on "    //d26725
             "common queues")                                                   //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 238;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 239
TPF_TESTCASE(mq_put_get_browse_option_memory_uncompressed_239,                  //d26725
             "Verify that BROWSE_MSG_UNDER_CURSOR and MSG_UNDER_CURSOR don't "  //d26725
             "require successful match of MsgId for swept uncompressed "        //d26725
             "messages on memory queues")                                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 239;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_put_get_browse_option_memory_compressed_239,                    //d26725
            "Verify that BROWSE_MSG_UNDER_CURSOR and MSG_UNDER_CURSOR don't "   //d26725
            "require successful match of MsgId for swept compressed messages "  //d26725
            "on memory queues")                                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 239;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

//test case 240
TPF_TESTCASE(mq_generate_new_id_memory_uncompressed_240,                        //d26725
             "Verify that MQPMO_NEW_MSG_ID generates a new MsgId in MQMD for "  //d26725
             "uncompressed messages on a memory queue")                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 240;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_common_uncompressed_240,                        //d26725
             "Verify that MQPMO_NEW_MSG_ID generates a new MsgId in MQMD for "  //d26725
             "uncompressed messages on a common queue")                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 240;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_memory_compressed_240,                          //d26725
             "Verify that MQPMO_NEW_MSG_ID generates a new MsgId in MQMD for "  //d26725
             "compressed messages on a memory queue")                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 240;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_common_compressed_240,                          //d26725
             "Verify that MQPMO_NEW_MSG_ID generates a new MsgId in MQMD for "  //d26725
             "compressed messages on a common queue")                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 240;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 241
TPF_TESTCASE(mq_generate_new_id_memory_uncompressed_241,                        //d26725
             "Verify that MQPMO_NEW_MSG_ID generates a new CorrelId in MQMD "   //d26725
             "for uncompressed messages on a memory queue")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 241;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_common_uncompressed_241,                        //d26725
             "Verify that MQPMO_NEW_MSG_ID generates a new CorrelId in MQMD "   //d26725
             "for uncompressed messages on a common queue")                     //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 241;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_memory_compressed_241,                          //d26725
             "Verify that MQPMO_NEW_MSG_ID generates a new CorrelId in MQMD "   //d26725
             "for compressed messages on a memory queue")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 241;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_common_compressed_241,                          //d26725
             "Verify that MQPMO_NEW_MSG_ID generates a new CorrelId in MQMD "   //d26725
             "for compressed messages on a common queue")                       //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 241;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 242
TPF_TESTCASE(mq_generate_new_id_memory_uncompressed_242,                        //d26725
             "Verify that MQPMO_NEW_MSG_ID and MQPMO_NEW_CORREL_ID generates "  //d26725
             "both a new MsgId and CorrelId in MQMD for uncompressed messages " //d26725
             "on a memory queue")                                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 242;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_common_uncompressed_242,                        //d26725
             "Verify that MQPMO_NEW_MSG_ID and MQPMO_NEW_CORREL_ID generates "  //d26725
             "both a new MsgId and CorrelId in MQMD for uncompressed messages " //d26725
             "on a common queue")                                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 242;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_memory_compressed_242,                          //d26725
             "Verify that MQPMO_NEW_MSG_ID and MQPMO_NEW_CORREL_ID generates "  //d26725
             "both a new MsgId and CorrelId in MQMD for compressed messages on "//d26725
             "a memory queue")                                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 242;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_common_compressed_242,                          //d26725
             "Verify that MQPMO_NEW_MSG_ID and MQPMO_NEW_CORREL_ID generates "  //d26725
             "both a new MsgId and CorrelId in MQMD for compressed messages on "//d26725
             "a common queue")                                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 242;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 243
TPF_TESTCASE(mq_generate_new_id_memory_uncompressed_243,                        //d26725
             "Verify that uncompressed messages on the Rear Message List can "  //d26725
             "be searched by MsgID and CorrelID")                               //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 243;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_generate_new_id_memory_compressed_243,                          //d26725
             "Verify that compressed messages on the Rear Message List can be " //d26725
             "searched by MsgID and CorrelID")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 243;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 244
TPF_TESTCASE(mq_rollback_memory_uncompressed_244,                               //d26725
             "Test handling of rolled back uncompressed message. A swept "      //d26725
             "message (not the first swept message) is gotten and rolled "      //d26725
             "back.  This puts it at the end of the Front Message List. "       //d26725
             "The message is then browsed and gotten")                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 244;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_rollback_memory_compressed_244,                                 //d26725
             "Test handling of rolled back compressed message. A swept "        //d26725
             "message (not the first swept message) is gotten and rolled "      //d26725
             "back.  This puts it at the end of the Front Message List. "       //d26725
             "The message is then browsed and gotten")                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 244;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 245
TPF_TESTCASE(mq_sweeper_memory_uncompressed_245,                                //d26725
             "Allow a queue to be fully swept, then get the first uncompressed "//d26725
             "message.  The queue should be empty before starting this test "   //d26725
             "case,  otherwise the get rate will prevent the queue from being " //d26725
             "swept")                                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 245;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_sweeper_memory_compressed_245,                                  //d26725
             "Allow a queue to be fully swept, then get the first compressed "  //d26725
             "message.  The queue should be empty before starting this test "   //d26725
             "case,  otherwise the get rate will prevent the queue from being " //d26725
             "swept")                                                           //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 245;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 246
TPF_TESTCASE(mq_locks_memory_uncompressed_246,                                  //d26725
             "Hold a lock for more than 60 seconds and verify that the "        //d26725
             "uncompressed message is still locked on the memory queue")        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 246;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_common_uncompressed_246,                                  //d26725
             "Hold a lock for more than 60 seconds and verify that the "        //d26725
             "uncompressed message is still locked on the common queue")        //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 246;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_memory_compressed_246,                                    //d26725
             "Hold a lock for more than 60 seconds and verify that the "        //d26725
             "compressed message is still locked on the memory queue")          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 246;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_locks_common_compressed_246,                                    //d26725
             "Hold a lock for more than 60 seconds and verify that the "        //d26725
             "compressed message is still locked on the common queue")          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 246;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 247
TPF_TESTCASE(mq_expiry_memory_uncompressed_247,                                 //d26725
             "Test expiry values for valid and invalid codes for uncompressed " //d26725
             "messages on a memory queue")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 247;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_common_uncompressed_247,                                 //d26725
             "Test expiry values for valid and invalid codes for uncompressed " //d26725
             "messages on a common queue")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 247;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_memory_compressed_247,                                   //d26725
             "Test expiry values for valid and invalid codes for compressed "   //d26725
             "messages on a memory queue")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 247;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_common_compressed_247,                                   //d26725
             "Test expiry values for valid and invalid codes for compressed "   //d26725
             "messages on a common queue")                                      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 247;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 248
TPF_TESTCASE(mq_expiry_memory_uncompressed_248,                                 //d26725
             "Put an expired uncompressed message onto a memory queue, then "   //d26725
             "attempt a browse. Verify the message expires.  Put another "      //d26725
             "message on the queue, verify it hasn't expired and try to "       //d26725
             "get it")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 248;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_common_uncompressed_248,                                 //d26725
             "Put an expired uncompressed message onto a common queue, then "   //d26725
             "attempt a browse. Verify the message expires.  Put another "      //d26725
             "message on the queue, verify it hasn't expired and try to "       //d26725
             "get it")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 248;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_memory_compressed_248,                                   //d26725
             "Put an expired compressed message onto a memory queue, then "     //d26725
             "attempt a browse. Verify the message expires.  Put another "      //d26725
             "message on the queue, verify it hasn't expired and try to "       //d26725
             "get it")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 248;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_common_compressed_248,                                   //d26725
             "Put an expired compressed message onto a common queue, then "     //d26725
             "attempt a browse. Verify the message expires.  Put another "      //d26725
             "message on the queue, verify it hasn't expired and try to "       //d26725
             "get it")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 248;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 249
TPF_TESTCASE(mq_expiry_memory_uncompressed_249,                                 //d26725
             "Put an expired uncompressed message onto a memory queue, "        //d26725
             "then attempt a browse that fails. Verify the message expires. "   //d26725
             "Put another message on the queue, verify it hasn't expired, "     //d26725
             "then issue an MQGET that fails")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 249;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_common_uncompressed_249,                                 //d26725
             "Put an expired uncompressed message onto a common queue, "        //d26725
             "then attempt a browse that fails. Verify the message expires. "   //d26725
             "Put another message on the queue, verify it hasn't expired, "     //d26725
             "then issue an MQGET that fails")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 249;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_memory_compressed_249,                                   //d26725
             "Put an expired compressed message onto a memory queue, "          //d26725
             "then attempt a browse that fails. Verify the message expires. "   //d26725
             "Put another message on the queue, verify it hasn't expired, "     //d26725
             "then issue an MQGET that fails")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 249;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_common_compressed_249,                                   //d26725
             "Put an expired compressed message onto a common queue, "          //d26725
             "then attempt a browse that fails. Verify the message expires. "   //d26725
             "Put another message on the queue, verify it hasn't expired, "     //d26725
             "then issue an MQGET that fails")                                  //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 249;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 250
TPF_TESTCASE(mq_expiry_memory_uncompressed_250,                                 //d26725
             "Put several uncompressed messages onto a memory queue with "      //d26725
             "varying expiry times.  Browse the messages by MessageID "         //d26725
             "to ensure they are there.  Sleep, then verify the ones "          //d26725
             "we expected to expire have done so, but verify the others "       //d26725
             "are still there")                                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 250;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_common_uncompressed_250,                                 //d26725
             "Put several uncompressed messages onto a common queue with "      //d26725
             "varying expiry times.  Browse the messages by MessageID "         //d26725
             "to ensure they are there.  Sleep, then verify the ones "          //d26725
             "we expected to expire have done so, but verify the others "       //d26725
             "are still there")                                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 250;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_memory_compressed_250,                                   //d26725
             "Put several compressed messages onto a memory queue with "        //d26725
             "varying expiry times.  Browse the messages by MessageID "         //d26725
             "to ensure they are there.  Sleep, then verify the ones "          //d26725
             "we expected to expire have done so, but verify the others "       //d26725
             "are still there")                                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 250;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_common_compressed_250,                                   //d26725
             "Put several compressed messages onto a common queue with "        //d26725
             "varying expiry times.  Browse the messages by MessageID "         //d26725
             "to ensure they are there.  Sleep, then verify the ones "          //d26725
             "we expected to expire have done so, but verify the others "       //d26725
             "are still there")                                                 //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 250;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 251
TPF_TESTCASE(mq_expiry_memory_uncompressed_251,                                 //d26725
             "Verify that uncompressed messages with a portion of a second of " //d26725
             "an EXPIRY value remaining are able to be gotten from memory "     //d26725
             "queues")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 251;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_common_uncompressed_251,                                 //d26725
             "Verify that uncompressed messages with a portion of a second of " //d26725
             "an EXPIRY value remaining are able to be gotten from common "     //d26725
             "queues")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 251;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_expiry_memory_compressed_251,                                   //d26725
             "Verify that compressed messages with a portion of a second of "   //d26725
             "an EXPIRY value remaining are able to be gotten from memory "     //d26725
             "queues")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 251;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
TPF_TESTCASE(mq_expiry_common_compressed_251,                                   //d26725
             "Verify that compressed messages with a portion of a second of "   //d26725
             "an EXPIRY value remaining are able to be gotten from common "     //d26725
             "queues")                                                          //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      TPF_TC_TIMEOUT(120);                                                      //d26725
      testcase = 251;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 252
TPF_TESTCASE(mq_validate_mem_memory_uncompressed_252,                           //d26725
             "Call MQCONN, then put an uncompressed message onto a memory "     //d26725
             "queue, get the message, then issue an MQCLOSE. Loop through this "//d26725
             "process 100 times, then verify the memory has not changed.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 252;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
TPF_TESTCASE_MANUAL(mq_validate_mem_common_uncompressed_252,                    //d26725
             "Call MQCONN, then put an uncompressed message onto a common "     //d26725
             "queue, get the message, then issue an MQCLOSE. Loop through this "//d26725
             "process 100 times, then verify the memory has not changed.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 252;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_validate_mem_memory_compressed_252,                             //d26725
             "Call MQCONN, then put a compressed message onto a memory "        //d26725
             "queue, get the message, then issue an MQCLOSE. Loop through this "//d26725
             "process 100 times, then verify the memory has not changed.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 252;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
TPF_TESTCASE_MANUAL(mq_validate_mem_common_compressed_252,                      //d26725
             "Call MQCONN, then put a compressed message onto a common "        //d26725
             "queue, get the message, then issue an MQCLOSE. Loop through this "//d26725
             "process 100 times, then verify the memory has not changed.")      //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 252;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
//test case 253
TPF_TESTCASE(mq_validate_mem_after_disc_memory_uncompressed_253,                //d26725
             "Verify memory in use before an MQCONN is equal to the memory "    //d26725
             "in use after an MQDISC for uncompressed messages on memory "      //d26725
             "queues.")                                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 253;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
TPF_TESTCASE_MANUAL(mq_validate_mem_after_disc_common_uncompressed_253,         //d26725
             "Verify memory in use before an MQCONN is equal to the memory "    //d26725
             "in use after an MQDISC for uncompressed messages on common "      //d26725
             "queues.")                                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = FALSE;                                                             //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 253;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725

TPF_TESTCASE(mq_validate_mem_after_disc_memory_compressed_253,                  //d26725
             "Verify memory in use before an MQCONN is equal to the memory "    //d26725
             "in use after an MQDISC for compressed messages on memory "        //d26725
             "queues.")                                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = FALSE;                                                               //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 253;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
TPF_TESTCASE_MANUAL(mq_validate_mem_after_disc_common_compressed_253,           //d26725
             "Verify memory in use before an MQCONN is equal to the memory "    //d26725
             "in use after an MQDISC for compressed messages on common "        //d26725
             "queues.")                                                         //d26725
{                                                                               //d26725
  MQLONG  Reason;                                                               //d26725
  MQLONG  CompCode;                                                             //d26725
  boolean rc = TRUE;                                                            //d26725
  common = TRUE;                                                                //d26725
  compsize = TRUE;                                                              //d26725

  //verify test case is valid
  rc = IsTestCaseValid();                                                       //d26725

  if (rc) {                                                                     //d26725
    //set up queues and run test case
    if (SetUpQueues() == MQ2RC_OK) {                                            //d26725
      testcase = 253;                                                           //d26725
      RunCase(PRT_INFO);                                                        //d26725
    }                                                                           //d26725

    //Disconnect QMGR
    MQDISC(&Hconn, &CompCode, &Reason);                                         //d26725
    CheckError("MQDISC", Reason);                                               //d26725
  }                                                                             //d26725
  else {                                                                        //d26725
    TPF_TC_IGNORE("Compression is not supported by the processor.");            //d26725
  }                                                                             //d26725
}                                                                               //d26725
